@@ -1,25 +1,19 @@
 /*  Copyright (C) 2006 yopyop
-    yopyop156@ifrance.com
-    yopyop156.ifrance.com
-
     Copyright 2007 shash
-	Copyright 2007-2009 DeSmuME team
+	Copyright 2007-2010 DeSmuME team
 
-    This file is part of DeSmuME
+	This file is free software: you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation, either version 2 of the License, or
+	(at your option) any later version.
 
-    DeSmuME is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+	This file is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
 
-    DeSmuME is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with DeSmuME; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
+	You should have received a copy of the GNU General Public License
+	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "FIFO.h"
@@ -75,7 +69,7 @@ void IPC_FIFOsend(u8 proc, u32 val)
 	T1WriteWord(MMU.MMU_MEM[proc][0x40], 0x184, cnt_l);
 	T1WriteWord(MMU.MMU_MEM[proc_remote][0x40], 0x184, cnt_r);
 
-	setIF(proc_remote, ((cnt_l & 0x0400)<<8));		// IRQ18: recv not empty
+	NDS_Reschedule();
 }
 
 u32 IPC_FIFOrecv(u8 proc)
@@ -109,14 +103,14 @@ u32 IPC_FIFOrecv(u8 proc)
 
 	if ( ipc_fifo[proc_remote].size == 0 )		// FIFO empty
 	{
-		cnt_l |= 0x0100;
+		cnt_l |= 0x0101;
 		cnt_r |= 0x0001;
 	}
 
 	T1WriteWord(MMU.MMU_MEM[proc][0x40], 0x184, cnt_l);
 	T1WriteWord(MMU.MMU_MEM[proc_remote][0x40], 0x184, cnt_r);
 
-	setIF(proc_remote, ((cnt_l & 0x0004)<<15));		// IRQ17: send empty
+	NDS_Reschedule();
 
 	return (val);
 }
@@ -126,28 +120,35 @@ void IPC_FIFOcnt(u8 proc, u16 val)
 	u16 cnt_l = T1ReadWord(MMU.MMU_MEM[proc][0x40], 0x184);
 	u16 cnt_r = T1ReadWord(MMU.MMU_MEM[proc^1][0x40], 0x184);
 
-	//LOG("IPC%s FIFO context 0x%X (local 0x%04X, remote 0x%04X)\n", proc?"7":"9", val, cnt_l, cnt_r);
-	if (val & 0x4008)
+	if (val & IPCFIFOCNT_FIFOERROR)
 	{
-		ipc_fifo[proc].head = 0; ipc_fifo[proc].tail = 0; ipc_fifo[proc].size = 0;
-		T1WriteWord(MMU.MMU_MEM[proc][0x40], 0x184, (cnt_l & 0x0301) | (val & 0x8404) | 1);
-		T1WriteWord(MMU.MMU_MEM[proc^1][0x40], 0x184, (cnt_r & 0x8407) | 0x100);
-		//MMU.reg_IF[proc^1] |= ((val & 0x0004) << 15);
-		setIF(proc^1, ((val & 0x0004)<<15));
-		return;
+		//at least SPP uses this, maybe every retail game
+		cnt_l &= ~IPCFIFOCNT_FIFOERROR;
 	}
 
-	T1WriteWord(MMU.MMU_MEM[proc][0x40], 0x184, val);
+	if (val & IPCFIFOCNT_SENDCLEAR)
+	{
+		ipc_fifo[proc].head = 0; ipc_fifo[proc].tail = 0; ipc_fifo[proc].size = 0;
+
+		cnt_l |= IPCFIFOCNT_SENDEMPTY;
+		cnt_l &= ~IPCFIFOCNT_SENDFULL;
+		cnt_r |= IPCFIFOCNT_RECVEMPTY;
+		cnt_r &= ~IPCFIFOCNT_RECVFULL;
+	}
+
+	cnt_l &= ~IPCFIFOCNT_WRITEABLE;
+	cnt_l |= val & IPCFIFOCNT_WRITEABLE;
+
+	T1WriteWord(MMU.MMU_MEM[proc][0x40], 0x184, cnt_l);
+	T1WriteWord(MMU.MMU_MEM[proc^1][0x40], 0x184, cnt_r);
+
+	NDS_Reschedule();
 }
 
 // ========================================================= GFX FIFO
 GFX_PIPE	gxPIPE;
 GFX_FIFO	gxFIFO;
 
-
-
-
-
 void GFX_PIPEclear()
 {
 	gxPIPE.head = 0;
@@ -164,25 +165,16 @@ void GFX_FIFOclear()
 
 static void GXF_FIFO_handleEvents()
 {
-	if(gxFIFO.size <= 127)
-	{
-		//TODO - should this always happen, over and over, until the dma is disabled?
-		//or only when we change to this state?
-		if(MMU_new.gxstat.gxfifo_irq == 1) 
-			setIF(0, (1<<21)); //the half gxfifo irq
+	bool low = gxFIFO.size <= 127;
+	bool lowchange = MMU_new.gxstat.fifo_low ^ low;
+	MMU_new.gxstat.fifo_low = low;
+	if(low) triggerDma(EDMAMode_GXFifo);
 
-		//might need to trigger a gxfifo dma
-		triggerDma(EDMAMode_GXFifo);
-	}
-
-
-
-	if(gxFIFO.size == 0) {
-		//we just went to empty
-		if(MMU_new.gxstat.gxfifo_irq == 2) 
-			setIF(0, (1<<21)); //the empty gxfifo irq
-	}
+	bool empty = gxFIFO.size == 0;
+	bool emptychange = MMU_new.gxstat.fifo_empty ^ empty;
+	MMU_new.gxstat.fifo_empty = empty;
 
+	if(emptychange||lowchange) NDS_Reschedule();
 }
 
 void GFX_FIFOsend(u8 cmd, u32 param)
@@ -249,30 +241,23 @@ BOOL GFX_PIPErecv(u8 *cmd, u32 *param)
 
 void GFX_FIFOcnt(u32 val)
 {
-	//zeromus: i dont like any of this.
-
 	////INFO("gxFIFO: write cnt 0x%08X (prev 0x%08X) FIFO size %03i PIPE size %03i\n", val, gxstat, gxFIFO.size, gxPIPE.size);
 
-	//if (val & (1<<29))		// clear? (only in homebrew?)
-	//{
-	//	GFX_PIPEclear();
-	//	GFX_FIFOclear();
-	//	return;
-	//}
+	if (val & (1<<29))		// clear? (only in homebrew?)
+	{
+		GFX_PIPEclear();
+		GFX_FIFOclear();
+		return;
+	}
 
+	//zeromus says: what happened to clear stack?
 	//if (val & (1<<15))		// projection stack pointer reset
 	//{
 	//	gfx3d_ClearStack();
 	//	val &= 0xFFFF5FFF;		// clear reset (bit15) & stack level (bit13)
 	//}
 
-	//T1WriteLong(MMU.MMU_MEM[ARMCPU_ARM9][0x40], 0x600, val);
-
-	//if (gxFIFO.size == 0)		// empty
-	//{
-	//	if (val & 0x80000000)	// IRQ: empty
-	//		setIF(0, (1<<21));
-	//}
+	T1WriteLong(MMU.MMU_MEM[ARMCPU_ARM9][0x40], 0x600, val);
 }
 
 // ========================================================= DISP FIFO
