@@ -1,24 +1,20 @@
-/*  GPU.cpp
-	Copyright (C) 2006 yopyop
+/*  Copyright (C) 2006 yopyop
     Copyright (C) 2006-2007 Theo Berkau
     Copyright (C) 2007 shash
 	Copyright (C) 2008-2010 DeSmuME team
 
-    This file is part of DeSmuME
+	This file is free software: you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation, either version 2 of the License, or
+	(at your option) any later version.
 
-    DeSmuME is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+	This file is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
 
-    DeSmuME is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with DeSmuME; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+	You should have received a copy of the GNU General Public License
+	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <algorithm>
@@ -200,6 +196,12 @@ void GPU_Reset(GPU *g, u8 l)
 {
 	memset(g, 0, sizeof(GPU));
 
+	//important for emulator stability for this to initialize, since we have to setup a table based on it
+	g->BLDALPHA_EVA = 0;
+	g->BLDALPHA_EVB = 0;
+	//make sure we have our blend table setup even if the game blends without setting the blend variables
+	g->updateBLDALPHA();
+
 	g->setFinalColorBck_funcNum = 0;
 	g->setFinalColor3d_funcNum = 0;
 	g->setFinalColorSpr_funcNum = 0;
@@ -673,10 +675,6 @@ static FORCEINLINE void _master_setFinalOBJColor(GPU *gpu, u8 *dst, u16 color, u
 			return;
 	}
 
-	//this inspects the layer beneath the sprite to see if the current blend flags make it a candidate for blending
-	const int bg_under = gpu->bgPixels[x];
-	const bool allowBlend = (bg_under != 4) && gpu->blend2[bg_under];
-
 	const bool sourceEffectSelected = gpu->blend1;
 
 	//note that the fadein and fadeout is done here before blending, 
@@ -685,14 +683,20 @@ static FORCEINLINE void _master_setFinalOBJColor(GPU *gpu, u8 *dst, u16 color, u
 	if(windowEffect && sourceEffectSelected)
 		switch(FUNC) 
 		{
-		case Increase: if(!allowBlend) color = gpu->currentFadeInColors[color&0x7FFF]; break;
-		case Decrease: if(!allowBlend) color = gpu->currentFadeOutColors[color&0x7FFF]; break;
+			//zero 13-jun-2010 : if(allowBlend) was removed from these;
+			//it should be possible to increase/decrease and also blend
+			//(the effect would be increase, but the obj properties permit blending and the target layers are configured correctly)
+		case Increase: color = gpu->currentFadeInColors[color&0x7FFF]; break;
+		case Decrease: color = gpu->currentFadeOutColors[color&0x7FFF]; break;
 
 		//only when blend color effect is selected, ordinarily opaque sprites are blended with the color effect params
 		case Blend: forceBlendingForNormal = true; break;
 		case None: break;
 		}
 
+	//this inspects the layer beneath the sprite to see if the current blend flags make it a candidate for blending
+	const int bg_under = gpu->bgPixels[x];
+	const bool allowBlend = (bg_under != 4) && gpu->blend2[bg_under];
 
 	if(allowBlend)
 	{
@@ -2164,15 +2168,15 @@ static void GPU_RenderLine_layer(NDS_Screen * screen, u16 l)
 template<bool SKIP> static void GPU_RenderLine_DispCapture(u16 l)
 {
 	//this macro takes advantage of the fact that there are only two possible values for capx
-	#define CAPCOPY(SRC,DST) \
+	#define CAPCOPY(SRC,DST,SETALPHABIT) \
 	switch(gpu->dispCapCnt.capx) { \
 		case DISPCAPCNT::_128: \
 			for (int i = 0; i < 128; i++)  \
-				HostWriteWord(DST, i << 1, HostReadWord(SRC, i << 1) | (1<<15)); \
+				HostWriteWord(DST, i << 1, HostReadWord(SRC, i << 1) | (SETALPHABIT?(1<<15):0)); \
 			break; \
 		case DISPCAPCNT::_256: \
 			for (int i = 0; i < 256; i++)  \
-				HostWriteWord(DST, i << 1, HostReadWord(SRC, i << 1) | (1<<15)); \
+				HostWriteWord(DST, i << 1, HostReadWord(SRC, i << 1) | (SETALPHABIT?(1<<15):0)); \
 			break; \
 			default: assert(false); \
 		}
@@ -2233,7 +2237,7 @@ template<bool SKIP> static void GPU_RenderLine_DispCapture(u16 l)
 
 									u8 *src;
 									src = (u8*)(gpu->tempScanline);
-									CAPCOPY(src,cap_dst);
+									CAPCOPY(src,cap_dst,true);
 								}
 							break;
 							case 1:			// Capture 3D
@@ -2241,7 +2245,7 @@ template<bool SKIP> static void GPU_RenderLine_DispCapture(u16 l)
 									//INFO("Capture 3D\n");
 									u16* colorLine;
 									gfx3d_GetLineData15bpp(l, &colorLine);
-									CAPCOPY(((u8*)colorLine),cap_dst);
+									CAPCOPY(((u8*)colorLine),cap_dst,false);
 								}
 							break;
 						}
@@ -2254,7 +2258,7 @@ template<bool SKIP> static void GPU_RenderLine_DispCapture(u16 l)
 						{
 							case 0:	
 								//Capture VRAM
-								CAPCOPY(cap_src,cap_dst);
+								CAPCOPY(cap_src,cap_dst,true);
 								break;
 							case 1:
 								//capture dispfifo
@@ -2752,49 +2756,6 @@ template<bool MOSAIC> void GPU::modeRender(int layer)
 	}
 }
 
-void gpu_UpdateRender()
-{
-	/*int x = 0, y = 0;
-	u16 *src = (u16*)GPU_screen;
-	u16	*dst = (u16*)GPU_screen;
-
-	switch (gpu_angle)
-	{
-		case 0:
-			memcpy(dst, src, 256*192*4);
-			break;
-
-		case 90:
-			for(y = 0; y < 384; y++)
-			{
-				for(x = 0; x < 256; x++)
-				{
-					dst[(383 - y) + (x * 384)] = src[x + (y * 256)];
-				}
-			}
-			break;
-		case 180:
-			for(y = 0; y < 384; y++)
-			{
-				for(x = 0; x < 256; x++)
-				{
-					dst[(255 - x) + ((383 - y) * 256)] = src[x + (y * 256)];
-				}
-			}
-			break;
-		case 270:
-			for(y = 0; y < 384; y++)
-			{
-				for(x = 0; x < 256; x++)
-				{
-					dst[y + ((255 - x) * 384)] = src[x + (y * 256)];
-				}
-			}
-		default:
-			break;
-	}*/
-}
-
 void gpu_SetRotateScreen(u16 angle)
 {
 	gpu_angle = angle;
