@@ -1,26 +1,20 @@
 /*  Copyright (C) 2006 yopyop
-	yopyop156@ifrance.com
-	yopyop156.ifrance.com
-
 	Copyright (C) 2006-2007 Theo Berkau
 	Copyright (C) 2007 shash
-	Copyright (C) 2009-2009 DeSmuME team
+	Copyright (C) 2009-2010 DeSmuME team
 
-	This file is part of DeSmuME
-
-	DeSmuME is free software; you can redistribute it and/or modify
+	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
-	the Free Software Foundation; either version 2 of the License, or
+	the Free Software Foundation, either version 2 of the License, or
 	(at your option) any later version.
 
-	DeSmuME is distributed in the hope that it will be useful,
+	This file is distributed in the hope that it will be useful,
 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 	GNU General Public License for more details.
 
 	You should have received a copy of the GNU General Public License
-	along with DeSmuME; if not, write to the Free Software
-	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #ifndef GPU_H
@@ -992,8 +986,6 @@ void SetupFinalPixelBlitter (GPU *gpu);
 //#define GPU_setBGxHOFS(bg, gpu, val) gpu->dispx_st->dispx_BGxOFS[bg].BGxHOFS = ((val) & 0x1FF)
 //#define GPU_setBGxVOFS(bg, gpu, val) gpu->dispx_st->dispx_BGxOFS[bg].BGxVOFS = ((val) & 0x1FF)
 
-// render
-void gpu_UpdateRender();
 void gpu_SetRotateScreen(u16 angle);
 
 //#undef FORCEINLINE
