@@ -116,6 +116,8 @@ struct TGXSTAT : public TRegister_32
 {
 	TGXSTAT() {
 		gxfifo_irq = se = tr = tb = sb = 0;
+		fifo_empty = true;
+		fifo_low = false;
 	}
 	u8 tb; //test busy
 	u8 tr; //test result
@@ -123,6 +125,8 @@ struct TGXSTAT : public TRegister_32
 	u8 sb; //stack busy
 	u8 gxfifo_irq; //irq configuration
 
+	bool fifo_empty, fifo_low;
+
 	virtual u32 read32();
 	virtual void write32(const u32 val);
 
@@ -357,8 +361,6 @@ struct MMU_struct
 
 	//Card rom & ram
 	u8 * CART_ROM;
-	u32 CART_ROM_MASK;
-	u8 CART_RAM[0x10000];
 
 	//Unused ram
 	u8 UNUSED_RAM[4];
@@ -384,7 +386,13 @@ struct MMU_struct
 
 	u32 reg_IME[2];
 	u32 reg_IE[2];
-	u32 reg_IF[2];
+	
+	//these are the user-controlled IF bits. some IF bits are generated as necessary from hardware conditions
+	u32 reg_IF_bits[2];
+
+	u32 reg_DISP3DCNT_bits;
+
+	template<int PROCNUM> u32 gen_IF();
 
 	BOOL divRunning;
 	s64 divResult;
@@ -586,8 +594,11 @@ inline void SetupMMU(BOOL debugConsole) {
 }
 
 
-FORCEINLINE void CheckMemoryDebugEvent(const EDEBUG_EVENT event, const MMU_ACCESS_TYPE type, const u32 procnum, const u32 addr, const u32 size, const u32 val)
+FORCEINLINE void CheckMemoryDebugEvent(EDEBUG_EVENT event, const MMU_ACCESS_TYPE type, const u32 procnum, const u32 addr, const u32 size, const u32 val)
 {
+	//TODO - ugh work out a better prefetch event system
+	if(type == MMU_AT_CODE && event == DEBUG_EVENT_READ)
+		event = DEBUG_EVENT_EXECUTE;
 	if(CheckDebugEvent(event))
 	{
 		DebugEventData.memAccessType = type;
@@ -596,11 +607,6 @@ FORCEINLINE void CheckMemoryDebugEvent(const EDEBUG_EVENT event, const MMU_ACCES
 		DebugEventData.size = size;
 		DebugEventData.val = val;
 		HandleDebugEvent(event);
-
-		if(type == MMU_AT_CODE && event == DEBUG_EVENT_READ)
-		{
-			HandleDebugEvent(DEBUG_EVENT_EXECUTE);
-		}
 	}
 }
 
@@ -704,6 +710,12 @@ FORCEINLINE u32 _MMU_read32(const int PROCNUM, const MMU_ACCESS_TYPE AT, const u
 		if(addr<0x02000000) 
 			return T1ReadLong_guaranteedAligned(MMU.ARM9_ITCM, addr&0x7FFC);
 
+		//what happens when we execute from DTCM? nocash makes it look like we get 0xFFFFFFFF but i can't seem to verify it
+		//historically, desmume would fall through to its old memory map struct
+		//which would return unused memory (0)
+		//it seems the hardware returns 0 or something benign because in actuality 0xFFFFFFFF is an undefined opcode
+		//and we know our handling for that is solid
+
 		goto dunno;
 	}
 
