@@ -1,21 +1,18 @@
 /*	Copyright (C) 2006 yopyop
 	Copyright (C) 2008-2010 DeSmuME team
 
-    This file is part of DeSmuME
+	This file is free software: you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation, either version 2 of the License, or
+	(at your option) any later version.
 
-    DeSmuME is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+	This file is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
 
-    DeSmuME is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with DeSmuME; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
+	You should have received a copy of the GNU General Public License
+	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <string.h>
@@ -40,9 +37,20 @@
 #include "readwrite.h"
 #include "debug.h"
 #include "firmware.h"
+#include "version.h"
 
 #include "path.h"
 
+//#define LOG_ARM9
+//#define LOG_ARM7
+//bool dolog = false;
+//#define LOG_TO_FILE
+//#define LOG_TO_FILE_REGS
+
+//===============================================================
+FILE *fp_dis7 = NULL;
+FILE *fp_dis9 = NULL;
+
 PathInfo path;
 
 TCommonSettings CommonSettings;
@@ -69,6 +77,11 @@ int TotalLagFrames;
 
 TSCalInfo TSCal;
 
+namespace DLDI
+{
+	bool tryPatch(void* data, size_t size);
+}
+
 void Desmume_InitOnce()
 {
 	static bool initOnce = false;
@@ -94,6 +107,9 @@ int NDS_Init( void) {
 	MMU_Init();
 	nds.VCount = 0;
 
+	//got to print this somewhere..
+	printf("%s\n", EMU_DESMUME_NAME_AND_VERSION());
+
 	if (Screen_Init(GFXCORE_DUMMY) != 0)
 		return -1;
 
@@ -147,6 +163,21 @@ void NDS_DeInit(void) {
 	if (cheatSearch)
 		delete cheatSearch;
 
+#ifdef LOG_ARM7
+	if (fp_dis7 != NULL) 
+	{
+		fclose(fp_dis7);
+		fp_dis7 = NULL;
+	}
+#endif
+
+#ifdef LOG_ARM9
+	if (fp_dis9 != NULL) 
+	{
+		fclose(fp_dis9);
+		fp_dis9 = NULL;
+	}
+#endif
 }
 
 BOOL NDS_SetROM(u8 * rom, u32 mask)
@@ -169,6 +200,7 @@ NDS_header * NDS_getROMHeader(void)
 	header->cardSize = MMU.CART_ROM[20];
 	memcpy(header->cardInfo, MMU.CART_ROM + 21, 8);
 	header->flags = MMU.CART_ROM[29];
+	header->romversion = MMU.CART_ROM[30];
 	header->ARM9src = T1ReadLong(MMU.CART_ROM, 32);
 	header->ARM9exe = T1ReadLong(MMU.CART_ROM, 36);
 	header->ARM9cpy = T1ReadLong(MMU.CART_ROM, 40);
@@ -263,6 +295,39 @@ static u32 ones32(u32 x)
 }
 #endif
 
+RomBanner::RomBanner(bool defaultInit)
+{
+	if(!defaultInit) return;
+	version = 1; //Version  (0001h)
+	crc16 = 0; //CRC16 across entries 020h..83Fh
+	memset(reserved,0,sizeof(reserved));
+	memset(bitmap,0,sizeof(bitmap));
+	memset(palette,0,sizeof(palette));
+	memset(titles,0,sizeof(titles));
+	for(int i=0;i<NUM_TITLES;i++)
+		wcscpy(titles[i],L"None");
+	memset(end0xFF,0,sizeof(end0xFF));
+}
+
+bool GameInfo::hasRomBanner()
+{
+	if(header.IconOff + sizeof(RomBanner) > romsize)
+		return false;
+	else return true;
+}
+
+const RomBanner& GameInfo::getRomBanner()
+{
+	//we may not have a valid banner. return a default one
+	if(!hasRomBanner())
+	{
+		static RomBanner defaultBanner(true);
+		return defaultBanner;
+	}
+	
+	return *(RomBanner*)(romdata+header.IconOff);
+}
+
 void GameInfo::populate()
 {
 	const char *regions[] = {	"JPFSEDIRKH",
@@ -285,16 +350,17 @@ void GameInfo::populate()
 
 	memset(ROMserial, 0, sizeof(ROMserial));
 	memset(ROMname, 0, sizeof(ROMname));
-	memset(ROMfullName, 0, sizeof(ROMfullName));
 
 	if (
-		// ??? in all Homebrews game title have is 2E0000EA
-		//(
+		//Option 1. - look for this instruction in the game title
+		//(did this ever work?)
 		//(header->gameTile[0] == 0x2E) && 
 		//(header->gameTile[1] == 0x00) && 
 		//(header->gameTile[2] == 0x00) && 
 		//(header->gameTile[3] == 0xEA)
 		//) &&
+		//option 2. - look for gamecode #### (default for ndstool)
+		//or an invalid gamecode
 		(
 			((header.gameCode[0] == 0x23) && 
 			(header.gameCode[1] == 0x23) && 
@@ -307,6 +373,7 @@ void GameInfo::populate()
 			header.makerCode == 0x0
 		)
 	{
+		//we can't really make a serial for a homebrew game that hasnt set a game code
 		strcpy(ROMserial, "Homebrew");
 	}
 	else
@@ -319,18 +386,24 @@ void GameInfo::populate()
 			strcat(ROMserial, regions[region]);
 		else
 			strcat(ROMserial, "Unknown");
-		memset(ROMname, 0, sizeof(ROMname));
-		memcpy(ROMname, header.gameTile, 12);
-		trim(ROMname);
-
-		u8 num = (T1ReadByte((u8*)romdata, header.IconOff) == 1)?6:7;
-		for (int i = 0; i < num; i++)
-		{
-			wcstombs(ROMfullName[i], (wchar_t *)(romdata+header.IconOff+0x240+(i*0x100)), 0x100);
-			trim(ROMfullName[i]);
-		}
-
 	}
+
+	//rom name is probably set even in homebrew
+	memset(ROMname, 0, sizeof(ROMname));
+	memcpy(ROMname, header.gameTile, 12);
+	trim(ROMname,20);
+
+		/*if(header.IconOff < romsize)
+		{
+			u8 num = (T1ReadByte((u8*)romdata, header.IconOff) == 1)?6:7;
+			for (int i = 0; i < num; i++)
+			{
+				wcstombs(ROMfullName[i], (wchar_t *)(romdata+header.IconOff+0x240+(i*0x100)), 0x100);
+				trim(ROMfullName[i]);
+			}
+		}*/
+
+	
 }
 #ifdef _WINDOWS
 
@@ -355,8 +428,8 @@ static void loadrom(std::string fname) {
 
 int NDS_LoadROM(const char *filename, const char *logicalFilename)
 {
-	int	type = ROM_NDS;
-	char buf[MAX_PATH];
+	int					type = ROM_NDS;
+	char				buf[MAX_PATH];
 
 	if (filename == NULL)
 		return -1;
@@ -391,8 +464,6 @@ int NDS_LoadROM(const char *filename, const char *logicalFilename)
 	if (gameInfo.romsize < 352) {
 		return -1;
 	}
-	
-
 
 	//decrypt if necessary..
 	//but this is untested and suspected to fail on big endian, so lets not support this on big endian
@@ -412,6 +483,18 @@ int NDS_LoadROM(const char *filename, const char *logicalFilename)
 
 	MMU_unsetRom();
 	NDS_SetROM((u8*)gameInfo.romdata, gameInfo.mask);
+
+	gameInfo.populate();
+	gameInfo.crc = crc32(0,(u8*)gameInfo.romdata,gameInfo.romsize);
+	INFO("\nROM crc: %08X\n", gameInfo.crc);
+	INFO("ROM serial: %s\n", gameInfo.ROMserial);
+	INFO("ROM internal name: %s\n\n", gameInfo.ROMname);
+	INFO("ROM game code: %c%c%c%c\n\n", gameInfo.header.gameCode[0], gameInfo.header.gameCode[1], gameInfo.header.gameCode[2], gameInfo.header.gameCode[3]);
+
+	//for homebrew, try auto-patching DLDI. should be benign if there is no DLDI or if it fails
+	if(!memcmp(gameInfo.header.gameCode,"####",4))
+		DLDI::tryPatch((void*)gameInfo.romdata, gameInfo.romsize);
+
 	NDS_Reset();
 
 	memset(buf, 0, MAX_PATH);
@@ -429,12 +512,6 @@ int NDS_LoadROM(const char *filename, const char *logicalFilename)
 	strcat(buf, ".dct");							// DeSmuME cheat		:)
 	cheats->init(buf);
 
-	gameInfo.populate();
-	gameInfo.crc = crc32(0,(u8*)gameInfo.romdata,gameInfo.romsize);
-	INFO("\nROM crc: %08X\n", gameInfo.crc);
-	INFO("ROM serial: %s\n", gameInfo.ROMserial);
-	INFO("ROM internal name: %s\n\n", gameInfo.ROMname);
-
 	return 1;
 }
 #else
@@ -522,6 +599,18 @@ int NDS_LoadROM(const char *filename, const char *logicalFilename)
 		cheatSearch->close();
 	MMU_unsetRom();
 	NDS_SetROM(data, gameInfo.mask);
+
+	gameInfo.populate();
+	gameInfo.crc = crc32(0,data,size);
+	INFO("\nROM crc: %08X\n", gameInfo.crc);
+	INFO("ROM serial: %s\n", gameInfo.ROMserial);
+	INFO("ROM internal name: %s\n\n", gameInfo.ROMname);
+	INFO("ROM game code: %c%c%c%c\n\n", gameInfo.header.gameCode[0], gameInfo.header.gameCode[1], gameInfo.header.gameCode[2], gameInfo.header.gameCode[3]);
+
+	//for homebrew, try auto-patching DLDI. should be benign if there is no DLDI or if it fails
+	if(!memcmp(gameInfo.header.gameCode,"####",4))
+		DLDI::tryPatch((void*)data, gameInfo.mask + 1);
+
 	NDS_Reset();
 
 	free(noext);
@@ -541,12 +630,6 @@ int NDS_LoadROM(const char *filename, const char *logicalFilename)
 	strcat(buf, ".dct");							// DeSmuME cheat		:)
 	cheats->init(buf);
 
-	gameInfo.populate();
-	gameInfo.crc = crc32(0,data,size);
-	INFO("\nROM crc: %08X\n", gameInfo.crc);
-	INFO("ROM serial: %s\n", gameInfo.ROMserial);
-	INFO("ROM internal name: %s\n\n", gameInfo.ROMname);
-
 	return ret;
 }
 #endif
@@ -856,7 +939,7 @@ void NDS_ToggleCardEject()
 	if(!nds.cardEjected)
 	{
 		//staff of kings will test this (it also uses the arm9 0xB8 poll)
-		NDS_makeInt(1, 20);
+		NDS_makeIrq(ARMCPU_ARM7, IRQ_BIT_GC_IREQ_MC);
 	}
 	nds.cardEjected ^= TRUE;
 }
@@ -1085,8 +1168,7 @@ template<int procnum, int num> struct TSequenceItem_Timer : public TSequenceItem
 				MMU.timer[procnum][i] = MMU.timerReload[procnum][i];
 				if(T1ReadWord(regs, 0x102 + i*4) & 0x40) 
 				{
-					if(procnum==0) NDS_makeARM9Int(3 + i);
-					else NDS_makeARM7Int(3 + i);
+					NDS_makeIrq(procnum, IRQ_BIT_TIMER_0 + i);
 				}
 			}
 			else
@@ -1117,6 +1199,8 @@ template<int procnum, int chan> struct TSequenceItem_DMA : public TSequenceItem
 	{
 		IF_DEVELOPER(DEBUG_statistics.sequencerExecutionCounters[5+procnum*4+chan]++);
 
+		//if (nds.freezeBus) return;
+
 		//printf("exec from TSequenceItem_DMA: %d %d\n",procnum,chan);
 		controller->exec();
 //		//give gxfifo dmas a chance to re-trigger
@@ -1391,7 +1475,10 @@ static void execHardware_hblank()
 		//for some reason the game will setup two hdma scroll register buffers
 		//to be run consecutively, and unless we do this, the second buffer will be offset by one scanline
 		//causing a glitch in the 0th scanline
-		triggerDma(EDMAMode_HBlank);
+		//triggerDma(EDMAMode_HBlank);
+
+		//BUT! this was removed in order to make glitches in megaman zero collection (mmz 4 1st level) work.
+		//and, it seems that it is no longer necessary in nsmb. perhaps something else fixed it
 	}
 
 
@@ -1400,8 +1487,8 @@ static void execHardware_hblank()
 	T1WriteWord(MMU.ARM7_REG, 4, T1ReadWord(MMU.ARM7_REG, 4) | 2);
 
 	//fire hblank interrupts if necessary
-	NDS_ARM9HBlankInt();
-	NDS_ARM7HBlankInt();
+	if(T1ReadWord(MMU.ARM9_REG, 4) & 0x10) NDS_makeIrq(ARMCPU_ARM9,IRQ_BIT_LCD_HBLANK);
+	if(T1ReadWord(MMU.ARM7_REG, 4) & 0x10) NDS_makeIrq(ARMCPU_ARM7,IRQ_BIT_LCD_HBLANK);
 
 	//emulation housekeeping. for some reason we always do this at hblank,
 	//even though it sounds more reasonable to do it at hstart
@@ -1432,8 +1519,8 @@ static void execHardware_hstart_vblankStart()
 	T1WriteWord(MMU.ARM7_REG, 4, T1ReadWord(MMU.ARM7_REG, 4) | 1);
 
 	//fire vblank interrupts if necessary
-	NDS_ARM9VBlankInt();
-	NDS_ARM7VBlankInt();
+	if(T1ReadWord(MMU.ARM9_REG, 4) & 0x8) NDS_makeIrq(ARMCPU_ARM9,IRQ_BIT_LCD_VBLANK);
+	if(T1ReadWord(MMU.ARM7_REG, 4) & 0x8) NDS_makeIrq(ARMCPU_ARM7,IRQ_BIT_LCD_VBLANK);
 
 	//some emulation housekeeping
 	gfx3d_VBlankSignal();
@@ -1442,10 +1529,12 @@ static void execHardware_hstart_vblankStart()
 	triggerDma(EDMAMode_VBlank);
 
 	//tracking for arm9 load average
-	nds.runCycleCollector[nds.idleFrameCounter] = 1120380-nds.idleCycles;
+	nds.runCycleCollector[0][nds.idleFrameCounter] = 1120380-nds.idleCycles[0];
+	nds.runCycleCollector[1][nds.idleFrameCounter] = 1120380-nds.idleCycles[1];
 	nds.idleFrameCounter++;
 	nds.idleFrameCounter &= 15;
-	nds.idleCycles = 0;
+	nds.idleCycles[0] = 0;
+	nds.idleCycles[1] = 0;
 }
 
 static void execHardware_hstart_vcount()
@@ -1457,7 +1546,7 @@ static void execHardware_hstart_vcount()
 		//arm9 vmatch
 		T1WriteWord(MMU.ARM9_REG, 4, T1ReadWord(MMU.ARM9_REG, 4) | 4);
 		if(T1ReadWord(MMU.ARM9_REG, 4) & 32) {
-			NDS_makeARM9Int(2);
+			NDS_makeIrq(ARMCPU_ARM9,IRQ_BIT_LCD_VMATCH);
 		}
 	}
 	else
@@ -1470,7 +1559,7 @@ static void execHardware_hstart_vcount()
 		//arm7 vmatch
 		T1WriteWord(MMU.ARM7_REG, 4, T1ReadWord(MMU.ARM7_REG, 4) | 4);
 		if(T1ReadWord(MMU.ARM7_REG, 4) & 32)
-			NDS_makeARM7Int(2);
+			NDS_makeIrq(ARMCPU_ARM7,IRQ_BIT_LCD_VMATCH);
 	}
 	else
 		T1WriteWord(MMU.ARM7_REG, 4, T1ReadWord(MMU.ARM7_REG, 4) & 0xFFFB);
@@ -1677,10 +1766,6 @@ bool nds_loadstate(EMUFILE* is, int size)
 	return temp;
 }
 
-//#define LOG_ARM9
-//#define LOG_ARM7
-//bool dolog = false;
-
 FORCEINLINE void arm9log()
 {
 #ifdef LOG_ARM9
@@ -1692,18 +1777,34 @@ FORCEINLINE void arm9log()
 		else
 			des_arm_instructions_set[INDEX(NDS_ARM9.instruction)](NDS_ARM9.instruct_adr, NDS_ARM9.instruction, dasmbuf);
 
+#ifdef LOG_TO_FILE
+		if (!fp_dis9) return;
+#ifdef LOG_TO_FILE_REGS
+		fprintf(fp_dis9, "\t\t;R0:%08X R1:%08X R2:%08X R3:%08X R4:%08X R5:%08X R6:%08X R7:%08X R8:%08X R9:%08X\n\t\t;R10:%08X R11:%08X R12:%08X R13:%08X R14:%08X R15:%08X| next %08X, N:%i Z:%i C:%i V:%i\n",
+			NDS_ARM9.R[0],  NDS_ARM9.R[1],  NDS_ARM9.R[2],  NDS_ARM9.R[3],  NDS_ARM9.R[4],  NDS_ARM9.R[5],  NDS_ARM9.R[6],  NDS_ARM9.R[7], 
+			NDS_ARM9.R[8],  NDS_ARM9.R[9],  NDS_ARM9.R[10],  NDS_ARM9.R[11],  NDS_ARM9.R[12],  NDS_ARM9.R[13],  NDS_ARM9.R[14],  NDS_ARM9.R[15],
+			NDS_ARM9.next_instruction, NDS_ARM9.CPSR.bits.N, NDS_ARM9.CPSR.bits.Z, NDS_ARM9.CPSR.bits.C, NDS_ARM9.CPSR.bits.V);
+#endif
+		fprintf(fp_dis9, "%s %08X\t%08X \t%s\n", NDS_ARM9.CPSR.bits.T?"THUMB":"ARM", NDS_ARM9.instruct_adr, NDS_ARM9.instruction, dasmbuf);
+		/*if (NDS_ARM9.instruction == 0)
+		{
+			dolog = false;
+			INFO("Disassembler is stopped\n");
+		}*/
+#else
 		printf("%05d:%03d %12lld 9:%08X %08X %-30s R00:%08X R01:%08X R02:%08X R03:%08X R04:%08X R05:%08X R06:%08X R07:%08X R08:%08X R09:%08X R10:%08X R11:%08X R12:%08X R13:%08X R14:%08X R15:%08X\n",
 			currFrameCounter, nds.VCount, nds_timer, 
 			NDS_ARM9.instruct_adr,NDS_ARM9.instruction, dasmbuf, 
 			NDS_ARM9.R[0],  NDS_ARM9.R[1],  NDS_ARM9.R[2],  NDS_ARM9.R[3],  NDS_ARM9.R[4],  NDS_ARM9.R[5],  NDS_ARM9.R[6],  NDS_ARM9.R[7], 
 			NDS_ARM9.R[8],  NDS_ARM9.R[9],  NDS_ARM9.R[10],  NDS_ARM9.R[11],  NDS_ARM9.R[12],  NDS_ARM9.R[13],  NDS_ARM9.R[14],  NDS_ARM9.R[15]);  
+#endif
 	}
 #endif
 }
 
 FORCEINLINE void arm7log()
 {
-	#ifdef LOG_ARM7
+#ifdef LOG_ARM7
 	if(dolog)
 	{
 		char dasmbuf[4096];
@@ -1711,14 +1812,29 @@ FORCEINLINE void arm7log()
 			des_thumb_instructions_set[((NDS_ARM7.instruction)>>6)&1023](NDS_ARM7.instruct_adr, NDS_ARM7.instruction, dasmbuf);
 		else
 			des_arm_instructions_set[INDEX(NDS_ARM7.instruction)](NDS_ARM7.instruct_adr, NDS_ARM7.instruction, dasmbuf);
-		
+#ifdef LOG_TO_FILE
+		if (!fp_dis7) return;
+#ifdef LOG_TO_FILE_REGS
+		fprintf(fp_dis7, "\t\t;R0:%08X R1:%08X R2:%08X R3:%08X R4:%08X R5:%08X R6:%08X R7:%08X R8:%08X R9:%08X\n\t\t;R10:%08X R11:%08X R12:%08X R13:%08X R14:%08X R15:%08X| next %08X, N:%i Z:%i C:%i V:%i\n",
+			NDS_ARM7.R[0],  NDS_ARM7.R[1],  NDS_ARM7.R[2],  NDS_ARM7.R[3],  NDS_ARM7.R[4],  NDS_ARM7.R[5],  NDS_ARM7.R[6],  NDS_ARM7.R[7], 
+			NDS_ARM7.R[8],  NDS_ARM7.R[9],  NDS_ARM7.R[10],  NDS_ARM7.R[11],  NDS_ARM7.R[12],  NDS_ARM7.R[13],  NDS_ARM7.R[14],  NDS_ARM7.R[15],
+			NDS_ARM7.next_instruction, NDS_ARM7.CPSR.bits.N, NDS_ARM7.CPSR.bits.Z, NDS_ARM7.CPSR.bits.C, NDS_ARM7.CPSR.bits.V);
+#endif
+		fprintf(fp_dis7, "%s %08X\t%08X \t%s\n", NDS_ARM7.CPSR.bits.T?"THUMB":"ARM", NDS_ARM7.instruct_adr, NDS_ARM7.instruction, dasmbuf);
+		/*if (NDS_ARM7.instruction == 0)
+		{
+			dolog = false;
+			INFO("Disassembler is stopped\n");
+		}*/
+#else		
 		printf("%05d:%03d %12lld 7:%08X %08X %-30s R00:%08X R01:%08X R02:%08X R03:%08X R04:%08X R05:%08X R06:%08X R07:%08X R08:%08X R09:%08X R10:%08X R11:%08X R12:%08X R13:%08X R14:%08X R15:%08X\n",
 			currFrameCounter, nds.VCount, nds_timer, 
 			NDS_ARM7.instruct_adr,NDS_ARM7.instruction, dasmbuf, 
 			NDS_ARM7.R[0],  NDS_ARM7.R[1],  NDS_ARM7.R[2],  NDS_ARM7.R[3],  NDS_ARM7.R[4],  NDS_ARM7.R[5],  NDS_ARM7.R[6],  NDS_ARM7.R[7], 
 			NDS_ARM7.R[8],  NDS_ARM7.R[9],  NDS_ARM7.R[10],  NDS_ARM7.R[11],  NDS_ARM7.R[12],  NDS_ARM7.R[13],  NDS_ARM7.R[14],  NDS_ARM7.R[15]);
+#endif
 	}
-	#endif
+#endif
 }
 
 //these have not been tuned very well yet.
@@ -1747,7 +1863,7 @@ static /*donotinline*/ std::pair<s32,s32> armInnerLoop(
 	{
 		if(doarm9 && (!doarm7 || arm9 <= timer))
 		{
-			if(!NDS_ARM9.waitIRQ)
+			if(!NDS_ARM9.waitIRQ&&!nds.freezeBus)
 			{
 				arm9log();
 				arm9 += armcpu_exec<ARMCPU_ARM9>();
@@ -1759,12 +1875,13 @@ static /*donotinline*/ std::pair<s32,s32> armInnerLoop(
 			{
 				s32 temp = arm9;
 				arm9 = min(s32next, arm9 + kIrqWait);
-				nds.idleCycles += arm9-temp;
+				nds.idleCycles[0] += arm9-temp;
+				if (gxFIFO.size < 255) nds.freezeBus = FALSE;
 			}
 		}
 		if(doarm7 && (!doarm9 || arm7 <= timer))
 		{
-			if(!NDS_ARM7.waitIRQ)
+			if(!NDS_ARM7.waitIRQ&&!nds.freezeBus)
 			{
 				arm7log();
 				arm7 += (armcpu_exec<ARMCPU_ARM7>()<<1);
@@ -1774,7 +1891,9 @@ static /*donotinline*/ std::pair<s32,s32> armInnerLoop(
 			}
 			else
 			{
+				s32 temp = arm7;
 				arm7 = min(s32next, arm7 + kIrqWait);
+				nds.idleCycles[1] += arm7-temp;
 				if(arm7 == s32next)
 				{
 					nds_timer = nds_timer_base + minarmtime<doarm9,false>(arm9,arm7);
@@ -1826,12 +1945,10 @@ void NDS_exec(s32 nb)
 
 	if(nds.sleeping)
 	{
-		gpu_UpdateRender();
-
 		//speculative code: if ANY irq happens, wake up the arm7.
 		//I think the arm7 program analyzes the system and may decide not to wake up
 		//if it is dissatisfied with the conditions
-		if((MMU.reg_IE[1] & MMU.reg_IF[1]))
+		if((MMU.reg_IE[1] & MMU.gen_IF<1>()))
 		{
 			nds.sleeping = FALSE;
 		}
@@ -1843,7 +1960,7 @@ void NDS_exec(s32 nb)
 			//trap the debug-stalled condition
 			#ifdef DEVELOPER
 				singleStep = false;
-				//(gbd stub doesnt yet know how to trigger these immediately by calling reschedule)
+				//(gdb stub doesnt yet know how to trigger these immediately by calling reschedule)
 				while((NDS_ARM9.stalled || NDS_ARM7.stalled) && execute)
 				{
 					driver->EMU_DebugIdleUpdate();
@@ -1906,9 +2023,17 @@ void NDS_exec(s32 nb)
 #endif
 
 			//if we were waiting for an irq, don't wait too long:
-			//let's re-analyze it after this hardware event
-			if(NDS_ARM9.waitIRQ) nds_arm9_timer = nds_timer;
-			if(NDS_ARM7.waitIRQ) nds_arm7_timer = nds_timer;
+			//let's re-analyze it after this hardware event (this rolls back a big burst of irq waiting which may have been interrupted by a resynch)
+			if(NDS_ARM9.waitIRQ)
+			{
+				nds.idleCycles[0] -= (s32)(nds_arm9_timer-nds_timer);
+				nds_arm9_timer = nds_timer;
+			}
+			if(NDS_ARM7.waitIRQ)
+			{
+				nds.idleCycles[1] -= (s32)(nds_arm7_timer-nds_timer);
+				nds_arm7_timer = nds_timer;
+			}
 		}
 	}
 
@@ -1932,34 +2057,30 @@ void NDS_exec(s32 nb)
 		cheats->process();
 }
 
-void execHardware_interrupts()
+template<int PROCNUM> static void execHardware_interrupts_core()
 {
-	if((MMU.reg_IF[0]&MMU.reg_IE[0]) && (MMU.reg_IME[0]))
+	u32 IF = MMU.gen_IF<PROCNUM>();
+	u32 IE = MMU.reg_IE[PROCNUM];
+	u32 masked = IF & IE;
+	if(ARMPROC.halt_IE_and_IF && masked)
 	{
-#ifdef GDB_STUB
-		if ( armcpu_flagIrq( &NDS_ARM9)) 
-#else
-		if ( armcpu_irqException(&NDS_ARM9))
-#endif
-		{
-			//printf("ARM9 interrupt! flags: %08X ; mask: %08X ; result: %08X\n",MMU.reg_IF[0],MMU.reg_IE[0],MMU.reg_IF[0]&MMU.reg_IE[0]);
-			//nds.ARM9Cycle = nds.cycles;
-		}
+		ARMPROC.halt_IE_and_IF = FALSE;
+		ARMPROC.waitIRQ = FALSE;
 	}
 
-	if((MMU.reg_IF[1]&MMU.reg_IE[1]) && (MMU.reg_IME[1]))
+	if(masked && MMU.reg_IME[PROCNUM] && !ARMPROC.CPSR.bits.I)
 	{
-#ifdef GDB_STUB
-		if ( armcpu_flagIrq( &NDS_ARM7)) 
-#else
-		if ( armcpu_irqException(&NDS_ARM7))
-#endif
-		{
-			//nds.ARM7Cycle = nds.cycles;
-		}
+		//printf("Executing IRQ on procnum %d with IF = %08X and IE = %08X\n",PROCNUM,IF,IE);
+		armcpu_irqException(&ARMPROC);
 	}
 }
 
+void execHardware_interrupts()
+{
+	execHardware_interrupts_core<ARMCPU_ARM9>();
+	execHardware_interrupts_core<ARMCPU_ARM7>();
+}
+
 static void resetUserInput();
 
 bool _HACK_DONT_STOPMOVIE = false;
@@ -1979,6 +2100,7 @@ void NDS_Reset()
 
 	nds.sleeping = FALSE;
 	nds.cardEjected = FALSE;
+	nds.freezeBus = FALSE;
 	nds.power1.lcd = nds.power1.gpuMain = nds.power1.gfx3d_render = nds.power1.gfx3d_geometry = nds.power1.gpuSub = nds.power1.dispswap = 1;
 	nds.power2.speakers = 1;
 	nds.power2.wifi = 0;
@@ -2004,6 +2126,11 @@ void NDS_Reset()
 
 	MMU_Reset();
 
+	NDS_ARM7.BIOS_loaded = false;
+	NDS_ARM9.BIOS_loaded = false;
+	memset(MMU.ARM7_BIOS, 0, sizeof(MMU.ARM7_BIOS));
+	memset(MMU.ARM9_BIOS, 0, sizeof(MMU.ARM9_BIOS));
+
 	//ARM7 BIOS IRQ HANDLER
 	if(CommonSettings.UseExtBIOS == true)
 		inf = fopen(CommonSettings.ARM7BIOS,"rb");
@@ -2012,33 +2139,39 @@ void NDS_Reset()
 
 	if(inf) 
 	{
-		fread(MMU.ARM7_BIOS,1,16384,inf);
+		if (fread(MMU.ARM7_BIOS,1,16384,inf) == 16384) NDS_ARM7.BIOS_loaded = true;
 		fclose(inf);
 
-		if(CommonSettings.SWIFromBIOS == true) NDS_ARM7.swi_tab = 0;
+		if((CommonSettings.SWIFromBIOS) && (NDS_ARM7.BIOS_loaded)) NDS_ARM7.swi_tab = 0;
 		else NDS_ARM7.swi_tab = ARM7_swi_tab;
 
 		if (CommonSettings.PatchSWI3)
 			_MMU_write16<ARMCPU_ARM7>(0x00002F08, 0x4770);
 
-		INFO("ARM7 BIOS is loaded.\n");
+		INFO("ARM7 BIOS is %s.\n", NDS_ARM7.BIOS_loaded?"loaded":"failed");
 	} 
 	else 
 	{
 		NDS_ARM7.swi_tab = ARM7_swi_tab;
 
-		for (int t = 0; t < 16384; t++)
-			MMU.ARM7_BIOS[t] = 0xFF;
-
-		T1WriteLong(MMU.ARM7_BIOS,0x00, 0xE25EF002);
-		T1WriteLong(MMU.ARM7_BIOS,0x04, 0xEAFFFFFE);
-		T1WriteLong(MMU.ARM7_BIOS,0x18, 0xEA000000);
-		T1WriteLong(MMU.ARM7_BIOS,0x20, 0xE92D500F);
-		T1WriteLong(MMU.ARM7_BIOS,0x24, 0xE3A00301);
-		T1WriteLong(MMU.ARM7_BIOS,0x28, 0xE28FE000);
-		T1WriteLong(MMU.ARM7_BIOS,0x2C, 0xE510F004);
-		T1WriteLong(MMU.ARM7_BIOS,0x30, 0xE8BD500F);
-		T1WriteLong(MMU.ARM7_BIOS,0x34, 0xE25EF004);
+#if 0
+		// TODO
+		T1WriteLong(MMU.ARM7_BIOS, 0x0000, 0xEAFFFFFE);		// loop for Reset !!!
+		T1WriteLong(MMU.ARM7_BIOS, 0x0004, 0xEAFFFFFE);		// loop for Undef instr expection
+		T1WriteLong(MMU.ARM7_BIOS, 0x0008, 0xEA00009C);		// SWI
+		T1WriteLong(MMU.ARM7_BIOS, 0x000C, 0xEAFFFFFE);		// loop for Prefetch Abort
+		T1WriteLong(MMU.ARM7_BIOS, 0x0010, 0xEAFFFFFE);		// loop for Data Abort
+		T1WriteLong(MMU.ARM7_BIOS, 0x0014, 0x00000000);		// Reserved
+		T1WriteLong(MMU.ARM7_BIOS, 0x001C, 0x00000000);		// Fast IRQ
+#endif
+		T1WriteLong(MMU.ARM7_BIOS, 0x0000, 0xE25EF002);
+		T1WriteLong(MMU.ARM7_BIOS, 0x0018, 0xEA000000);
+		T1WriteLong(MMU.ARM7_BIOS, 0x0020, 0xE92D500F);
+		T1WriteLong(MMU.ARM7_BIOS, 0x0024, 0xE3A00301);
+		T1WriteLong(MMU.ARM7_BIOS, 0x0028, 0xE28FE000);
+		T1WriteLong(MMU.ARM7_BIOS, 0x002C, 0xE510F004);
+		T1WriteLong(MMU.ARM7_BIOS, 0x0030, 0xE8BD500F);
+		T1WriteLong(MMU.ARM7_BIOS, 0x0034, 0xE25EF004);
 	}
 
 	//ARM9 BIOS IRQ HANDLER
@@ -2046,44 +2179,105 @@ void NDS_Reset()
 		inf = fopen(CommonSettings.ARM9BIOS,"rb");
 	else
 		inf = NULL;
-	//memcpy(MMU.ARM9_BIOS + 0x20, gba_header_data_0x04, 156);
 
 	if(inf) 
 	{
-		fread(MMU.ARM9_BIOS,1,4096,inf);
+		if (fread(MMU.ARM9_BIOS,1,4096,inf) == 4096) NDS_ARM9.BIOS_loaded = true;
 		fclose(inf);
 
-		if(CommonSettings.SWIFromBIOS == true) NDS_ARM9.swi_tab = 0;
+		if((CommonSettings.SWIFromBIOS) && (NDS_ARM9.BIOS_loaded)) NDS_ARM9.swi_tab = 0;
 		else NDS_ARM9.swi_tab = ARM9_swi_tab;
 
 		if (CommonSettings.PatchSWI3)
 			_MMU_write16<ARMCPU_ARM9>(0xFFFF07CC, 0x4770);
 
-		INFO("ARM9 BIOS is loaded.\n");
+		INFO("ARM9 BIOS is %s.\n", NDS_ARM9.BIOS_loaded?"loaded":"failed");
 	} 
 	else 
 	{
 		NDS_ARM9.swi_tab = ARM9_swi_tab;
 
-		for (int t = 0; t < 4096; t++)
-			MMU.ARM9_BIOS[t] = 0xFF;
+		//bios chains data abort to fast irq
 
-		_MMU_write32<ARMCPU_ARM9>(0xFFFF0018, 0xEA000095);
-
-		for (int t = 0; t < 156; t++)		// load logo
+		//exception vectors:
+		T1WriteLong(MMU.ARM9_BIOS, 0x0000, 0xEAFFFFFE);		// (infinite loop for) Reset !!!
+		//T1WriteLong(MMU.ARM9_BIOS, 0x0004, 0xEAFFFFFE);		// (infinite loop for) Undefined instruction
+		T1WriteLong(MMU.ARM9_BIOS, 0x0004, 0xEA000004);		// Undefined instruction -> Fast IRQ (just guessing)
+		T1WriteLong(MMU.ARM9_BIOS, 0x0008, 0xEA00009C);		// SWI -> ?????
+		T1WriteLong(MMU.ARM9_BIOS, 0x000C, 0xEAFFFFFE);		// (infinite loop for) Prefetch Abort
+		T1WriteLong(MMU.ARM9_BIOS, 0x0010, 0xEA000001);		// Data Abort -> Fast IRQ
+		T1WriteLong(MMU.ARM9_BIOS, 0x0014, 0x00000000);		// Reserved
+		T1WriteLong(MMU.ARM9_BIOS, 0x0018, 0xEA000095);		// Normal IRQ -> 0x0274
+		T1WriteLong(MMU.ARM9_BIOS, 0x001C, 0xEA00009D);		// Fast IRQ -> 0x0298
+		
+		// logo (do some games fail to boot without this? example?)
+		for (int t = 0; t < 0x9C; t++)
 			MMU.ARM9_BIOS[t + 0x20] = logo_data[t];
 
-		_MMU_write32<ARMCPU_ARM9>(0xFFFF0274, 0xE92D500F);
-		_MMU_write32<ARMCPU_ARM9>(0xFFFF0278, 0xEE190F11);
-		_MMU_write32<ARMCPU_ARM9>(0xFFFF027C, 0xE1A00620);
-		_MMU_write32<ARMCPU_ARM9>(0xFFFF0280, 0xE1A00600);
-		_MMU_write32<ARMCPU_ARM9>(0xFFFF0284, 0xE2800C40);
-		_MMU_write32<ARMCPU_ARM9>(0xFFFF0288, 0xE28FE000);
-		_MMU_write32<ARMCPU_ARM9>(0xFFFF028C, 0xE510F004);
-		_MMU_write32<ARMCPU_ARM9>(0xFFFF0290, 0xE8BD500F);
-		_MMU_write32<ARMCPU_ARM9>(0xFFFF0294, 0xE25EF004);
+		//...0xBC:
+
+		//(now what goes in this gap??)
+
+		//IRQ handler: get dtcm address and jump to a vector in it
+		T1WriteLong(MMU.ARM9_BIOS, 0x0274, 0xE92D500F); //STMDB SP!, {R0-R3,R12,LR} 
+		T1WriteLong(MMU.ARM9_BIOS, 0x0278, 0xEE190F11); //MRC CP15, 0, R0, CR9, CR1, 0
+		T1WriteLong(MMU.ARM9_BIOS, 0x027C, 0xE1A00620); //MOV R0, R0, LSR #C
+		T1WriteLong(MMU.ARM9_BIOS, 0x0280, 0xE1A00600); //MOV R0, R0, LSL #C 
+		T1WriteLong(MMU.ARM9_BIOS, 0x0284, 0xE2800C40); //ADD R0, R0, #4000
+		T1WriteLong(MMU.ARM9_BIOS, 0x0288, 0xE28FE000); //ADD LR, PC, #0   
+		T1WriteLong(MMU.ARM9_BIOS, 0x028C, 0xE510F004); //LDR PC, [R0, -#4] 
+
+		//????
+		T1WriteLong(MMU.ARM9_BIOS, 0x0290, 0xE8BD500F); //LDMIA SP!, {R0-R3,R12,LR}
+		T1WriteLong(MMU.ARM9_BIOS, 0x0294, 0xE25EF004); //SUBS PC, LR, #4
+
+		//-------
+		//FIQ and abort exception handler
+		//TODO - this code is copied from the bios. refactor it
+		//friendly reminder: to calculate an immediate offset: encoded = (desired_address-cur_address-8)
+
+		T1WriteLong(MMU.ARM9_BIOS, 0x0298, 0xE10FD000); //MRS SP, CPSR  
+		T1WriteLong(MMU.ARM9_BIOS, 0x029C, 0xE38DD0C0); //ORR SP, SP, #C0
+		
+		T1WriteLong(MMU.ARM9_BIOS, 0x02A0, 0xE12FF00D); //MSR CPSR_fsxc, SP
+		T1WriteLong(MMU.ARM9_BIOS, 0x02A4, 0xE59FD000 | (0x2D4-0x2A4-8)); //LDR SP, [FFFF02D4]
+		T1WriteLong(MMU.ARM9_BIOS, 0x02A8, 0xE28DD001); //ADD SP, SP, #1   
+		T1WriteLong(MMU.ARM9_BIOS, 0x02AC, 0xE92D5000); //STMDB SP!, {R12,LR}
+		
+		T1WriteLong(MMU.ARM9_BIOS, 0x02B0, 0xE14FE000); //MRS LR, SPSR
+		T1WriteLong(MMU.ARM9_BIOS, 0x02B4, 0xEE11CF10); //MRC CP15, 0, R12, CR1, CR0, 0
+		T1WriteLong(MMU.ARM9_BIOS, 0x02B8, 0xE92D5000); //STMDB SP!, {R12,LR}
+		T1WriteLong(MMU.ARM9_BIOS, 0x02BC, 0xE3CCC001); //BIC R12, R12, #1
+
+		T1WriteLong(MMU.ARM9_BIOS, 0x02C0, 0xEE01CF10); //MCR CP15, 0, R12, CR1, CR0, 0
+		T1WriteLong(MMU.ARM9_BIOS, 0x02C4, 0xE3CDC001); //BIC R12, SP, #1    
+		T1WriteLong(MMU.ARM9_BIOS, 0x02C8, 0xE59CC010); //LDR R12, [R12, #10] 
+		T1WriteLong(MMU.ARM9_BIOS, 0x02CC, 0xE35C0000); //CMP R12, #0  
+
+		T1WriteLong(MMU.ARM9_BIOS, 0x02D0, 0x112FFF3C); //BLXNE R12    
+		T1WriteLong(MMU.ARM9_BIOS, 0x02D4, 0x027FFD9C); //0x027FFD9C  
+		//---------
+
 	}
 
+#ifdef LOG_ARM7
+	if (fp_dis7 != NULL) 
+	{
+		fclose(fp_dis7);
+		fp_dis7 = NULL;
+	}
+	fp_dis7 = fopen("D:\\desmume_dis7.asm", "w");
+#endif
+
+#ifdef LOG_ARM9
+	if (fp_dis9 != NULL) 
+	{
+		fclose(fp_dis9);
+		fp_dis9 = NULL;
+	}
+	fp_dis9 = fopen("D:\\desmume_dis9.asm", "w");
+#endif
+
 	if (firmware)
 	{
 		delete firmware;
@@ -2091,7 +2285,7 @@ void NDS_Reset()
 	}
 	firmware = new CFIRMWARE();
 	fw_success = firmware->load();
-	if ((CommonSettings.UseExtBIOS == true) && (CommonSettings.BootFromFirmware == true) && (fw_success == TRUE))
+	if (NDS_ARM7.BIOS_loaded && NDS_ARM9.BIOS_loaded && CommonSettings.BootFromFirmware && fw_success)
 	{
 		// Copy secure area to memory if needed
 		if ((header->ARM9src >= 0x4000) && (header->ARM9src < 0x8000))
@@ -2165,6 +2359,9 @@ void NDS_Reset()
 	NDS_ARM9.R13_svc = 0x00803FC0;
 	NDS_ARM9.R13_irq = 0x00803FA0;
 	NDS_ARM9.R13_usr = 0x00803EC0;
+	NDS_ARM9.R13_abt = NDS_ARM9.R13_usr; //????? 
+	//I think it is wrong to take gbatek's "SYS" and put it in USR--maybe USR doesnt matter. 
+	//i think SYS is all the misc modes. please verify by setting nonsensical stack values for USR here
 	NDS_ARM9.R[13] = NDS_ARM9.R13_usr;
 	//n.b.: im not sure about all these, I dont know enough about arm9 svc/irq/etc modes
 	//and how theyre named in desmume to match them up correctly. i just guessed.
@@ -2224,8 +2421,8 @@ void NDS_Reset()
 	std::string rompath = "fat:/" + path.RomName;
 	const u32 kCommandline = 0x027E0000;
 	//const u32 kCommandline = 0x027FFF84;
-	
-	// 
+
+	//
 	_MMU_write32<ARMCPU_ARM9>(0x02FFFE70, 0x5f617267);
 	_MMU_write32<ARMCPU_ARM9>(0x02FFFE74, kCommandline); //(commandline starts here)
 	_MMU_write32<ARMCPU_ARM9>(0x02FFFE78, rompath.size()+1);
@@ -2535,11 +2732,11 @@ static void NDS_applyFinalInput()
 		u16 k_cnt_selected = (k_cnt & 0x3F);
 		if (k_cnt&(1<<15))	// AND
 		{
-			if ((~pad & k_cnt_selected) == k_cnt_selected) NDS_makeARM9Int(12);
+			if ((~pad & k_cnt_selected) == k_cnt_selected) NDS_makeIrq(ARMCPU_ARM9,IRQ_BIT_KEYPAD);
 		}
 		else				// OR
 		{
-			if (~pad & k_cnt_selected) NDS_makeARM9Int(12);
+			if (~pad & k_cnt_selected) NDS_makeIrq(ARMCPU_ARM9,IRQ_BIT_KEYPAD);
 		}
 	}
 
@@ -2550,11 +2747,11 @@ static void NDS_applyFinalInput()
 		u16 k_cnt_selected = (k_cnt & 0x3F);
 		if (k_cnt&(1<<15))	// AND
 		{
-			if ((~pad & k_cnt_selected) == k_cnt_selected) NDS_makeARM7Int(12);
+			if ((~pad & k_cnt_selected) == k_cnt_selected) NDS_makeIrq(ARMCPU_ARM7,IRQ_BIT_KEYPAD);
 		}
 		else				// OR
 		{
-			if (~pad & k_cnt_selected) NDS_makeARM7Int(12);
+			if (~pad & k_cnt_selected) NDS_makeIrq(ARMCPU_ARM7,IRQ_BIT_KEYPAD);
 		}
 	}
 
@@ -2583,7 +2780,7 @@ static void NDS_applyFinalInput()
 		if (!LidClosed)
 		{
 		//	SPU_Pause(FALSE);
-			NDS_makeARM7Int(22);
+			NDS_makeIrq(ARMCPU_ARM7,IRQ_BIT_ARM7_FOLD);
 
 		}
 		//else
@@ -2657,6 +2854,25 @@ void NDS_suspendProcessingInput(bool suspend)
 void emu_halt() {
 	//printf("halting emu: ARM9 PC=%08X/%08X, ARM7 PC=%08X/%08X\n", NDS_ARM9.R[15], NDS_ARM9.instruct_adr, NDS_ARM7.R[15], NDS_ARM7.instruct_adr);
 	execute = false;
+#ifdef LOG_ARM9
+	if (fp_dis9)
+	{
+		char buf[256] = { 0 };
+		sprintf(buf, "halting emu: ARM9 PC=%08X/%08X\n", NDS_ARM9.R[15], NDS_ARM9.instruct_adr);
+		fwrite(buf, 1, strlen(buf), fp_dis9);
+		INFO("ARM9 halted\n");
+	}
+#endif
+
+#ifdef LOG_ARM7
+	if (fp_dis7)
+	{
+		char buf[256] = { 0 };
+		sprintf(buf, "halting emu: ARM7 PC=%08X/%08X\n", NDS_ARM7.R[15], NDS_ARM7.instruct_adr);
+		fwrite(buf, 1, strlen(buf), fp_dis7);
+		INFO("ARM7 halted\n");
+	}
+#endif
 }
 
 //these templates needed to be instantiated manually
