@@ -1,7 +1,6 @@
 /*  Copyright (C) 2006 yopyop
 	Copyright (C) 2006 Mic
-    Copyright (C) 2009 CrazyMax 
-	Copyright (C) 2009 DeSmuME team
+	Copyright (C) 2009-2010 DeSmuME team
 
     This file is part of DeSmuME
 
@@ -28,6 +27,12 @@
 #include <stdio.h>
 #include <stdlib.h>
 
+#include "../emufat.h"
+
+#include "../utils/libfat/libfat_public_api.h"
+
+#include <stack>
+
 #include <fcntl.h>
 #include <sys/stat.h>
 #ifdef _MSC_VER
@@ -77,9 +82,6 @@ typedef struct {
 #define CF_CMD_READ 0x20
 #define CF_CMD_WRITE 0x30
 
-static int disk_image = -1;
-static off_t file_size;
-
 static u16	cf_reg_sts, 
 			cf_reg_lba1,
 			cf_reg_lba2,
@@ -119,6 +121,8 @@ static BOOL cflashDeviceEnabled = FALSE;
 
 static std::string sFlashPath;
 
+static EMUFILE* file;
+
 // ===========================
 BOOL	inited;
 
@@ -245,8 +249,14 @@ static void add_file(char *fname, FsEntry * entry, int fileLevel)
 	}
 }
 
+enum EListCallbackArg {
+	EListCallbackArg_Item, EListCallbackArg_Pop
+};
+
+typedef void (*ListCallback)(FsEntry* fs, EListCallbackArg);
+
 // List all files and subdirectories recursively
-static void list_files(const char *filepath)
+static void list_files(const char *filepath, ListCallback list_callback)
 {
 	char DirSpec[255+1], SubDir[255+1];
 	FsEntry entry;
@@ -265,13 +275,15 @@ static void list_files(const char *filepath)
 	if (hFind == NULL) return;
 
 	fname = (strlen(entry.cAlternateFileName)>0) ? entry.cAlternateFileName : entry.cFileName;
-	add_file(fname, &entry, fileLevel);
+	list_callback(&entry,EListCallbackArg_Item);
+	//add_file(fname, &entry, fileLevel);
 
 	while (FsReadNext(hFind, &entry) != 0)
 	{
 		fname = (strlen(entry.cAlternateFileName)>0) ? entry.cAlternateFileName : entry.cFileName;
-		add_file(fname, &entry, fileLevel);
-		CFLASHLOG("cflash added %s\n",fname);
+		//add_file(fname, &entry, fileLevel);
+		list_callback(&entry,EListCallbackArg_Item);
+		printf("cflash added %s\n",entry.cFileName);
 
 		if (numFiles==MAXFILES-1) break;
 
@@ -280,7 +292,8 @@ static void list_files(const char *filepath)
 			if (strlen(fname)+strlen(filepath)+2 < 256) 
 			{
 				sprintf(SubDir, "%s%c%s", filepath, FS_SEPARATOR, fname);
-				list_files(SubDir);
+				list_files(SubDir, list_callback);
+				list_callback(&entry, EListCallbackArg_Pop);
 			}
 		}
 	}
@@ -289,212 +302,332 @@ static void list_files(const char *filepath)
 	FsClose(hFind);
 	if (dwError != FS_ERR_NO_MORE_FILES) return;
 
-	if (numFiles < MAXFILES)
-	{
-		fileLink[numFiles].parent = fileLevel;
-		files[numFiles++].name[0] = 0;
-	}
+	//if (numFiles < MAXFILES)
+	//{
+	//	fileLink[numFiles].parent = fileLevel;
+	//	files[numFiles++].name[0] = 0;
+	//}
 }
 
+static u32 dataSectors = 0;
+void count_ListCallback(FsEntry* fs, EListCallbackArg arg)
+{
+	if(arg == EListCallbackArg_Pop) return;
+	u32 sectors = 1;
+	if(fs->flags & FS_IS_DIR)
+	{
+	}
+	else
+		sectors += (fs->fileSize+511)/512 + 1;
+	dataSectors += sectors; 
+}
+
+static std::string currPath;
+static std::stack<std::string> pathStack;
+static std::stack<std::string> virtPathStack;
+static std::string currVirtPath;
+void build_ListCallback(FsEntry* fs, EListCallbackArg arg)
+{
+	char* fname = (strlen(fs->cAlternateFileName)>0) ? fs->cAlternateFileName : fs->cFileName;
+
+	//we use cFileName always because it is a LFN and we are making sure that we always make a fat32 image
+	fname = fs->cFileName;
+
+	if(arg == EListCallbackArg_Pop) 
+	{
+		currPath = pathStack.top();
+		pathStack.pop();
+		currVirtPath = virtPathStack.top();
+		virtPathStack.pop();
+		return;
+	}
+	
+	if(fs->flags & FS_IS_DIR)
+	{
+		if(!strcmp(fname,".")) return;
+		if(!strcmp(fname,"..")) return;
+
+		pathStack.push(currPath);
+		virtPathStack.push(currVirtPath);
+
+		currVirtPath = currVirtPath + "/" + fname;
+		bool ok = LIBFAT::MkDir(currVirtPath.c_str());
+
+		if(!ok)
+			printf("ERROR adding dir %s via libfat\n",currVirtPath.c_str());
+
+		currPath = currPath + std::string(1,FS_SEPARATOR) + fname;
+		return;
+	}
+	else
+	{
+		std::string path = currPath + std::string(1,FS_SEPARATOR) + fname;
+
+		FILE* inf = fopen(path.c_str(),"rb");
+		if(inf)
+		{
+			fseek(inf,0,SEEK_END);
+			long len = ftell(inf);
+			fseek(inf,0,SEEK_SET);
+			u8 *buf = new u8[len];
+			fread(buf,1,len,inf);
+			fclose(inf);
+
+			std::string path = currVirtPath + "/" + fname;
+			bool ok = LIBFAT::WriteFile(path.c_str(),buf,len);
+			if(!ok) 
+				printf("ERROR adding file %s via libfat\n",path.c_str());
+			delete[] buf;
+		}
+	}
+		
+}
+
+
 // Set up the MBR, FAT and DIR_ENTs
 static BOOL cflash_build_fat()
 {
-	int i,j,k,l,
-	clust,numClusters,
-	clusterNum2,rootCluster;
-	int fileLevel;
+	dataSectors = 0;
+	currPath = sFlashPath;
+	currVirtPath = "";
+	list_files(sFlashPath.c_str(), count_ListCallback);
 
-	numFiles  = 0;
-	fileLevel = -1;
-	maxLevel  = -1;
+	dataSectors += 8; //a few for reserved sectors, etc.
 
-	files = (DIR_ENT *) malloc(MAXFILES*sizeof(DIR_ENT));
-	if (files == NULL) return FALSE;
-	fileLink = (FILE_INFO *) malloc(MAXFILES*sizeof(FILE_INFO));
-	if (fileLink == NULL)
+	dataSectors += 16*1024*1024/512; //add 16MB worth of write space. this is probably enough for anyone, but maybe it should be configurable.
+	//we could always suggest to users to add a big file to their directory to overwrite (that would cause the image to get padded)
+
+	//this seems to be the minimum size that will turn into a solid fat32
+	if(dataSectors<36*1024*1024/512)
+		dataSectors = 36*1024*1024/512;
+	
+	delete file;
+	file = new EMUFILE_MEMORY(dataSectors*512);
+	//file = new EMUFILE_FILE("c:\\temp.ima","rb+");
+	
+	//format the disk
 	{
-		free(files);
-		return FALSE;
+		EmuFat fat(file);
+		EmuFatVolume vol;
+		u8 ok = vol.init(&fat);
+		vol.formatNew(dataSectors);
+
+		//ensure we are working in memory, just in case we were testing with a disk file.
+		//libfat will need to go straight to memory (for now; we could easily change it to work with the disk)
+		file = file->memwrap();
 	}
+	EMUFILE_MEMORY* memf = (EMUFILE_MEMORY*)file;
 
-	for (i=0; i<MAXFILES; i++)
-	{
-		files[i].attrib = 0;
-		files[i].name[0] = FILE_FREE;
-		files[i].fileSize = 0;
+	//setup libfat and write all the files through it
+	LIBFAT::Init(memf->buf(),memf->size());
+	list_files(sFlashPath.c_str(), build_ListCallback);
+	LIBFAT::Shutdown();
 
-		fileLink[i].filesInDir = 0;
 
-		extraDirEntries[i] = NULL;
-		numExtraEntries[i] = 0;
-	}
+	//FILE* outf = fopen("d:\\test.ima","wb");
+	//EMUFILE_MEMORY* memf = (EMUFILE_MEMORY*)file;
+	//fwrite(memf->buf(),1,memf->size(),outf);
+	//fclose(outf);
 
-	list_files(sFlashPath.c_str());
+	
 
-	k            = 0;
-	clusterNum   = rootCluster = (SECRESV + SECPERFAT)/SECPERCLUS;
-	numClusters  = 0;
-	clust        = 0;
-	numRootFiles = 0;
+	//int i,j,k,l,
+	//clust,numClusters,
+	//clusterNum2,rootCluster;
+	//int fileLevel;
 
-	// Allocate memory to hold information about the files 
-	dirEntries = (DIR_ENT *) malloc(numFiles*sizeof(DIR_ENT));
-	if (dirEntries==NULL) return FALSE;
+	//numFiles  = 0;
+	//fileLevel = -1;
+	//maxLevel  = -1;
 
-	dirEntryLink = (FILE_INFO *) malloc(numFiles*sizeof(FILE_INFO));
-	if (dirEntryLink==NULL)
-	{
-		free(dirEntries);
-		return FALSE;
-	}
-	dirEntriesInCluster = (int *) malloc(NUMCLUSTERS*sizeof(int));
-	if (dirEntriesInCluster==NULL)
-	{
-		free(dirEntries);
-		free(dirEntryLink);
-		return FALSE;
-	}
-	dirEntryPtr = (DIR_ENT **) malloc(NUMCLUSTERS*sizeof(DIR_ENT*));
-	if (dirEntryPtr==NULL)
-	{
-		free(dirEntries);
-		free(dirEntryLink);
-		free(dirEntriesInCluster);
-		return FALSE;
-	}
+	//files = (DIR_ENT *) malloc(MAXFILES*sizeof(DIR_ENT));
+	//memset(files,0,MAXFILES*sizeof(DIR_ENT));
+	//if (files == NULL) return FALSE;
+	//fileLink = (FILE_INFO *) malloc(MAXFILES*sizeof(FILE_INFO));
+	//if (fileLink == NULL)
+	//{
+	//	free(files);
+	//	return FALSE;
+	//}
 
-	memset(dirEntriesInCluster, 0, NUMCLUSTERS*sizeof(int));
-	memset(dirEntryPtr, NULL, NUMCLUSTERS*sizeof(DIR_ENT*));
+	//for (i=0; i<MAXFILES; i++)
+	//{
+	//	files[i].attrib = 0;
+	//	files[i].name[0] = FILE_FREE;
+	//	files[i].fileSize = 0;
 
-	// Change the hierarchical layout to a flat one 
-	for (i=0; i<=maxLevel; i++)
-	{
-		clusterNum2 = clusterNum;
-		for (j=0; j<numFiles; j++)
-		{
-			if (fileLink[j].parent == i)
-			{
-				if (dirEntryPtr[clusterNum] == NULL)
-					dirEntryPtr[clusterNum] = &dirEntries[k];
-				dirEntryLink[k] = fileLink[j];
-				dirEntries[k++] = files[j];
-				if ((files[j].attrib & ATTRIB_LFN)==0)
-				{
-					if (files[j].attrib & ATTRIB_DIR)
-					{
-						if (strncmp((char*)&files[j].name[0],".       ",NAME_LEN)==0)
-							dirEntries[k-1].startCluster = dirEntryLink[k-1].level; 
-						else
-							if (strncmp((char*)&files[j].name[0],"..      ",NAME_LEN)==0)
-							{
-								dirEntries[k-1].startCluster = dirEntryLink[k-1].parent; 
-							}
-							else
-							{
-								clust++;
-								dirEntries[k-1].startCluster = clust;
-								l = (fileLink[fileLink[j].level].filesInDir)>>8;
-								clust += l;
-								numClusters += l;
-							}
-					}
-					else
-						dirEntries[k-1].startCluster = clusterNum;
-				}
-				if (i==0) numRootFiles++;
-				dirEntriesInCluster[clusterNum]++;
-				if (dirEntriesInCluster[clusterNum]==256)
-					clusterNum++;
-			}
-		}
-		clusterNum = clusterNum2 + ((fileLink[i].filesInDir)>>8) + 1;
-		numClusters++;
-	}
+	//	fileLink[i].filesInDir = 0;
 
-	// Free the file indexing buffer
-	free(files);
-	free(fileLink);
+	//	extraDirEntries[i] = NULL;
+	//	numExtraEntries[i] = 0;
+	//}
 
-	// Set up the MBR
-	MBR.bytesPerSector = 512;
-	MBR.numFATs = 1;
-	// replaced strcpy with strncpy. It doesnt matter here, as the strings are constant
-	// but we should extingish all unrestricted strcpy,strcat from the project
-	strncpy((char*)&MBR.OEMName[0],"DESMUM",8);	
-	strncpy((char*)&MBR.fat16.fileSysType[0],"FAT16  ",8);
-	MBR.reservedSectors = SECRESV;
-	MBR.numSectors = 524288;
-	MBR.numSectorsSmall = 0;
-	MBR.sectorsPerCluster = SECPERCLUS;
-	MBR.sectorsPerFAT = SECPERFAT;
-	MBR.rootEntries = 512;
-	MBR.fat16.signature = 0xAA55;
-	MBR.mediaDesc = 1;
+	//list_files(sFlashPath.c_str());
 
-	filesysFAT = 0 + MBR.reservedSectors;
-	filesysRootDir = filesysFAT + (MBR.numFATs * MBR.sectorsPerFAT);
-	filesysData = filesysRootDir + ((MBR.rootEntries * sizeof(DIR_ENT)) / 512);
+	//k            = 0;
+	//clusterNum   = rootCluster = (SECRESV + SECPERFAT)/SECPERCLUS;
+	//numClusters  = 0;
+	//clust        = 0;
+	//numRootFiles = 0;
 
-	// Set up the cluster values for all subdirectories
-	clust = filesysData / SECPERCLUS;
-	firstDirEntCluster = clust;
-	for (i=1; i<numFiles; i++)
-	{
-		if (((dirEntries[i].attrib & ATTRIB_DIR)!=0) &&	((dirEntries[i].attrib & ATTRIB_LFN)==0))
-		{
-			if (dirEntries[i].startCluster > rootCluster)
-				dirEntries[i].startCluster += clust-rootCluster;
-		}
-	}
-	lastDirEntCluster = clust+numClusters-1;
+	//// Allocate memory to hold information about the files 
+	//dirEntries = (DIR_ENT *) malloc(numFiles*sizeof(DIR_ENT));
+	//if (dirEntries==NULL) return FALSE;
 
-	// Set up the cluster values for all files
-	clust += numClusters; //clusterNum;
-	for (i=0; i<numFiles; i++)
-	{
-		if (((dirEntries[i].attrib & ATTRIB_DIR)==0) &&	((dirEntries[i].attrib & ATTRIB_LFN)==0))
-		{
-			dirEntries[i].startCluster = clust;
-			clust += (dirEntries[i].fileSize/(512*SECPERCLUS));
-			clust++;
-		}
-	}
-	lastFileDataCluster = clust-1;
+	//dirEntryLink = (FILE_INFO *) malloc(numFiles*sizeof(FILE_INFO));
+	//if (dirEntryLink==NULL)
+	//{
+	//	free(dirEntries);
+	//	return FALSE;
+	//}
+	//dirEntriesInCluster = (int *) malloc(NUMCLUSTERS*sizeof(int));
+	//if (dirEntriesInCluster==NULL)
+	//{
+	//	free(dirEntries);
+	//	free(dirEntryLink);
+	//	return FALSE;
+	//}
+	//dirEntryPtr = (DIR_ENT **) malloc(NUMCLUSTERS*sizeof(DIR_ENT*));
+	//if (dirEntryPtr==NULL)
+	//{
+	//	free(dirEntries);
+	//	free(dirEntryLink);
+	//	free(dirEntriesInCluster);
+	//	return FALSE;
+	//}
 
-	// Set up the FAT16
-	memset(FAT16,0,SECPERFAT*256*sizeof(u16));
-	FAT16[0] = 0xFF01;
-	FAT16[1] = 0xFFFF;
-	for (i=2; i<=lastDirEntCluster; i++)
-	FAT16[i] = 0xFFFF;
-	k = 2;
-	for (i=0; i<numFiles; i++)
-	{
-		if (((dirEntries[i].attrib & ATTRIB_LFN)==0) &&	(dirEntries[i].name[0] != FILE_FREE))
-		{
-			j = 0;
-			l = dirEntries[i].fileSize - (512*SECPERCLUS);
-			while (l > 0)
-			{
-				if (dirEntries[i].startCluster+j < MAXFILES)
-					FAT16[dirEntries[i].startCluster+j] = dirEntries[i].startCluster+j+1;
-				j++;
-				l -= (512*16);
-			} 
-			if ((dirEntries[i].attrib & ATTRIB_DIR)==0)
-			{
-				if (dirEntries[i].startCluster+j < MAXFILES)
-				FAT16[dirEntries[i].startCluster+j] = 0xFFFF;
-			}
-			k = dirEntries[i].startCluster+j;
-		}
-	}
+	//memset(dirEntriesInCluster, 0, NUMCLUSTERS*sizeof(int));
+	//memset(dirEntryPtr, NULL, NUMCLUSTERS*sizeof(DIR_ENT*));
 
-	for (i=(filesysData/SECPERCLUS); i<NUMCLUSTERS; i++)
-	{
-		if (dirEntriesInCluster[i]==256)
-			FAT16[i] = i+1;
-	}
+	//// Change the hierarchical layout to a flat one 
+	//for (i=0; i<=maxLevel; i++)
+	//{
+	//	clusterNum2 = clusterNum;
+	//	for (j=0; j<numFiles; j++)
+	//	{
+	//		if (fileLink[j].parent == i)
+	//		{
+	//			if (dirEntryPtr[clusterNum] == NULL)
+	//				dirEntryPtr[clusterNum] = &dirEntries[k];
+	//			dirEntryLink[k] = fileLink[j];
+	//			dirEntries[k++] = files[j];
+	//			if ((files[j].attrib & ATTRIB_LFN)==0)
+	//			{
+	//				if (files[j].attrib & ATTRIB_DIR)
+	//				{
+	//					if (strncmp((char*)&files[j].name[0],".       ",NAME_LEN)==0)
+	//						dirEntries[k-1].startCluster = dirEntryLink[k-1].level; 
+	//					else
+	//						if (strncmp((char*)&files[j].name[0],"..      ",NAME_LEN)==0)
+	//						{
+	//							dirEntries[k-1].startCluster = dirEntryLink[k-1].parent; 
+	//						}
+	//						else
+	//						{
+	//							clust++;
+	//							dirEntries[k-1].startCluster = clust;
+	//							l = (fileLink[fileLink[j].level].filesInDir)>>8;
+	//							clust += l;
+	//							numClusters += l;
+	//						}
+	//				}
+	//				else
+	//					dirEntries[k-1].startCluster = clusterNum;
+	//			}
+	//			if (i==0) numRootFiles++;
+	//			dirEntriesInCluster[clusterNum]++;
+	//			if (dirEntriesInCluster[clusterNum]==256)
+	//				clusterNum++;
+	//		}
+	//	}
+	//	clusterNum = clusterNum2 + ((fileLink[i].filesInDir)>>8) + 1;
+	//	numClusters++;
+	//}
+
+	//// Free the file indexing buffer
+	//free(files);
+	//free(fileLink);
+
+	//// Set up the MBR
+	//MBR.bytesPerSector = 512;
+	//MBR.numFATs = 1;
+	//// replaced strcpy with strncpy. It doesnt matter here, as the strings are constant
+	//// but we should extingish all unrestricted strcpy,strcat from the project
+	//strncpy((char*)&MBR.OEMName[0],"DESMUM",8);	
+	//strncpy((char*)&MBR.fat16.fileSysType[0],"FAT16  ",8);
+	//MBR.reservedSectors = SECRESV;
+	//MBR.numSectors = 524288;
+	//MBR.numSectorsSmall = 0;
+	//MBR.sectorsPerCluster = SECPERCLUS;
+	//MBR.sectorsPerFAT = SECPERFAT;
+	//MBR.rootEntries = 512;
+	//MBR.fat16.signature = 0xAA55;
+	//MBR.mediaDesc = 1;
+
+	//filesysFAT = 0 + MBR.reservedSectors;
+	//filesysRootDir = filesysFAT + (MBR.numFATs * MBR.sectorsPerFAT);
+	//filesysData = filesysRootDir + ((MBR.rootEntries * sizeof(DIR_ENT)) / 512);
+
+	//// Set up the cluster values for all subdirectories
+	//clust = filesysData / SECPERCLUS;
+	//firstDirEntCluster = clust;
+	//for (i=1; i<numFiles; i++)
+	//{
+	//	if (((dirEntries[i].attrib & ATTRIB_DIR)!=0) &&	((dirEntries[i].attrib & ATTRIB_LFN)==0))
+	//	{
+	//		if (dirEntries[i].startCluster > rootCluster)
+	//			dirEntries[i].startCluster += clust-rootCluster;
+	//	}
+	//}
+	//lastDirEntCluster = clust+numClusters-1;
+
+	//// Set up the cluster values for all files
+	//clust += numClusters; //clusterNum;
+	//for (i=0; i<numFiles; i++)
+	//{
+	//	if (((dirEntries[i].attrib & ATTRIB_DIR)==0) &&	((dirEntries[i].attrib & ATTRIB_LFN)==0))
+	//	{
+	//		dirEntries[i].startCluster = clust;
+	//		clust += (dirEntries[i].fileSize/(512*SECPERCLUS));
+	//		clust++;
+	//	}
+	//}
+	//lastFileDataCluster = clust-1;
+
+	//// Set up the FAT16
+	//memset(FAT16,0,SECPERFAT*256*sizeof(u16));
+	//FAT16[0] = 0xFF01;
+	//FAT16[1] = 0xFFFF;
+	//for (i=2; i<=lastDirEntCluster; i++)
+	//FAT16[i] = 0xFFFF;
+	//k = 2;
+	//for (i=0; i<numFiles; i++)
+	//{
+	//	if (((dirEntries[i].attrib & ATTRIB_LFN)==0) &&	(dirEntries[i].name[0] != FILE_FREE))
+	//	{
+	//		j = 0;
+	//		l = dirEntries[i].fileSize - (512*SECPERCLUS);
+	//		while (l > 0)
+	//		{
+	//			if (dirEntries[i].startCluster+j < MAXFILES)
+	//				FAT16[dirEntries[i].startCluster+j] = dirEntries[i].startCluster+j+1;
+	//			j++;
+	//			l -= (512*16);
+	//		} 
+	//		if ((dirEntries[i].attrib & ATTRIB_DIR)==0)
+	//		{
+	//			if (dirEntries[i].startCluster+j < MAXFILES)
+	//			FAT16[dirEntries[i].startCluster+j] = 0xFFFF;
+	//		}
+	//		k = dirEntries[i].startCluster+j;
+	//	}
+	//}
+
+	//for (i=(filesysData/SECPERCLUS); i<NUMCLUSTERS; i++)
+	//{
+	//	if (dirEntriesInCluster[i]==256)
+	//		FAT16[i] = i+1;
+	//}
 
 	return TRUE;
 }
@@ -539,25 +672,13 @@ static BOOL cflash_init()
 	{
 		sFlashPath = CFlash_Path;
 		INFO("Using CFlash disk image file %s\n", sFlashPath.c_str());
-		disk_image = OPEN_FN( sFlashPath.c_str(), OPEN_MODE);
-
-		if ( disk_image != -1)
+		file = new EMUFILE_FILE(sFlashPath.c_str(),"rb+");
+		if(file->fail())
 		{
-			file_size = LSEEK_FN( disk_image, 0, SEEK_END);
-			if (0 && file_size == -1)
-			{
-				CFLASHLOG( "Error when seeking to end of disk image" );
-			}
-			else
-			{
-				LSEEK_FN( disk_image, 0, SEEK_SET);
-				CFLASHLOG( "Disk image size = %ld (%ld sectors)\n", file_size, file_size / 512);
-				init_good = TRUE;
-			}
+			INFO("Failed to open file %s\n", sFlashPath.c_str());
+			delete file;
+			file = NULL;
 		}
-		else
-			// TODO: create image if not exist
-			CFLASHLOG("Failed to open file %s: \"%s\"\n", sFlashPath.c_str(), strerror( errno));
 	}
 
 	// READY
@@ -676,11 +797,6 @@ static unsigned int cflash_read(unsigned int address)
 {
 	unsigned int ret_value = 0;
 	size_t elems_read = 0;
-#if 0 /* used by next if 0 block */
-#define BUFFERED_BLOCK_SIZE 512
-	static u8 block_buffer[BUFFERED_BLOCK_SIZE];
-	static s32 buffered_start_index = -1;
-#endif
 
 	switch (address)
 	{
@@ -691,126 +807,16 @@ static unsigned int cflash_read(unsigned int address)
 		case CF_REG_DATA:
 			if (cf_reg_cmd == CF_CMD_READ)
 			{
-				if (!CFlash_IsUsingPath())
+				if(file)
 				{
-					if ( disk_image != -1)
-					{
-						u8 data[2];
-#if 0
-						if (currLBA < buffered_start_index || currLBA >= (buffered_start_index + BUFFERED_BLOCK_SIZE))
-						{
-							size_t read_bytes = 0;
-							LSEEK_FN( disk_image, currLBA, SEEK_SET);
-
-							while (read_bytes < BUFFERED_BLOCK_SIZE)
-							{
-								size_t cur_read = READ_FN( disk_image, &block_buffer[read_bytes],
-									BUFFERED_BLOCK_SIZE - read_bytes);
-
-								if ( cur_read == -1)
-								{
-									CFLASHLOG( "Error during read: %s\n", strerror(errno) );
-									break;
-								}
-								read_bytes += cur_read;
-							}
-
-							CFLASHLOG( "Read %d bytes\n", read_bytes);
-
-							buffered_start_index = currLBA;
-						}
-						data[0] = block_buffer[currLBA - buffered_start_index];
-						data[1] = block_buffer[currLBA + 1 - buffered_start_index];
-#else
-						LSEEK_FN( disk_image, currLBA, SEEK_SET);
-						elems_read += READ_FN( disk_image, data, 2);
-#endif
-						ret_value = data[1] << 8 | data[0];
-					}
-					currLBA += 2;
-				}
-				else		// use path
-				{
-					unsigned char *p;
-					int i;
-					u32 cluster,cluster2,cluster3,fileLBA;
-					cluster = (currLBA / (512 * SECPERCLUS));
-					cluster2 = (((currLBA/512) - filesysData) / SECPERCLUS) + 2;
-
-					// Reading from the MBR 
-					if (currLBA < 512)
-					{
-						p = (unsigned char*)&MBR;
-						ret_value = T1ReadWord(p, currLBA);
-
-						// Reading the FAT 
-					}
-					else
-						if (((u32)currLBA >= filesysFAT*512) && ((u32)currLBA < filesysRootDir*512))
-						{
-							p = (unsigned char*)&FAT16[0];
-							ret_value = T1ReadWord(p, currLBA - filesysFAT * 512);
-
-							// Reading directory entries 
-						}
-						else
-							if (((u32)currLBA >= filesysRootDir*512) &&	(cluster <= (u32)lastDirEntCluster))
-							{
-								cluster3 = ((currLBA - (SECRESV * 512)) / (512 * SECPERCLUS));
-								i = (currLBA-(((cluster3-(filesysRootDir/SECPERCLUS))*SECPERCLUS+filesysRootDir)*512)); //(currLBA - cluster*BYTESPERCLUS);
-								if (i < (dirEntriesInCluster[cluster3]*32))
-								{
-									p = (unsigned char*)dirEntryPtr[cluster3];
-									ret_value = T1ReadWord(p, i);
-								}
-								else
-								{
-									i /= 32;
-									i -= dirEntriesInCluster[cluster3];
-									if ((i>=0)&&(i<numExtraEntries[cluster3])) 
-									{
-										p = (unsigned char*)extraDirEntries[cluster3];
-										ret_value = T1ReadWord(p, i * 32 + (currLBA & 0x1F));
-									}
-									else 
-										if ((currLBA&0x1F)==0)
-											ret_value = FILE_FREE;
-										else
-											ret_value = 0;
-								}
-								// Reading file data 
-							}
-							else
-								if ((cluster2 > (u32)lastDirEntCluster) && (cluster2 <= (u32)lastFileDataCluster)) 
-								{
-									//else if ((cluster>lastDirEntCluster)&&(cluster<=lastFileDataCluster)) {
-									fileLBA = currLBA - (filesysData-32)*512;	// 32 = # sectors used for the root entries
-
-									// Check if the read is from the currently opened file 
-									if ((fileLBA >= fileStartLBA) && (fileLBA < fileEndLBA))
-									{
-										cluster = (fileLBA / (512 * SECPERCLUS));
-										ret_value = fread_buffered(activeDirEnt,cluster-dirEntries[activeDirEnt].startCluster,(fileLBA-fileStartLBA)&(BYTESPERCLUS-1)); 
-									}
-									else
-									{
-										for (i=0; i<numFiles; i++)
-										{
-											if ((fileLBA>=(u32)(dirEntries[i].startCluster*512*SECPERCLUS)) &&
-												(fileLBA <(dirEntries[i].startCluster*512*SECPERCLUS)+dirEntries[i].fileSize) &&
-												((dirEntries[i].attrib & (ATTRIB_DIR|ATTRIB_LFN))==0))
-											{
-												cluster = (fileLBA / (512 * SECPERCLUS));
-												ret_value = fread_buffered(i,cluster-dirEntries[i].startCluster,fileLBA&(BYTESPERCLUS-1));
-												break;
-											}
-										}
-									}
-								}
-							currLBA += 2;
+					u8 data[2];
+					file->fseek(currLBA, SEEK_SET);
+					elems_read += file->fread(data,2);
+					ret_value = data[1] << 8 | data[0];
 				}
+				currLBA += 2;
 			}
-	break;
+		break;
 
 	case CF_REG_CMD:
 	break;
@@ -837,7 +843,6 @@ static void cflash_write(unsigned int address,unsigned int data)
 		case CF_REG_DATA:
 			if (cf_reg_cmd == CF_CMD_WRITE)
 			{
-				if (!CFlash_IsUsingPath()) 
 				{
 					sector_data[sector_write_index] = (data >> 0) & 0xff;
 					sector_data[sector_write_index + 1] = (data >> 8) & 0xff;
@@ -849,22 +854,21 @@ static void cflash_write(unsigned int address,unsigned int data)
 						CFLASHLOG( "Write sector to %ld\n", currLBA);
 						size_t written = 0;
 
-						if (currLBA + 512 < file_size) 
-						{
-							if (disk_image != -1) 
+						//TODO - calling size() every time we write something is sort of unnecessarily slow in the case of disk-backed files...
+						if(file) 
+							if(currLBA + 512 < file->size()) 
 							{
-								LSEEK_FN( disk_image, currLBA, SEEK_SET);
+								file->fseek(currLBA,SEEK_SET);
 				      
 								while(written < 512) 
 								{
-									size_t cur_write =
-										WRITE_FN( disk_image, &sector_data[written], 512 - written);
+									size_t todo = 512-written;
+									file->fwrite(&sector_data[written], todo);
+									size_t cur_write = todo;
 									written += cur_write;
-
 									if ( cur_write == (size_t)-1) break;
 								}
 							}
-						}
 
 						CFLASHLOG("Wrote %u bytes\n", written);
 					
@@ -872,9 +876,6 @@ static void cflash_write(unsigned int address,unsigned int data)
 						sector_write_index = 0;
 					}
 				}
-				else		// TODO: write to real partition
-				{
-				}
 			}
 		break;
 
@@ -916,11 +917,8 @@ static void cflash_close( void)
 	if (!inited) return;
 	if (!CFlash_IsUsingPath())
 	{
-		if (disk_image != -1)
-		{
-			CLOSE_FN(disk_image);
-			disk_image = -1;
-		}
+		delete file;
+		file = NULL;
 	}
 	else
 	{
