@@ -22,6 +22,7 @@
 #include "../mem.h"
 #include <string.h>
 #include "../MMU.h"
+#include "path.h"
 
 //SRAM is going to be stored just above the rom.
 //that is convenient for us, since it mirrors the nds memory map
@@ -218,9 +219,9 @@ static u8 getSaveTypeGBA(const u8 *data, const u32 size)
 {
 	u8	*dat = (u8 *)data;
 	
-	for (u32 i = 0; i < (size / 4); i++)
+	for (u32 i = 0; i < size; i++)
 	{
-		u32	tmp = T1ReadLong(dat, i);
+		u32	tmp = T1ReadLong(dat, 0);
 
 		if (tmp == 0x52504545)
 		{
@@ -258,7 +259,7 @@ static u8 getSaveTypeGBA(const u8 *data, const u32 size)
 		{
 			return 5;
 		}
-		dat += 4;
+		dat++;
 	}
 
 	return 0xFF;		// NONE
@@ -292,6 +293,12 @@ static void GBAgame_reset(void)
 	if (!strlen(GBAgameName)) return;
 	FILE *fgame = 0;
 
+	//perk: if the gbagame name is "self" this is a special indicator that we should mount the main rom that we're running as the gba game
+	if(!strcasecmp(GBAgameName,"self"))
+	{
+		strcpy(GBAgameName,path.path.c_str());
+	}
+
 	fgame = fopen(GBAgameName,"rb");
 	if (!fgame) return;
 	fseek(fgame, 0, SEEK_END);
@@ -391,7 +398,7 @@ static void GBAgame_write32(u32 adr, u32 val)
 static u8   GBAgame_read08(u32 adr)
 { 
 	//INFO("GBAgame: read08 at 0x%08X value 0x%02X\n", adr, (u8)T1ReadByte(GBArom, (adr - 0x08000000)));
-	
+
 	if (adr < 0x0A000000)
 		return (u8)T1ReadByte(GBArom, (adr - 0x08000000));
 
