@@ -28,9 +28,9 @@ static void None_config(void) {}
 static void None_write08(u32 adr, u8 val) {}
 static void None_write16(u32 adr, u16 val) {}
 static void None_write32(u32 adr, u32 val) {}
-static u8   None_read08(u32 adr){ return (0); }
-static u16  None_read16(u32 adr){ return (0); }
-static u32  None_read32(u32 adr){ return (0); }
+static u8   None_read08(u32 adr){ return (0xFF); }
+static u16  None_read16(u32 adr){ return (0xFFFF); }
+static u32  None_read32(u32 adr){ return (0xFFFFFFFF); }
 static void None_info(char *info) { strcpy(info, "Nothing in GBA slot"); }
 
 ADDONINTERFACE addonNone = {
