@@ -1,21 +1,18 @@
 /*  Copyright (C) 2006 yopyop
-	Copyright (C) 2009 DeSmuME team
+	Copyright (C) 2009-2010 DeSmuME team
 
-    This file is part of DeSmuME
+	This file is free software: you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation, either version 3 of the License, or
+	(at your option) any later version.
 
-    DeSmuME is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+	This file is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
 
-    DeSmuME is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with DeSmuME; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+	You should have received a copy of the GNU General Public License
+	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <stdlib.h>
@@ -141,23 +138,20 @@ remove_post_exec_fn( void *instance) {
 #endif
 
 #ifdef GDB_STUB
-static u32
-read_cpu_reg( void *instance, u32 reg_num) {
-  armcpu_t *armcpu = (armcpu_t *)instance;
-  u32 reg_value = 0;
+static u32 read_cpu_reg( void *instance, u32 reg_num)
+{
+	armcpu_t *armcpu = (armcpu_t *)instance;
 
-  if ( reg_num <= 14) {
-    reg_value = armcpu->R[reg_num];
-  }
-  else if ( reg_num == 15) {
-    reg_value = armcpu->next_instruction;
-  }
-  else if ( reg_num == 16) {
-    /* CPSR */
-    reg_value = armcpu->CPSR.val;
-  }
-
-  return reg_value;
+	if ( reg_num <= 14) {
+	  return armcpu->R[reg_num];
+	}
+	else if ( reg_num == 15) {
+	  return armcpu->instruct_adr;
+	}
+	else if ( reg_num == 16) {
+	  //CPSR
+	  return armcpu->CPSR.val;
+	}
 }
 
 static void
@@ -214,29 +208,29 @@ int armcpu_new( armcpu_t *armcpu, u32 id)
 void armcpu_t::changeCPSR()
 {
 	//but all it does is give them a chance to unleash by forcing an immediate reschedule
+	//TODO - we could actually set CPSR through here and look for a change in the I bit
+	//that would be a little optimization as well as a safety measure if we prevented setting CPSR directly
 	NDS_Reschedule();
 }
 
 void armcpu_init(armcpu_t *armcpu, u32 adr)
 {
-   u32 i;
-
 	armcpu->LDTBit = (armcpu->proc_ID==0); //Si ARM9 utiliser le syte v5 pour le load
 	armcpu->intVector = 0xFFFF0000 * (armcpu->proc_ID==0);
 	armcpu->waitIRQ = FALSE;
-	armcpu->wirq = FALSE;
+	armcpu->halt_IE_and_IF = FALSE;
+	armcpu->intrWaitARM_state = 0;
 
-#ifdef GDB_STUB
-    armcpu->irq_flag = 0;
-#endif
+//#ifdef GDB_STUB
+//    armcpu->irq_flag = 0;
+//#endif
 
-	if(armcpu->coproc[15]) free(armcpu->coproc[15]);
-	
-   for(i = 0; i < 15; ++i)
+	for(int i = 0; i < 16; ++i)
 	{
 		armcpu->R[i] = 0;
+		if(armcpu->coproc[i]) free(armcpu->coproc[i]);
 		armcpu->coproc[i] = NULL;
-   }
+	}
 	
 	armcpu->CPSR.val = armcpu->SPSR.val = SYS;
 	
@@ -249,20 +243,22 @@ void armcpu_init(armcpu_t *armcpu, u32 adr)
 	
 	armcpu->SPSR_svc.val = armcpu->SPSR_abt.val = armcpu->SPSR_und.val = armcpu->SPSR_irq.val = armcpu->SPSR_fiq.val = 0;
 
-#ifdef GDB_STUB
-    armcpu->instruct_adr = adr;
-	armcpu->R[15] = adr + 8;
-#else
-	armcpu->R[15] = adr;
-#endif
+//#ifdef GDB_STUB
+//    armcpu->instruct_adr = adr;
+//	armcpu->R[15] = adr + 8;
+//#else
+	//armcpu->R[15] = adr;
+//#endif
 
 	armcpu->next_instruction = adr;
 	
-	armcpu->coproc[15] = (armcp_t*)armcp15_new(armcpu);
+	// only ARM9 have co-processor
+	if (armcpu->proc_ID==0)
+		armcpu->coproc[15] = (armcp_t*)armcp15_new(armcpu);
 
-#ifndef GDB_STUB
+//#ifndef GDB_STUB
 	armcpu_prefetch(armcpu);
-#endif
+//#endif
 }
 
 u32 armcpu_switchMode(armcpu_t *armcpu, u8 mode)
@@ -373,56 +369,68 @@ u32 armcpu_switchMode(armcpu_t *armcpu, u8 mode)
 	return oldmode;
 }
 
+u32 armcpu_Wait4IRQ(armcpu_t *cpu)
+{
+	cpu->waitIRQ = TRUE;
+	cpu->halt_IE_and_IF = TRUE;
+	return 1;
+}
+
 template<u32 PROCNUM>
 FORCEINLINE static u32 armcpu_prefetch()
 {
 	armcpu_t* const armcpu = &ARMPROC;
-#ifdef GDB_STUB
-	u32 temp_instruction;
-#endif
+//#ifdef GDB_STUB
+//	u32 temp_instruction;
+//#endif
+	u32 curInstruction = armcpu->next_instruction;
 
 	if(armcpu->CPSR.bits.T == 0)
 	{
-		u32 curInstruction = armcpu->next_instruction;
-#ifdef GDB_STUB
-		temp_instruction =
-			armcpu->mem_if->prefetch32( armcpu->mem_if->data,
-			armcpu->next_instruction);
-
-		if ( !armcpu->stalled) {
-			armcpu->instruction = temp_instruction;
-			armcpu->instruct_adr = armcpu->next_instruction;
-			armcpu->next_instruction += 4;
-			armcpu->R[15] = armcpu->next_instruction + 4;
-		}
-#else
-		armcpu->instruction = _MMU_read32<PROCNUM,MMU_AT_CODE>(curInstruction&0xFFFFFFFC);
+//#ifdef GDB_STUB
+//		temp_instruction =
+//			armcpu->mem_if->prefetch32( armcpu->mem_if->data,
+//			armcpu->next_instruction);
+//
+//		if ( !armcpu->stalled) {
+//			armcpu->instruction = temp_instruction;
+//			armcpu->instruct_adr = armcpu->next_instruction;
+//			armcpu->next_instruction += 4;
+//			armcpu->R[15] = armcpu->next_instruction + 4;
+//		}
+//#else
+		curInstruction &= 0xFFFFFFFC; //please don't change this to 0x0FFFFFFC -- the NDS will happily run on 0xF******* addresses all day long
+		//please note that we must setup R[15] before reading the instruction since there is a protection
+		//which prevents PC > 0x3FFF from reading the bios region
 		armcpu->instruct_adr = curInstruction;
 		armcpu->next_instruction = curInstruction + 4;
 		armcpu->R[15] = curInstruction + 8;
-#endif
+		armcpu->instruction = _MMU_read32<PROCNUM, MMU_AT_CODE>(curInstruction);
+//#endif
 
 		return MMU_codeFetchCycles<PROCNUM,32>(curInstruction);
 	}
 
-	u32 curInstruction = armcpu->next_instruction;
-#ifdef GDB_STUB
-	temp_instruction =
-		armcpu->mem_if->prefetch16( armcpu->mem_if->data,
-		armcpu->next_instruction);
-
-	if ( !armcpu->stalled) {
-		armcpu->instruction = temp_instruction;
-		armcpu->instruct_adr = armcpu->next_instruction;
-		armcpu->next_instruction = armcpu->next_instruction + 2;
-		armcpu->R[15] = armcpu->next_instruction + 2;
-	}
-#else
-	armcpu->instruction = _MMU_read16<PROCNUM, MMU_AT_CODE>(curInstruction&0xFFFFFFFE);
+//#ifdef GDB_STUB
+//	temp_instruction =
+//		armcpu->mem_if->prefetch16( armcpu->mem_if->data,
+//		armcpu->next_instruction);
+//
+//	if ( !armcpu->stalled) {
+//		armcpu->instruction = temp_instruction;
+//		armcpu->instruct_adr = armcpu->next_instruction;
+//		armcpu->next_instruction = armcpu->next_instruction + 2;
+//		armcpu->R[15] = armcpu->next_instruction + 2;
+//	}
+//#else
+	curInstruction &= 0xFFFFFFFE; //please don't change this to 0x0FFFFFFE -- the NDS will happily run on 0xF******* addresses all day long
+	//please note that we must setup R[15] before reading the instruction since there is a protection
+	//which prevents PC > 0x3FFF from reading the bios region
 	armcpu->instruct_adr = curInstruction;
 	armcpu->next_instruction = curInstruction + 2;
 	armcpu->R[15] = curInstruction + 4;
-#endif
+	armcpu->instruction = _MMU_read16<PROCNUM, MMU_AT_CODE>(curInstruction);
+//#endif
 
 	if(PROCNUM==0)
 	{
@@ -467,52 +475,115 @@ static BOOL (FASTCALL* test_conditions[])(Status_Reg CPSR)= {
 	(cond<15&&test_conditions[cond](CPSR))
 #endif
 
+//TODO - merge with armcpu_irqException?
+//http://www.ethernut.de/en/documents/arm-exceptions.html
+//http://docs.google.com/viewer?a=v&q=cache:V4ht1YkxprMJ:www.cs.nctu.edu.tw/~wjtsai/EmbeddedSystemDesign/Ch3-1.pdf+arm+exception+handling&hl=en&gl=us&pid=bl&srcid=ADGEEShx9VTHbUhWdDOrTVRzLkcCsVfJiijncNDkkgkrlJkLa7D0LCpO8fQ_hhU3DTcgZh9rcZWWQq4TYhhCovJ625h41M0ZUX3WGasyzWQFxYzDCB-VS6bsUmpoJnRxAc-bdkD0qmsu&sig=AHIEtbR9VHvDOCRmZFQDUVwy53iJDjoSPQ
+void armcpu_exception(armcpu_t *cpu, u32 number)
+{
+	Mode cpumode = USR;
+	switch(number)
+	{
+	case EXCEPTION_RESET: cpumode = SVC; break;
+	case EXCEPTION_UNDEFINED_INSTRUCTION: cpumode = UND; break;
+	case EXCEPTION_SWI: cpumode = SVC; break;
+	case EXCEPTION_PREFETCH_ABORT: cpumode = ABT; break;
+	case EXCEPTION_DATA_ABORT: cpumode = ABT; break;
+	case EXCEPTION_RESERVED_0x14: emu_halt(); break;
+	case EXCEPTION_IRQ: cpumode = IRQ; break;
+	case EXCEPTION_FAST_IRQ: cpumode = FIQ; break;
+	}
+
+	Status_Reg tmp = cpu->CPSR;
+	armcpu_switchMode(cpu, cpumode);				//enter new mode
+	cpu->R[14] = cpu->next_instruction;
+	cpu->SPSR = tmp;							//save old CPSR as new SPSR
+	cpu->CPSR.bits.T = 0;						//handle as ARM32 code
+	cpu->CPSR.bits.I = 1;
+	cpu->changeCPSR();
+	cpu->R[15] = cpu->intVector + number;
+	cpu->next_instruction = cpu->R[15];
+	printf("armcpu_exception!\n");
+	//extern bool dolog;
+	//dolog=true;
+
+	//HOW DOES THIS WORTK WITHOUT A PREFETCH, LIKE IRQ BELOW?
+	//I REALLY WISH WE DIDNT PREFETCH BEFORE EXECUTING
+}
 
 BOOL armcpu_irqException(armcpu_t *armcpu)
 {
     Status_Reg tmp;
 
-	if(armcpu->CPSR.bits.I) return FALSE;
-
-#ifdef GDB_STUB
-	armcpu->irq_flag = 0;
-#endif
+	//TODO - remove GDB specific code
+//#ifdef GDB_STUB
+//	armcpu->irq_flag = 0;
+//#endif
       
 	tmp = armcpu->CPSR;
 	armcpu_switchMode(armcpu, IRQ);
 
-#ifdef GDB_STUB
-	armcpu->R[14] = armcpu->next_instruction + 4;
-#else
+	//TODO - remove GDB specific code
+//#ifdef GDB_STUB
+//	armcpu->R[14] = armcpu->next_instruction + 4;
+//#else
 	armcpu->R[14] = armcpu->instruct_adr + 4;
-#endif
+//#endif
 	armcpu->SPSR = tmp;
 	armcpu->CPSR.bits.T = 0;
 	armcpu->CPSR.bits.I = 1;
 	armcpu->next_instruction = armcpu->intVector + 0x18;
 	armcpu->waitIRQ = 0;
 
-#ifndef GDB_STUB
-	armcpu->R[15] = armcpu->next_instruction + 8;
+	//must retain invariant of having next instruction to be executed prefetched
+	//(yucky)
 	armcpu_prefetch(armcpu);
-#endif
 
 	return TRUE;
 }
 
-BOOL
-armcpu_flagIrq( armcpu_t *armcpu) {
-  if(armcpu->CPSR.bits.I) return FALSE;
+//TODO - remove GDB specific code
+//BOOL
+//armcpu_flagIrq( armcpu_t *armcpu) {
+//  if(armcpu->CPSR.bits.I) return FALSE;
+//
+//  armcpu->waitIRQ = 0;
+//
+//#ifdef GDB_STUB
+//  armcpu->irq_flag = 1;
+//#endif
+//
+//  return TRUE;
+//}
 
-  armcpu->waitIRQ = 0;
+u32 TRAPUNDEF(armcpu_t* cpu)
+{
+	INFO("ARM%c: Undefined instruction: 0x%08X (%s) PC=0x%08X\n", cpu->proc_ID?'7':'9', cpu->instruction, decodeIntruction(false, cpu->instruction), cpu->instruct_adr);
 
-#ifdef GDB_STUB
-  armcpu->irq_flag = 1;
-#endif
-
-  return TRUE;
+	if (((cpu->intVector != 0) ^ (cpu->proc_ID == ARMCPU_ARM9)))
+	{
+		armcpu_exception(&NDS_ARM9,0x04);
+		return 4;
+	}
+	else
+	{
+		emu_halt();
+		return 4;
+	}
 }
 
+//BOOL
+//armcpu_flagIrq( armcpu_t *armcpu) {
+//  if(armcpu->CPSR.bits.I) return FALSE;
+//
+//  armcpu->waitIRQ = 0;
+//
+//#ifdef GDB_STUB
+//  armcpu->irq_flag = 1;
+//#endif
+//
+//  return TRUE;
+//}
+
 template<int PROCNUM>
 u32 armcpu_exec()
 {
@@ -524,8 +595,46 @@ u32 armcpu_exec()
 
 	//this assert is annoying. but sometimes it is handy.
 	//assert(ARMPROC.instruct_adr!=0x00000000);
+//#ifdef DEVELOPER
+#if 0
+	if ((((ARMPROC.instruct_adr & 0x0F000000) == 0x0F000000) && (PROCNUM == 0)) ||
+		(((ARMPROC.instruct_adr & 0x0F000000) == 0x00000000) && (PROCNUM == 1)))
+	{
+		switch (ARMPROC.instruct_adr & 0xFFFF)
+		{
+			case 0x00000000:
+				printf("BIOS%c: Reset!!!\n", PROCNUM?'7':'9');
+				emu_halt();
+				break;
+			case 0x00000004:
+				printf("BIOS%c: Undefined instruction\n", PROCNUM?'7':'9');
+				//emu_halt();
+				break;
+			case 0x00000008:
+				//printf("BIOS%c: SWI\n", PROCNUM?'7':'9');
+				break;
+			case 0x0000000C:
+				printf("BIOS%c: Prefetch Abort!!!\n", PROCNUM?'7':'9');
+				//emu_halt();
+				break;
+			case 0x00000010:
+				//printf("BIOS%c: Data Abort!!!\n", PROCNUM?'7':'9');
+				//emu_halt();
+				break;
+			case 0x00000014:
+				printf("BIOS%c: Reserved!!!\n", PROCNUM?'7':'9');
+				break;
+			case 0x00000018:
+				//printf("BIOS%c: IRQ\n", PROCNUM?'7':'9');
+				break;
+			case 0x0000001C:
+				printf("BIOS%c: Fast IRQ\n", PROCNUM?'7':'9');
+				break;
+		}
+	}
+#endif
 
-#ifdef GDB_STUB
+#if 0 //#ifdef GDB_STUB
 	if (ARMPROC.stalled) {
 		return STALLED_CYCLE_COUNT;
 	}
@@ -542,6 +651,10 @@ u32 armcpu_exec()
 	}
 #endif
 
+	//cFetch = armcpu_prefetch(&ARMPROC);
+
+	//printf("%d: %08X\n",PROCNUM,ARMPROC.instruct_adr);
+
 	if(ARMPROC.CPSR.bits.T == 0)
 	{
 		if(
@@ -572,9 +685,9 @@ u32 armcpu_exec()
 			/* call the external post execute function */
 			ARMPROC.post_ex_fn(ARMPROC.post_ex_fn_data, ARMPROC.instruct_adr, 0);
 		}
-#else
-		cFetch = armcpu_prefetch<PROCNUM>();
+		ARMPROC.mem_if->prefetch32( ARMPROC.mem_if->data, ARMPROC.next_instruction);
 #endif
+		cFetch = armcpu_prefetch<PROCNUM>();
 		return MMU_fetchExecuteCycles<PROCNUM>(cExecute, cFetch);
 	}
 
@@ -600,9 +713,9 @@ u32 armcpu_exec()
 		/* call the external post execute function */
 		ARMPROC.post_ex_fn( ARMPROC.post_ex_fn_data, ARMPROC.instruct_adr, 1);
 	}
-#else
-	cFetch = armcpu_prefetch<PROCNUM>();
+	ARMPROC.mem_if->prefetch32( ARMPROC.mem_if->data, ARMPROC.next_instruction);
 #endif
+	cFetch = armcpu_prefetch<PROCNUM>();
 	return MMU_fetchExecuteCycles<PROCNUM>(cExecute, cFetch);
 }
 
