@@ -1,21 +1,18 @@
 /*  Copyright (C) 2006 yopyop
 	Copyright (C) 2008-2010 DeSmuME team
 
-    This file is part of DeSmuME
+	This file is free software: you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation, either version 3 of the License, or
+	(at your option) any later version.
 
-    DeSmuME is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+	This file is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
 
-    DeSmuME is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with DeSmuME; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
+	You should have received a copy of the GNU General Public License
+	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #ifndef ARM_CPU
@@ -24,11 +21,21 @@
 #include "types.h"
 #include "bits.h"
 #include "MMU.h"
+#include "common.h"
 
 #define CODE(i)     (((i)>>25)&0x7)
 #define OPCODE(i)   (((i)>>21)&0xF)
 #define SIGNEBIT(i) BIT_N(i,20)
 
+#define EXCEPTION_RESET 0x00
+#define EXCEPTION_UNDEFINED_INSTRUCTION 0x04
+#define EXCEPTION_SWI 0x08
+#define EXCEPTION_PREFETCH_ABORT 0x0C
+#define EXCEPTION_DATA_ABORT 0x10
+#define EXCEPTION_RESERVED_0x14 0x14
+#define EXCEPTION_IRQ 0x18
+#define EXCEPTION_FAST_IRQ 0x1C
+
 #define INSTRUCTION_INDEX(i) ((((i)>>16)&0xFF0)|(((i)>>4)&0xF))
 
 inline u32 ROR(u32 i, u32 j)   { return ((((u32)(i))>>(j)) | (((u32)(i))<<(32-(j)))); }
@@ -45,6 +52,31 @@ inline T SIGNED_OVERFLOW(T a,T b,T c) { return BIT31(((a)&(b)&(~c)) | ((~a)&(~(b
 template<typename T>
 inline T SIGNED_UNDERFLOW(T a,T b,T c) { return BIT31(((a)&(~(b))&(~c)) | ((~a)&(b)&(c))); }
 
+// ============================= CPRS flags funcs
+inline bool CarryFrom(s32 left, s32 right)
+{
+  u32 res  = (0xFFFFFFFFU - (u32)left);
+
+  return ((u32)right > res);
+}
+
+inline bool BorrowFrom(s32 left, s32 right)
+{
+  return ((u32)right > (u32)left);
+}
+
+inline bool OverflowFromADD(s32 alu_out, s32 left, s32 right)
+{
+    return ((left >= 0 && right >= 0) || (left < 0 && right < 0))
+			&& ((left < 0 && alu_out >= 0) || (left >= 0 && alu_out < 0));
+}
+
+inline bool OverflowFromSUB(s32 alu_out, s32 left, s32 right)
+{
+    return ((left < 0 && right >= 0) || (left >= 0 && right < 0))
+			&& ((left < 0 && alu_out >= 0) || (left >= 0 && alu_out < 0));
+}
+
 //zero 15-feb-2009 - these werent getting used and they were getting in my way
 //#define EQ	0x0
 //#define NE	0x1
@@ -173,7 +205,10 @@ struct armcpu_t
 	u32 intVector;
 	u8 LDTBit;  //1 : ARMv5 style 0 : non ARMv5
 	BOOL waitIRQ;
-	BOOL wirq;
+	BOOL halt_IE_and_IF; //the cpu is halted, waiting for IE&IF to signal something
+	u8 intrWaitARM_state;
+
+	BOOL BIOS_loaded;
 
 	u32 (* *swi_tab)();
 
@@ -214,6 +249,9 @@ template<int PROCNUM> u32 armcpu_exec();
 
 BOOL armcpu_irqException(armcpu_t *armcpu);
 BOOL armcpu_flagIrq( armcpu_t *armcpu);
+void armcpu_exception(armcpu_t *cpu, u32 number);
+u32 TRAPUNDEF(armcpu_t* cpu);
+u32 armcpu_Wait4IRQ(armcpu_t *cpu);
 
 extern armcpu_t NDS_ARM7;
 extern armcpu_t NDS_ARM9;
@@ -221,42 +259,35 @@ extern armcpu_t NDS_ARM9;
 
 static INLINE void setIF(int PROCNUM, u32 flag)
 {
-	MMU.reg_IF[PROCNUM] |= flag;
+	//don't set generated bits!!!
+	assert(!(flag&0x00260000));
+
+	MMU.reg_IF_bits[PROCNUM] |= flag;
 
 	extern void NDS_Reschedule();
 	NDS_Reschedule();
+}
 
-    //generate the interrupt if enabled
-	if ((MMU.reg_IE[PROCNUM] & (flag)) && MMU.reg_IME[PROCNUM])
+static INLINE void NDS_makeIrq(int PROCNUM, u32 num)
+{
+	setIF(PROCNUM,1<<num);
+}
+
+static INLINE char *decodeIntruction(bool thumb_mode, u32 instr)
+{
+	char txt[20] = {0};
+	u32 tmp = 0;
+	if (thumb_mode == true)
 	{
-		if(PROCNUM==0)
-			NDS_ARM9.waitIRQ = FALSE;
-		else 
-			NDS_ARM7.waitIRQ = FALSE;
+		tmp = (instr >> 6);
+		strcpy(txt, intToBin((u16)tmp)+6);
 	}
-}
-
-static INLINE void NDS_makeARM9Int(u32 num)
-{
-	setIF(0, (1<<num));
-}
-
-static INLINE void NDS_makeARM7Int(u32 num)
-{
-	setIF(1, (1<<num));
-}
-
-static INLINE void NDS_makeInt(u8 proc_ID,u32 num)
-{
-	switch (proc_ID)
+	else
 	{
-		case 0:
-			NDS_makeARM9Int(num) ;
-			break ;
-		case 1:
-			NDS_makeARM7Int(num) ;
-			break ;
+		tmp = ((instr >> 16) & 0x0FF0) | ((instr >> 4) & 0x0F);
+		strcpy(txt, intToBin((u32)tmp)+20);
 	}
+	return strdup(txt);
 }
 
 #endif
