@@ -1,8 +1,4 @@
-/*  Copyright (C) 2006 yopyop
-    yopyop156@ifrance.com
-    yopyop156.ifrance.com
-
-	Copyright 2009 DeSmuME team
+/*  Copyright 2009-2010 DeSmuME team
 
     This file is part of DeSmuME
 
@@ -27,16 +23,18 @@
 #include "mem.h"
 #include "MMU.h"
 #include "debug.h"
+#include "utils/xstring.h"
+
+#ifndef _MSC_VER
+#include <stdint.h>
+#endif
 
 CHEATS *cheats = NULL;
 CHEATSEARCH *cheatSearch = NULL;
 
 void CHEATS::clear()
 {
-	memset(list, 0, sizeof(list));
-	for (int i = 0; i < MAX_CHEAT_LIST; i++)
-		list[i].type = 0xFF;
-	num = 0;
+	list.resize(0);
 	currentGet = 0;
 }
 
@@ -45,15 +43,13 @@ void CHEATS::init(char *path)
 	clear();
 	strcpy((char *)filename, path);
 
-	if (stack) delete [] stack;
-	stack = NULL;
-
 	load();
 }
 
 BOOL CHEATS::add(u8 size, u32 address, u32 val, char *description, BOOL enabled)
 {
-	if (num == MAX_CHEAT_LIST) return FALSE;
+	size_t num = list.size();
+	list.push_back(CHEATS_LIST());
 	list[num].code[0][0] = address & 0x00FFFFFF;
 	list[num].code[0][1] = val;
 	list[num].num = 1;
@@ -61,13 +57,12 @@ BOOL CHEATS::add(u8 size, u32 address, u32 val, char *description, BOOL enabled)
 	list[num].size = size;
 	strcpy(list[num].description, description);
 	list[num].enabled = enabled;
-	num++;
 	return TRUE;
 }
 
 BOOL CHEATS::update(u8 size, u32 address, u32 val, char *description, BOOL enabled, u32 pos)
 {
-	if (pos > num) return FALSE;
+	if (pos >= list.size()) return FALSE;
 	list[pos].code[0][0] = address & 0x00FFFFFF;
 	list[pos].code[0][1] = val;
 	list[pos].num = 1;
@@ -78,7 +73,7 @@ BOOL CHEATS::update(u8 size, u32 address, u32 val, char *description, BOOL enabl
 	return TRUE;
 }
 
-void CHEATS::ARparser(CHEATS_LIST list)
+void CHEATS::ARparser(CHEATS_LIST& list)
 {
 	u8	type = 0;
 	u8	subtype = 0;
@@ -155,6 +150,7 @@ void CHEATS::ARparser(CHEATS_LIST list)
 			break;
 
 			case 0x03:
+				if (hi == 0) hi = offset;	// V1.54+
 				val = T1ReadLong(MMU.MMU_MEM[ARMCPU_ARM9][hi>>20], hi & MMU.MMU_MASK[ARMCPU_ARM9][hi>>20]);
 				if ( lo > val )
 				{
@@ -167,6 +163,7 @@ void CHEATS::ARparser(CHEATS_LIST list)
 			break;
 
 			case 0x04:
+				if (hi == 0) hi = offset;	// V1.54+
 				val = T1ReadLong(MMU.MMU_MEM[ARMCPU_ARM9][hi>>20], hi & MMU.MMU_MASK[ARMCPU_ARM9][hi>>20]);
 				if ( lo < val )
 				{
@@ -179,6 +176,7 @@ void CHEATS::ARparser(CHEATS_LIST list)
 			break;
 
 			case 0x05:
+				if (hi == 0) hi = offset;	// V1.54+
 				val = T1ReadLong(MMU.MMU_MEM[ARMCPU_ARM9][hi>>20], hi & MMU.MMU_MASK[ARMCPU_ARM9][hi>>20]);
 				if ( lo == val )
 				{
@@ -191,6 +189,7 @@ void CHEATS::ARparser(CHEATS_LIST list)
 			break;
 
 			case 0x06:
+				if (hi == 0) hi = offset;	// V1.54+
 				val = T1ReadLong(MMU.MMU_MEM[ARMCPU_ARM9][hi>>20], hi & MMU.MMU_MASK[ARMCPU_ARM9][hi>>20]);
 				if ( lo != val )
 				{
@@ -203,6 +202,7 @@ void CHEATS::ARparser(CHEATS_LIST list)
 			break;
 
 			case 0x07:
+				if (hi == 0) hi = offset;	// V1.54+
 				val = T1ReadWord(MMU.MMU_MEM[ARMCPU_ARM9][hi>>20], hi & MMU.MMU_MASK[ARMCPU_ARM9][hi>>20]) & 0x0000FFFF;
 				if ( (lo & 0xFFFF) > ( (~(lo >> 16)) & val) )
 				{
@@ -215,6 +215,7 @@ void CHEATS::ARparser(CHEATS_LIST list)
 			break;
 
 			case 0x08:
+				if (hi == 0) hi = offset;	// V1.54+
 				val = T1ReadWord(MMU.MMU_MEM[ARMCPU_ARM9][hi>>20], hi & MMU.MMU_MASK[ARMCPU_ARM9][hi>>20]) & 0x0000FFFF;
 				if ( (lo & 0xFFFF) < ( (~(lo >> 16)) & val) )
 				{
@@ -227,6 +228,7 @@ void CHEATS::ARparser(CHEATS_LIST list)
 			break;
 
 			case 0x09:
+				if (hi == 0) hi = offset;	// V1.54+
 				val = T1ReadWord(MMU.MMU_MEM[ARMCPU_ARM9][hi>>20], hi & MMU.MMU_MASK[ARMCPU_ARM9][hi>>20]);
 				if ( (lo & 0xFFFF) == ( (~(lo >> 16)) & val) )
 				{
@@ -239,6 +241,7 @@ void CHEATS::ARparser(CHEATS_LIST list)
 			break;
 
 			case 0x0A:
+				if (hi == 0) hi = offset;	// V1.54+
 				val = T1ReadWord(MMU.MMU_MEM[ARMCPU_ARM9][hi>>20], hi & MMU.MMU_MASK[ARMCPU_ARM9][hi>>20]) & 0x0000FFFF;
 				if ( (lo & 0xFFFF) != ( (~(lo >> 16)) & val) )
 				{
@@ -417,11 +420,11 @@ BOOL CHEATS::XXcodePreParser(CHEATS_LIST *list, char *code)
 	count = (strlen(tmp_buf) / 16);
 	for (int i=0; i < count; i++)
 	{
-		char buf[8] = {0};
-		strncpy(buf, tmp_buf+(i*16), 8);
-		sscanf_s(buf, "%x", &list->code[i][0]);
-		strncpy(buf, tmp_buf+(i*16) + 8, 8);
-		sscanf_s(buf, "%x", &list->code[i][1]);
+		char buf[9] = {0};
+		memcpy(buf, tmp_buf+(i*16), 8);
+		sscanf(buf, "%x", &list->code[i][0]);
+		memcpy(buf, tmp_buf+(i*16) + 8, 8);
+		sscanf(buf, "%x", &list->code[i][1]);
 	}
 	
 	list->num = count;
@@ -429,23 +432,34 @@ BOOL CHEATS::XXcodePreParser(CHEATS_LIST *list, char *code)
 	return TRUE;
 }
 
+BOOL CHEATS::add_AR_Direct(CHEATS_LIST cheat)
+{
+	size_t num = list.size();
+	list.push_back(cheat);
+	list[num].type = 1;
+	return TRUE;
+}
+
 BOOL CHEATS::add_AR(char *code, char *description, BOOL enabled)
 {
-	if (num == MAX_CHEAT_LIST) return FALSE;
+	//if (num == MAX_CHEAT_LIST) return FALSE;
+	size_t num = list.size();
 
-	if (!XXcodePreParser(&list[num], code)) return FALSE;
+	CHEATS_LIST temp;
+	if (!XXcodePreParser(&temp, code)) return FALSE;
+
+	list.push_back(temp);
 	
 	list[num].type = 1;
 	
 	strcpy(list[num].description, description);
 	list[num].enabled = enabled;
-	num++;
 	return TRUE;
 }
 
 BOOL CHEATS::update_AR(char *code, char *description, BOOL enabled, u32 pos)
 {
-	if (pos > num) return FALSE;
+	if (pos >= list.size()) return FALSE;
 
 	if (code != NULL)
 	{
@@ -460,7 +474,8 @@ BOOL CHEATS::update_AR(char *code, char *description, BOOL enabled, u32 pos)
 
 BOOL CHEATS::add_CB(char *code, char *description, BOOL enabled)
 {
-	if (num == MAX_CHEAT_LIST) return FALSE;
+	//if (num == MAX_CHEAT_LIST) return FALSE;
+	size_t num = list.size();
 
 	if (!XXcodePreParser(&list[num], code)) return FALSE;
 	
@@ -468,13 +483,12 @@ BOOL CHEATS::add_CB(char *code, char *description, BOOL enabled)
 	
 	strcpy(list[num].description, description);
 	list[num].enabled = enabled;
-	num++;
 	return TRUE;
 }
 
 BOOL CHEATS::update_CB(char *code, char *description, BOOL enabled, u32 pos)
 {
-	if (pos > num) return FALSE;
+	if (pos >= list.size()) return FALSE;
 
 	if (code != NULL)
 	{
@@ -488,15 +502,11 @@ BOOL CHEATS::update_CB(char *code, char *description, BOOL enabled, u32 pos)
 
 BOOL CHEATS::remove(u32 pos)
 {
-	if (pos > num) return FALSE;
-	if (num == 0) return FALSE;
+	if (pos >= list.size()) return FALSE;
+	if (list.size() == 0) return FALSE;
 
-	for (int i = pos; i < num; i++)
-		memcpy(&list[i], &list[i+1], sizeof(CHEATS_LIST));
+	list.erase(list.begin()+pos);
 
-	memset(&list[num], 0, sizeof(CHEATS_LIST));
-
-	num--;
 	return TRUE;
 }
 
@@ -508,13 +518,14 @@ void CHEATS::getListReset()
 
 BOOL CHEATS::getList(CHEATS_LIST *cheat)
 {
-	if (currentGet > num) 
+	if (currentGet >= list.size()) 
 	{
 		currentGet = 0;
 		return FALSE;
 	}
-	memcpy(cheat, &list[currentGet++], sizeof(CHEATS_LIST));
-	if (currentGet > num) 
+	//memcpy(cheat, &list[currentGet++], sizeof(CHEATS_LIST));
+	*cheat = list[currentGet++];
+	if (currentGet > list.size()) 
 	{
 		currentGet = 0;
 		return FALSE;
@@ -524,14 +535,14 @@ BOOL CHEATS::getList(CHEATS_LIST *cheat)
 
 BOOL CHEATS::get(CHEATS_LIST *cheat, u32 pos)
 {
-	if (pos > num) return FALSE;
-	memcpy(cheat, &list[pos], sizeof(CHEATS_LIST));
+	if (pos >= list.size()) return FALSE;
+	*cheat = list[pos];
 	return TRUE;
 }
 
 u32	CHEATS::getSize()
 {
-	return num;
+	return list.size();
 }
 
 BOOL CHEATS::save()
@@ -543,13 +554,15 @@ BOOL CHEATS::save()
 	if (flist)
 	{
 		fprintf(flist, "; DeSmuME cheats file. VERSION %i.%03i\n", CHEAT_VERSION_MAJOR, CHEAT_VERSION_MINOR);
-		strcpy(buf, gameInfo.ROMfullName[0]);
+		std::wstring wtemp = (std::wstring)gameInfo.getRomBanner().titles[1];
+		std::string temp = wcstombs(wtemp);
+		strcpy(buf, temp.c_str());
 		trim(buf);
 		removeSpecialChars(buf);
 		fprintf(flist, "Name=%s\n", buf);
 		fprintf(flist, "Serial=%s\n", gameInfo.ROMserial);
 		fputs("\n; cheats list\n", flist);
-		for (int i = 0;  i < num; i++)
+		for (size_t i = 0;  i < list.size(); i++)
 		{
 			if (list[i].num == 0) continue;
 			memset(buf, 0, sizeof(buf));
@@ -609,7 +622,6 @@ BOOL CHEATS::load()
 	FILE			*flist = fopen((char *)filename, "r");
 	char			buf[sizeof(list[0].code) * 2 + 200] = { 0 };
 	u32				last = 0;
-	CHEATS_LIST		tmp_cht = { 0 };
 	char			tmp_code[sizeof(list[0].code) * 2] = { 0 };
 	u32				line = 0;
 
@@ -620,6 +632,7 @@ BOOL CHEATS::load()
 		last = 0; line = 0;
 		while (!feof(flist))
 		{
+			CHEATS_LIST		tmp_cht;
 			line++;				// only for debug
 			memset(buf, 0, sizeof(buf));
 			if (fgets(buf, sizeof(buf), flist) == NULL) {
@@ -628,6 +641,8 @@ BOOL CHEATS::load()
 			}
 			trim(buf);
 			if ((strlen(buf) == 0) || (buf[0] == ';')) continue;
+			if(!strncasecmp(buf,"name=",5)) continue;
+			if(!strncasecmp(buf,"serial=",7)) continue;
 
 			memset(&tmp_cht, 0, sizeof(tmp_cht));
 			if ((buf[0] == 'D') && (buf[1] == 'S'))		// internal
@@ -656,7 +671,7 @@ BOOL CHEATS::load()
 				continue;
 			}
 
-			tmp_cht.enabled = (buf[3] == '0')?false:true;
+			tmp_cht.enabled = (buf[3] == '0')?FALSE:TRUE;
 			u32 descr_pos = (u32)(std::max<s32>(strchr((char*)buf, ';') - buf, 0));
 			if (descr_pos != 0)
 				strcpy(tmp_cht.description, (buf + descr_pos + 1));
@@ -684,49 +699,23 @@ BOOL CHEATS::load()
 				sscanf_s(tmp_buf, "%x", &tmp_cht.code[i][1]);
 			}
 
-			memcpy(&list[last], &tmp_cht, sizeof(tmp_cht));
+			list.push_back(tmp_cht);
 			last++;
 		}
 
 		fclose(flist);
-		num = last;
-		INFO("Added %i cheat codes\n", num);
+		INFO("Added %i cheat codes\n", list.size());
 		return TRUE;
 	}
 	return FALSE;
 }
 
-BOOL CHEATS::push()
-{
-	if (stack) return FALSE;
-	stack = new u8 [sizeof(list)];
-	memcpy(stack, list, sizeof(list));
-	numStack = num;
-	return TRUE;
-}
-
-BOOL CHEATS::pop()
-{
-	if (!stack) return FALSE;
-	memcpy(list, stack, sizeof(list));
-	num = numStack;
-	delete [] stack;
-	stack = NULL;
-	return TRUE;
-}
-
-void CHEATS::stackClear()
-{
-	if (!stack) return;
-	delete [] stack;
-	stack = NULL;
-}
-
 void CHEATS::process()
 {
 	if (CommonSettings.cheatsDisable) return;
-	if (!num) return;
-	for (int i = 0; i < num; i++)
+	if (list.size() == 0) return;
+	size_t num = list.size();
+	for (size_t i = 0; i < num; i++)
 	{
 		if (!list[i].enabled) continue;
 
@@ -1059,3 +1048,186 @@ void CHEATSEARCH::getListReset()
 {
 	lastRecord = 0;
 }
+
+// ========================================================================= Export
+bool CHEATSEXPORT::load(char *path)
+{
+	fp = fopen(path, "rb");
+	if (!fp)
+	{
+		printf("Error open database\n");
+		return false;
+	}
+
+	fseek(fp, 0, SEEK_END);
+	fsize = ftell(fp);
+	fseek(fp, 0, SEEK_SET);
+
+	if (!search())
+	{
+		printf("ERROR: cheat in database not founded\n");
+		return false;
+	}
+	
+	if (!getCodes())
+	{
+		printf("ERROR: export cheats failed\n");
+		return false;
+	}
+
+	return true;
+}
+void CHEATSEXPORT::close()
+{
+	if (fp)
+		fclose(fp);
+	if (cheats)
+	{
+		delete [] cheats;
+		cheats = NULL;
+	}
+}
+
+// TODO: decrypting database
+bool CHEATSEXPORT::search()
+{
+	if (!fp) return false;
+
+	u32 pos = 0x0100;
+	FAT_R4	fat_empty;
+
+	memset(&fat, 0, sizeof(FAT_R4));
+	memset(&fat_empty, 0, sizeof(FAT_R4));
+
+	fseek(fp, pos, SEEK_SET);
+	while (pos < fsize)
+	{
+		fread(&fat, sizeof(FAT_R4), 1, fp);
+		if (memcmp(&fat, &fat_empty, sizeof(FAT_R4)) == 0) break;
+		if (memcmp(gameInfo.header.gameCode, &fat.serial[0], 4) == 0)
+		{
+			FAT_R4	fat_tmp;
+
+			memset(&fat_tmp, 0, sizeof(FAT_R4));
+			fread(&fat_tmp, sizeof(FAT_R4), 1, fp);
+			if (memcmp(&fat_tmp, &fat_empty, sizeof(FAT_R4)) == 0)
+			{
+				// TODO
+				dataSize = 0;
+			}
+			else
+			{
+				dataSize = (fat_tmp.addr - fat.addr);
+			}
+			char buf[5] = {0};
+			memcpy(&buf, &fat.serial[0], 4);
+			printf("Found %s CRC %08X at 0x%08llX (size %i)\n", buf, fat.CRC, fat.addr, dataSize);
+			return true;
+		}
+		pos += sizeof(FAT_R4);
+	}
+
+	memset(&fat, 0, sizeof(FAT_R4));
+	return false;
+}
+
+bool CHEATSEXPORT::getCodes()
+{
+	if (!fp) return false;
+
+	u32	pos = 0;
+	u32	pos_cht = 0;
+
+	u8 *data = new u8 [dataSize+8];
+	if (!data) return false;
+	memset(data, 0, dataSize+8);
+	
+	fseek(fp, fat.addr, SEEK_SET);
+
+	if (fread(data, 1, dataSize, fp) != dataSize)
+	{
+		delete [] data;
+		data = NULL;
+		return false;
+	}
+
+	u8 *title = data;
+	u32 *cmd = (u32 *)(((intptr_t)title + strlen((char*)title) + 4) & 0xFFFFFFFC);
+	numCheats = (*cmd) & (~0xF0000000);
+	cmd += 9;
+	cheats = new CHEATS_LIST[numCheats];
+	memset(cheats, 0, sizeof(CHEATS_LIST) * numCheats);
+
+	while (pos < numCheats)
+	{
+		u32 folderNum = 1;
+		u8	*folderName = NULL;
+		u8	*folderNote = NULL;
+		if ((*cmd & 0xF0000000) == 0x10000000)	// Folder
+		{
+			folderNum = (*cmd  & 0x00FFFFFF);
+			folderName = (u8*)((intptr_t)cmd + 4);
+			folderNote = (u8*)((intptr_t)folderName + strlen((char*)folderName) + 1);
+			pos++;
+			numCheats--;
+			cmd = (u32 *)(((intptr_t)folderName + strlen((char*)folderName) + 1 + strlen((char*)folderNote) + 1 + 3) & 0xFFFFFFFC);
+		}
+
+		for (int i = 0; i < folderNum; i++)		// in folder
+		{
+			u8 *cheatName = (u8 *)((intptr_t)cmd + 4);
+			u8 *cheatNote = (u8 *)((intptr_t)cheatName + strlen((char*)cheatName) + 1);
+			u32 *cheatData = (u32 *)(((intptr_t)cheatNote + strlen((char*)cheatNote) + 1 + 3) & 0xFFFFFFFC);
+			u32 cheatDataLen = *cheatData++;
+
+			if (cheatDataLen)
+			{
+				char buf[2048];
+				memset(buf, 0, 2048);
+
+				cheats[pos_cht].num = cheatDataLen/2;
+				cheats[pos_cht].type = 1;
+
+				if ( folderName && *folderName )
+				{
+					sprintf(cheats[pos_cht].description, "%s: ", folderName);
+				}
+				strcat(cheats[pos_cht].description, (char*)cheatName);
+				
+				if ( cheatNote && *cheatNote )
+				{
+					strcat(cheats[pos_cht].description, "| ");
+					strcat(cheats[pos_cht].description, (char*)cheatNote);
+				}
+
+				for(u32 j = 0, t = 0; j < (cheatDataLen/2); j++, t+=2 )
+				{
+					cheats[pos_cht].code[j][0] = (u32)*(cheatData+t);
+					//printf("%i: %08X ", j, cheats[pos_cht].code[j][0]);
+					cheats[pos_cht].code[j][1] = (u32)*(cheatData+t+1);
+					//printf("%08X\n", cheats[pos_cht].code[j][1]);
+					
+				}
+			}
+			pos++; pos_cht++;
+			cmd = (u32 *)((intptr_t)cmd + ((*cmd + 1)*4));
+		}
+		
+	};
+
+	delete [] data;
+
+	//for (int i = 0; i < numCheats; i++)
+	//	printf("%i: %s\n", i, cheats[i].description);
+	
+	return true;
+}
+
+CHEATS_LIST *CHEATSEXPORT::getCheats()
+{
+	return cheats;
+}
+u32 CHEATSEXPORT::getCheatsNum()
+{
+	return numCheats;
+}
