@@ -52,7 +52,6 @@
 #include "render3D.h"
 #include "rasterize.h"
 #include "saves.h"
-#include "mic.h"
 #include "firmware.h"
 #include "GPU_osd.h"
 #include "desmume_config.h"
@@ -125,10 +124,6 @@ const u16 cli_kb_cfg[NB_KEYS] =
     SDLK_o          // BOOST
   };
 
-#ifdef FAKE_MIC
-static BOOL enable_fake_mic;
-#endif
-
 class configured_features : public CommandLine
 {
 public:
@@ -148,7 +143,7 @@ public:
 };
 
 static void
-init_config( struct configured_features *config) {
+init_config( class configured_features *config) {
 
   config->auto_pause = 0;
   config->frameskip = 0;
@@ -168,7 +163,7 @@ init_config( struct configured_features *config) {
 
 
 static int
-fill_config( struct configured_features *config,
+fill_config( class configured_features *config,
              int argc, char ** argv) {
   GOptionEntry options[] = {
     { "auto-pause", 0, 0, G_OPTION_ARG_NONE, &config->auto_pause, "Pause emulation if focus is lost", NULL},
@@ -459,6 +454,11 @@ opengl_Draw( GLuint *texture, int software_convert) {
 }
 #endif
 
+/* this is a stub for resizeWindow_stub in the case of no gl headers or no opengl 2d */
+static void
+resizeWindow_stub (u16 width, u16 height) {
+}
+
 static void
 Draw( void) {
   SDL_Surface *rawImage;
@@ -474,80 +474,23 @@ Draw( void) {
   return;
 }
 
-static void desmume_cycle(int *sdl_quit, int *boost, struct configured_features * my_config)
+static void desmume_cycle(struct ctrls_event_config * cfg)
 {
-    static unsigned short keypad;
-    static int focused = 1;
     SDL_Event event;
 
+    cfg->nds_screen_size_ratio = nds_screen_size_ratio;
+
     /* Look for queued events and update keypad status */
     /* IMPORTANT: Reenable joystick events iif needed. */
     if(SDL_JoystickEventState(SDL_QUERY) == SDL_IGNORE)
       SDL_JoystickEventState(SDL_ENABLE);
 
     /* There's an event waiting to be processed? */
-    while ( !*sdl_quit &&
-        (SDL_PollEvent(&event) || (!focused && SDL_WaitEvent(&event))))
+    while ( !cfg->sdl_quit &&
+        (SDL_PollEvent(&event) || (!cfg->focused && SDL_WaitEvent(&event))))
       {
-        process_ctrls_event( event, &keypad, nds_screen_size_ratio);
-
-        switch (event.type)
-        {
-#ifdef INCLUDE_OPENGL_2D
-          case SDL_VIDEORESIZE:
-            resizeWindow( event.resize.w, event.resize.h);
-            break;
-#endif
-          case SDL_ACTIVEEVENT:
-            if (my_config->auto_pause && (event.active.state & SDL_APPINPUTFOCUS ))
-            {
-              if (event.active.gain)
-              {
-                focused = 1;
-                SPU_Pause(0);
-                osd->addLine("Auto pause disabled\n");
-              }
-              else
-              {
-                focused = 0;
-                SPU_Pause(1);
-              }
-            }
-            break;
-
-          case SDL_KEYUP:
-            switch (event.key.keysym.sym)
-            {
-              case SDLK_ESCAPE:
-                *sdl_quit = 1;
-                break;
-#ifdef FAKE_MIC
-              case SDLK_m:
-                enable_fake_mic = !enable_fake_mic;
-                Mic_DoNoise(enable_fake_mic);
-                if (enable_fake_mic)
-                  osd->addLine("Fake mic enabled\n");
-                else
-                  osd->addLine("Fake mic disabled\n");
-                break;
-#endif
-              case SDLK_o:
-                *boost = !(*boost);
-                if (*boost)
-                  osd->addLine("Boost mode enabled\n");
-                else
-                  osd->addLine("Boost mode disabled\n");
-                break;
-              default:
-                break;
-            }
-            break;
- 
-          case SDL_QUIT:
-            *sdl_quit = 1;
-            break;
-        }
-      }
+        process_ctrls_event( event, cfg);
+    }
 
     /* Update mouse position and click */
     if(mouse.down) NDS_setTouchPos(mouse.x, mouse.y);
@@ -557,13 +500,14 @@ static void desmume_cycle(int *sdl_quit, int *boost, struct configured_features
         mouse.click = FALSE;
       }
 
-    update_keypad(keypad);     /* Update keypad */
+    update_keypad(cfg->keypad);     /* Update keypad */
     NDS_exec<false>();
     SPU_Emulate_user();
 }
 
 int main(int argc, char ** argv) {
-  struct configured_features my_config;
+  class configured_features my_config;
+  struct ctrls_event_config ctrls_cfg;
 #ifdef GDB_STUB
   gdbstub_handle_t arm9_gdb_stub;
   gdbstub_handle_t arm7_gdb_stub;
@@ -576,8 +520,6 @@ int main(int argc, char ** argv) {
   int limiter_frame_counter = 0;
   SDL_sem *fps_limiter_semaphore = NULL;
   SDL_TimerID limiter_timer = NULL;
-  int sdl_quit = 0;
-  int boost = 0;
   int error;
 
   GKeyFile *keyfile;
@@ -805,7 +747,7 @@ int main(int argc, char ** argv) {
     }
   }
 
-  if(my_config.load_slot){
+  if(my_config.load_slot != -1){
     loadstate_slot(my_config.load_slot);
   }
 
@@ -815,14 +757,23 @@ int main(int argc, char ** argv) {
   aggDraw.hud->attach(GPU_screen, 256, 384, 512);
 #endif
 
-  while(!sdl_quit) {
-    desmume_cycle(&sdl_quit, &boost, &my_config);
+  ctrls_cfg.boost = 0;
+  ctrls_cfg.sdl_quit = 0;
+  ctrls_cfg.auto_pause = my_config.auto_pause;
+  ctrls_cfg.focused = 1;
+  ctrls_cfg.fake_mic = 0;
+  ctrls_cfg.keypad = 0;
+  ctrls_cfg.resize_cb = &resizeWindow_stub;
+
+  while(!ctrls_cfg.sdl_quit) {
+    desmume_cycle(&ctrls_cfg);
 
     osd->update();
     DrawHUD();
 #ifdef INCLUDE_OPENGL_2D
     if ( my_config.opengl_2d) {
       opengl_Draw( screen_texture, my_config.soft_colour_convert);
+      ctrls_cfg.resize_cb = &resizeWindow;
     }
     else
 #endif
@@ -831,10 +782,10 @@ int main(int argc, char ** argv) {
 
     for ( int i = 0; i < my_config.frameskip; i++ ) {
         NDS_SkipNextFrame();
-        desmume_cycle(&sdl_quit, &boost, &my_config);
+        desmume_cycle(&ctrls_cfg);
     }
 
-    if ( !my_config.disable_limiter && !boost) {
+    if ( !my_config.disable_limiter && !ctrls_cfg.boost) {
       limiter_frame_counter += 1 + my_config.frameskip;
       if ( limiter_frame_counter >= my_config.fps_limiter_frame_period) {
         limiter_frame_counter = 0;
