@@ -28,7 +28,9 @@
 #include "types.h"
 #include "movie.h"
 #include "addons.h"
+#include "slot1.h"
 #include "NDSSystem.h"
+#include "utils/xstring.h"
 
 int scanline_filter_a = 2, scanline_filter_b = 4;
 int _commandline_linux_nojoy = 0;
@@ -49,10 +51,13 @@ CommandLine::CommandLine()
 , _num_cores(-1)
 , _rigorous_timing(0)
 , _advanced_timing(-1)
+, _slot1(NULL)
+, depth_threshold(-1)
+, load_slot(-1)
+, arm9_gdb_port(0)
+, arm7_gdb_port(0)
+, start_paused(FALSE)
 {
-	load_slot = 0;
-	arm9_gdb_port = arm7_gdb_port = 0;
-	start_paused = FALSE;
 #ifndef _MSC_VER
 	disable_sound = 0;
 	disable_limiter = 0;
@@ -88,6 +93,8 @@ void CommandLine::loadCommonOptions()
 		{ "scanline-filter-b", 0, 0, G_OPTION_ARG_INT, &scanline_filter_b, "Intensity of fadeout for scanlines filter (corner) (default 4)", "SCANLINE_FILTER_B"},
 		{ "rigorous-timing", 0, 0, G_OPTION_ARG_INT, &_rigorous_timing, "Use some rigorous timings instead of unrealistically generous (default 0)", "RIGOROUS_TIMING"},
 		{ "advanced-timing", 0, 0, G_OPTION_ARG_INT, &_advanced_timing, "Use advanced BUS-level timing (default 1)", "ADVANCED_TIMING"},
+		{ "slot1", 0, 0, G_OPTION_ARG_STRING, &_slot1, "Device to load in slot 1 (default retail)", "SLOT1"},
+		{ "depth-threshold", 0, 0, G_OPTION_ARG_INT, &depth_threshold, "Depth comparison threshold (default 0)", "DEPTHTHRESHOLD"},
 #ifndef _MSC_VER
 		{ "disable-sound", 0, 0, G_OPTION_ARG_NONE, &disable_sound, "Disables the sound emulation", NULL},
 		{ "disable-limiter", 0, 0, G_OPTION_ARG_NONE, &disable_limiter, "Disables the 60fps limiter", NULL},
@@ -112,6 +119,8 @@ bool CommandLine::parse(int argc,char **argv)
 		return false;
 	}
 
+	if(_slot1) slot1 = _slot1;
+	if(slot1.size() != 0) str_lcase((char*)&slot1[0]);
 	if(_play_movie_file) play_movie_file = _play_movie_file;
 	if(_record_movie_file) record_movie_file = _record_movie_file;
 	if(_cflash_image) cflash_image = _cflash_image;
@@ -121,6 +130,8 @@ bool CommandLine::parse(int argc,char **argv)
 	if(_num_cores != -1) CommonSettings.num_cores = _num_cores;
 	if(_rigorous_timing) CommonSettings.rigorous_timing = true;
 	if(_advanced_timing != -1) CommonSettings.advanced_timing = _advanced_timing==1;
+	if(depth_threshold != -1)
+		CommonSettings.GFX3D_Zelda_Shadow_Depth_Hack = depth_threshold;
 
 	//TODO MAX PRIORITY! change ARM9BIOS etc to be a std::string
 	if(_bios_arm9) { CommonSettings.UseExtBIOS = true; strcpy(CommonSettings.ARM9BIOS,_bios_arm9); }
@@ -138,8 +149,16 @@ bool CommandLine::parse(int argc,char **argv)
 
 bool CommandLine::validate()
 {
-	if (load_slot < 0 || load_slot > 10) {
-		g_printerr("I only know how to load from slots 1-10, 0 means 'do not load savegame' and is default\n");
+	if(slot1 != "")
+	{
+		if(slot1 != "r4" && slot1 != "retail" && slot1 != "none") {
+			g_printerr("Invalid slot1 device specified.\n");
+			return false;
+		}
+	}
+
+	if (load_slot < -1 || load_slot > 10) {
+		g_printerr("I only know how to load from slots 0-10; -1 means 'do not load savegame' and is default\n");
 		return false;
 	}
 
@@ -148,7 +167,7 @@ bool CommandLine::validate()
 		return false;
 	}
 
-	if(record_movie_file != "" && load_slot != 0) {
+	if(record_movie_file != "" && load_slot != -1) {
 		g_printerr("Cannot both record a movie and load a savestate.\n");
 		return false;
 	}
@@ -208,5 +227,10 @@ void CommandLine::process_addonCommands()
 		is_cflash_configured = true;
     }
 
+	if(slot1 == "retail")
+		slot1Change(NDS_SLOT1_RETAIL);
+	else if(slot1 == "r4")
+		slot1Change(NDS_SLOT1_R4);
+
 }
 
