@@ -1,22 +1,18 @@
 /*  Copyright (C) 2006 yopyop
-    yopyop156@ifrance.com
-    yopyop156.ifrance.com
+	Copyright (C) 2006-2010 DeSmuME team
 
-    This file is part of DeSmuME
+	This file is free software: you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation, either version 3 of the License, or
+	(at your option) any later version.
 
-    DeSmuME is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+	This file is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
 
-    DeSmuME is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with DeSmuME; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+	You should have received a copy of the GNU General Public License
+	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <stdlib.h>
@@ -30,12 +26,13 @@ armcp15_t *armcp15_new(armcpu_t * c)
 	int i;
 	armcp15_t *armcp15 = (armcp15_t*)malloc(sizeof(armcp15_t));
 	if(!armcp15) return NULL;
-	
+
+
 	armcp15->cpu = c;
 	armcp15->IDCode = 0x41049460;
 	armcp15->cacheType = 0x0F0D2112;
-	armcp15->TCMSize = 0x00140140;
-	armcp15->ctrl = 0x00000000;
+	armcp15->TCMSize = 0x00140180;
+	armcp15->ctrl = 0x00012078;
 	armcp15->DCConfig = 0x0;    
 	armcp15->ICConfig = 0x0;    
 	armcp15->writeBuffCtrl = 0x0;
@@ -57,22 +54,26 @@ armcp15_t *armcp15_new(armcpu_t * c)
 	armcp15->DTCMRegion = 0x0080000A;
 	armcp15->processID = 0;
 
-    /* preset calculated regionmasks */	
+	MMU.ARM9_RW_MODE = BIT7(armcp15->ctrl);
+	armcp15->cpu->intVector = 0xFFFF0000 * (BIT13(armcp15->ctrl));
+	armcp15->cpu->LDTBit = !BIT15(armcp15->ctrl); //TBit
+
+	/* preset calculated regionmasks */	
 	for (i=0;i<8;i++) {
-        armcp15->regionWriteMask_USR[i] = 0 ;
-        armcp15->regionWriteMask_SYS[i] = 0 ;
-        armcp15->regionReadMask_USR[i] = 0 ;
-        armcp15->regionReadMask_SYS[i] = 0 ;
-        armcp15->regionExecuteMask_USR[i] = 0 ;
-        armcp15->regionExecuteMask_SYS[i] = 0 ;
-        armcp15->regionWriteSet_USR[i] = 0 ;
-        armcp15->regionWriteSet_SYS[i] = 0 ;
-        armcp15->regionReadSet_USR[i] = 0 ;
-        armcp15->regionReadSet_SYS[i] = 0 ;
-        armcp15->regionExecuteSet_USR[i] = 0 ;
-        armcp15->regionExecuteSet_SYS[i] = 0 ;
-    } ;
-	
+		armcp15->regionWriteMask_USR[i] = 0 ;
+		armcp15->regionWriteMask_SYS[i] = 0 ;
+		armcp15->regionReadMask_USR[i] = 0 ;
+		armcp15->regionReadMask_SYS[i] = 0 ;
+		armcp15->regionExecuteMask_USR[i] = 0 ;
+		armcp15->regionExecuteMask_SYS[i] = 0 ;
+		armcp15->regionWriteSet_USR[i] = 0 ;
+		armcp15->regionWriteSet_SYS[i] = 0 ;
+		armcp15->regionReadSet_USR[i] = 0 ;
+		armcp15->regionReadSet_SYS[i] = 0 ;
+		armcp15->regionExecuteSet_USR[i] = 0 ;
+		armcp15->regionExecuteSet_SYS[i] = 0 ;
+	} ;
+
 	return armcp15;
 }
 
@@ -193,19 +194,19 @@ static void armcp15_setSingleRegionAccess(armcp15_t *armcp15,u32 dAccess,u32 iAc
 /* precalculate region masks/sets from cp15 register */
 static void armcp15_maskPrecalc(armcp15_t *armcp15)
 {
-	#define precalc(num) {  \
+#define precalc(num) {  \
 	u32 mask = 0, set = 0xFFFFFFFF ; /* (x & 0) == 0xFF..FF is allways false (disabled) */  \
 	if (BIT_N(armcp15->protectBaseSize##num,0)) /* if region is enabled */ \
 	{    /* reason for this define: naming includes var */  \
-		mask = MASKFROMREG(armcp15->protectBaseSize##num) ;   \
-		set = SETFROMREG(armcp15->protectBaseSize##num) ; \
-		if (SIZEIDENTIFIER(armcp15->protectBaseSize##num)==0x1F)  \
-		{   /* for the 4GB region, u32 suffers wraparound */   \
-		mask = 0 ; set = 0 ;   /* (x & 0) == 0  is allways true (enabled) */  \
-		} \
-	}  \
-		armcp15_setSingleRegionAccess(armcp15,armcp15->DaccessPerm,armcp15->IaccessPerm,num,mask,set) ;  \
-	}
+	mask = MASKFROMREG(armcp15->protectBaseSize##num) ;   \
+	set = SETFROMREG(armcp15->protectBaseSize##num) ; \
+	if (SIZEIDENTIFIER(armcp15->protectBaseSize##num)==0x1F)  \
+	{   /* for the 4GB region, u32 suffers wraparound */   \
+	mask = 0 ; set = 0 ;   /* (x & 0) == 0  is allways true (enabled) */  \
+} \
+}  \
+	armcp15_setSingleRegionAccess(armcp15,armcp15->DaccessPerm,armcp15->IaccessPerm,num,mask,set) ;  \
+}
 	precalc(0) ;
 	precalc(1) ;
 	precalc(2) ;
@@ -248,348 +249,321 @@ BOOL armcp15_isAccessAllowed(armcp15_t *armcp15,u32 address,u32 access)
 
 BOOL armcp15_dataProcess(armcp15_t *armcp15, u8 CRd, u8 CRn, u8 CRm, u8 opcode1, u8 opcode2)
 {
-		LOG("Unsupported CP15 operation : DataProcess\n");
-   return FALSE;
+	LOG("Unsupported CP15 operation : DataProcess\n");
+	return FALSE;
 }
 
 BOOL armcp15_load(armcp15_t *armcp15, u8 CRd, u8 adr)
 {
 	LOG("Unsupported CP15 operation : Load\n");
-   return FALSE;
+	return FALSE;
 }
 
 BOOL armcp15_store(armcp15_t *armcp15, u8 CRd, u8 adr)
 {
 	LOG("Unsupported CP15 operation : Store\n");
-   return FALSE;
+	return FALSE;
 }
 
 BOOL armcp15_moveCP2ARM(armcp15_t *armcp15, u32 * R, u8 CRn, u8 CRm, u8 opcode1, u8 opcode2)
 {
 	if(armcp15->cpu->CPSR.bits.mode == USR) return FALSE;
-	
+
 	switch(CRn)
 	{
-		case 0 :
-			if((opcode1 == 0)&&(CRm==0))
-			{
+	case 0:
+		if((opcode1 == 0)&&(CRm==0))
+		{
 			switch(opcode2)
 			{
-				case 1 :
-					*R = armcp15->cacheType;
-					return TRUE;
-				case 2 :
-					*R = armcp15->TCMSize;
-					return TRUE;
-				default :
-					*R = armcp15->IDCode;
-					return TRUE;
-			}
-			}
-			return FALSE;
-		case 1 :
-			if((opcode1==0) && (opcode2==0) && (CRm==0))
-			{
-				*R = armcp15->ctrl;
+			case 1:
+				*R = armcp15->cacheType;
+				return TRUE;
+			case 2:
+				*R = armcp15->TCMSize;
+				return TRUE;
+			default:
+				*R = armcp15->IDCode;
 				return TRUE;
 			}
-			return FALSE;
-			
-		case 2 :
-			if((opcode1==0) && (CRm==0))
+		}
+		return FALSE;
+	case 1:
+		if((opcode1==0) && (opcode2==0) && (CRm==0))
+		{
+			*R = armcp15->ctrl;
+			//LOG("CP15: CPtoARM ctrl %08X\n", armcp15->ctrl);
+			return TRUE;
+		}
+		return FALSE;
+
+	case 2:
+		if((opcode1==0) && (CRm==0))
+		{
+			switch(opcode2)
 			{
-				switch(opcode2)
-				{
-					case 0 :
-						*R = armcp15->DCConfig;
-					return TRUE;
-					case 1 :
-						*R = armcp15->ICConfig;
-					return TRUE;
-					default :
-					return FALSE;
-				}
+			case 0:
+				*R = armcp15->DCConfig;
+				return TRUE;
+			case 1:
+				*R = armcp15->ICConfig;
+				return TRUE;
+			default:
+				return FALSE;
 			}
-			return FALSE;
-		case 3 :
-			if((opcode1==0) && (opcode2==0) && (CRm==0))
+		}
+		return FALSE;
+	case 3:
+		if((opcode1==0) && (opcode2==0) && (CRm==0))
+		{
+			*R = armcp15->writeBuffCtrl;
+			//LOG("CP15: CPtoARM writeBuffer ctrl %08X\n", armcp15->writeBuffCtrl);
+			return TRUE;
+		}
+		return FALSE;
+	case 5:
+		if((opcode1==0) && (CRm==0))
+		{
+			switch(opcode2)
 			{
-				*R = armcp15->writeBuffCtrl;
+			case 2:
+				*R = armcp15->DaccessPerm;
+				return TRUE;
+			case 3:
+				*R = armcp15->IaccessPerm;
 				return TRUE;
+			default:
+				return FALSE;
 			}
-			return FALSE;
-		case 5 :
-			if((opcode1==0) && (CRm==0))
+		}
+		return FALSE;
+	case 6:
+		if((opcode1==0) && (opcode2==0))
+		{
+			switch(CRm)
 			{
+			case 0:
+				*R = armcp15->protectBaseSize0;
+				return TRUE;
+			case 1:
+				*R = armcp15->protectBaseSize1;
+				return TRUE;
+			case 2:
+				*R = armcp15->protectBaseSize2;
+				return TRUE;
+			case 3:
+				*R = armcp15->protectBaseSize3;
+				return TRUE;
+			case 4:
+				*R = armcp15->protectBaseSize4;
+				return TRUE;
+			case 5:
+				*R = armcp15->protectBaseSize5;
+				return TRUE;
+			case 6:
+				*R = armcp15->protectBaseSize6;
+				return TRUE;
+			case 7:
+				*R = armcp15->protectBaseSize7;
+				return TRUE;
+			default:
+				return FALSE;
+			}
+		}
+		return FALSE;
+	case 9:
+		if((opcode1==0))
+		{
+			switch(CRm)
+			{
+			case 0:
 				switch(opcode2)
 				{
-					case 2 :
-						*R = armcp15->DaccessPerm;
-					    return TRUE;
-					case 3 :
-						*R = armcp15->IaccessPerm;
+				case 0:
+					*R = armcp15->DcacheLock;
 					return TRUE;
-					default :
+				case 1:
+					*R = armcp15->IcacheLock;
+					return TRUE;
+				default:
 					return FALSE;
 				}
-			}
-			return FALSE;
-		case 6 :
-			if((opcode1==0) && (opcode2==0))
-			{
-				switch(CRm)
+			case 1:
+				switch(opcode2)
 				{
-					case 0 :
-						*R = armcp15->protectBaseSize0;
-					return TRUE;
-					case 1 :
-						*R = armcp15->protectBaseSize1;
-					return TRUE;
-					case 2 :
-						*R = armcp15->protectBaseSize2;
+				case 0:
+					*R = armcp15->DTCMRegion;
 					return TRUE;
-					case 3 :
-						*R = armcp15->protectBaseSize3;
+				case 1:
+					*R = armcp15->ITCMRegion;
 					return TRUE;
-					case 4 :
-						*R = armcp15->protectBaseSize4;
-					return TRUE;
-					case 5 :
-						*R = armcp15->protectBaseSize5;
-					return TRUE;
-					case 6 :
-						*R = armcp15->protectBaseSize6;
-					return TRUE;
-					case 7 :
-						*R = armcp15->protectBaseSize7;
-					return TRUE;
-					default :
+				default:
 					return FALSE;
-				}
-			}
-			return FALSE;
-		case 9 :
-			if((opcode1==0))
-			{
-				switch(CRm)
-				{
-					case 0 :
-						switch(opcode2)
-						{
-							case 0 :
-							*R = armcp15->DcacheLock;
-							return TRUE;
-							case 1 :
-							*R = armcp15->IcacheLock;
-							return TRUE;
-							default :
-								return FALSE;
-						}
-					case 1 :
-						switch(opcode2)
-						{
-						case 0 :
-						*R = armcp15->DTCMRegion;
-						return TRUE;
-						case 1 :
-						*R = armcp15->ITCMRegion;
-						return TRUE;
-						default :
-							return FALSE;
-						}
 				}
 			}
+		}
+		return FALSE;
+	default:
+		LOG("Unsupported CP15 operation : MRC\n");
 		return FALSE;
-		default :
-			LOG("Unsupported CP15 operation : MRC\n");
-			return FALSE;
 	}
 }
 
-
 static u32 CP15wait4IRQ(armcpu_t *cpu)
 {
-	u32 instructAddr = cpu->instruct_adr;
-	/* on the first call, wirq is not set */
-	if(cpu->wirq)
-	{
-		/* check wether an irq was issued */
-		if(!cpu->waitIRQ)
-		{
-			cpu->waitIRQ = 0;
-			cpu->wirq = 0;
-			return 1;   /* return execution */
-		}
-		/* otherwise, repeat this instruction */
-		cpu->R[15] = instructAddr;
-		cpu->next_instruction = instructAddr;
-		return 1;
-	}
-	/* first run, set us into waiting state */
-	cpu->waitIRQ = 1;
-	cpu->wirq = 1;
-	/* and set next instruction to repeat this */
-	cpu->R[15] = instructAddr;
-	cpu->next_instruction = instructAddr;
-	/* CHECKME: IME shouldn't be modified (?) */
-	MMU.reg_IME[0] = 1;
+	cpu->waitIRQ = TRUE;
+	cpu->halt_IE_and_IF = TRUE;
+	//IME set deliberately omitted: only SWI sets IME to 1
 	return 1;
 }
 
 BOOL armcp15_moveARM2CP(armcp15_t *armcp15, u32 val, u8 CRn, u8 CRm, u8 opcode1, u8 opcode2)
 {
 	if(armcp15->cpu->CPSR.bits.mode == USR) return FALSE;
-	
+
 	switch(CRn)
 	{
-		case 1 :
+	case 1:
 		if((opcode1==0) && (opcode2==0) && (CRm==0))
 		{
-			armcp15->ctrl = val;
+
+			//On the NDS bit0,2,7,12..19 are R/W, Bit3..6 are always set, all other bits are always zero.
+			armcp15->ctrl = (val & 0x000FF085) | 0x00000078;
 			MMU.ARM9_RW_MODE = BIT7(val);
 			//zero 31-jan-2010: change from 0x0FFF0000 to 0xFFFF0000 per gbatek
 			armcp15->cpu->intVector = 0xFFFF0000 * (BIT13(val));
 			armcp15->cpu->LDTBit = !BIT15(val); //TBit
-			/*if(BIT17(val))
-			{
-				log::ajouter("outch !!!!!!!");
-			}
-			if(BIT19(val))
-			{
-				log::ajouter("outch !!!!!!!");
-			}*/
+			//LOG("CP15: ARMtoCP ctrl %08X (val %08X)\n", armcp15->ctrl, val);
 			return TRUE;
 		}
 		return FALSE;
-		case 2 :
+	case 2:
 		if((opcode1==0) && (CRm==0))
 		{
 			switch(opcode2)
 			{
-				case 0 :
-					armcp15->DCConfig = val;
-					return TRUE;
-				case 1 :
-					armcp15->ICConfig = val;
-					return TRUE;
-				default :
-					return FALSE;
+			case 0:
+				armcp15->DCConfig = val;
+				return TRUE;
+			case 1:
+				armcp15->ICConfig = val;
+				return TRUE;
+			default:
+				return FALSE;
 			}
 		}
 		return FALSE;
-		case 3 :
+	case 3:
 		if((opcode1==0) && (opcode2==0) && (CRm==0))
 		{
 			armcp15->writeBuffCtrl = val;
+			//LOG("CP15: ARMtoCP writeBuffer ctrl %08X\n", armcp15->writeBuffCtrl);
 			return TRUE;
 		}
 		return FALSE;
-		case 5 :
+	case 5:
 		if((opcode1==0) && (CRm==0))
 		{
 			switch(opcode2)
 			{
-				case 2 :
-					armcp15->DaccessPerm = val;
-					armcp15_maskPrecalc(armcp15);
- 					return TRUE;
-				case 3 :
-					armcp15->IaccessPerm = val;
-					armcp15_maskPrecalc(armcp15);
-					return TRUE;
-				default :
-					return FALSE;
+			case 2:
+				armcp15->DaccessPerm = val;
+				armcp15_maskPrecalc(armcp15);
+				return TRUE;
+			case 3:
+				armcp15->IaccessPerm = val;
+				armcp15_maskPrecalc(armcp15);
+				return TRUE;
+			default:
+				return FALSE;
 			}
 		}
 		return FALSE;
-		case 6 :
+	case 6:
 		if((opcode1==0) && (opcode2==0))
 		{
 			switch(CRm)
 			{
-				case 0 :
-					armcp15->protectBaseSize0 = val;
-					armcp15_maskPrecalc(armcp15) ;
-					return TRUE;
-				case 1 :
-					armcp15->protectBaseSize1 = val;
-					armcp15_maskPrecalc(armcp15) ;
-					return TRUE;
-				case 2 :
-					armcp15->protectBaseSize2 = val;
-					armcp15_maskPrecalc(armcp15) ;
-					return TRUE;
-				case 3 :
-					armcp15->protectBaseSize3 = val;
-					armcp15_maskPrecalc(armcp15) ;
-					return TRUE;
-				case 4 :
-					armcp15->protectBaseSize4 = val;
-					armcp15_maskPrecalc(armcp15) ;
-					return TRUE;
-				case 5 :
-					armcp15->protectBaseSize5 = val;
-					armcp15_maskPrecalc(armcp15) ;
-					return TRUE;
-				case 6 :
-					armcp15->protectBaseSize6 = val;
-					armcp15_maskPrecalc(armcp15) ;
-					return TRUE;
-				case 7 :
-					armcp15->protectBaseSize7 = val;
-					armcp15_maskPrecalc(armcp15) ;
-					return TRUE;
-				default :
-					return FALSE;
+			case 0:
+				armcp15->protectBaseSize0 = val;
+				armcp15_maskPrecalc(armcp15) ;
+				return TRUE;
+			case 1:
+				armcp15->protectBaseSize1 = val;
+				armcp15_maskPrecalc(armcp15) ;
+				return TRUE;
+			case 2:
+				armcp15->protectBaseSize2 = val;
+				armcp15_maskPrecalc(armcp15) ;
+				return TRUE;
+			case 3:
+				armcp15->protectBaseSize3 = val;
+				armcp15_maskPrecalc(armcp15) ;
+				return TRUE;
+			case 4:
+				armcp15->protectBaseSize4 = val;
+				armcp15_maskPrecalc(armcp15) ;
+				return TRUE;
+			case 5:
+				armcp15->protectBaseSize5 = val;
+				armcp15_maskPrecalc(armcp15) ;
+				return TRUE;
+			case 6:
+				armcp15->protectBaseSize6 = val;
+				armcp15_maskPrecalc(armcp15) ;
+				return TRUE;
+			case 7:
+				armcp15->protectBaseSize7 = val;
+				armcp15_maskPrecalc(armcp15) ;
+				return TRUE;
+			default:
+				return FALSE;
 			}
 		}
 		return FALSE;
-		case 7 :
+	case 7:
 		if((CRm==0)&&(opcode1==0)&&((opcode2==4)))
 		{
 			CP15wait4IRQ(armcp15->cpu);
 			return TRUE;
 		}
 		return FALSE;
-		case 9 :
+	case 9:
 		if((opcode1==0))
 		{
 			switch(CRm)
 			{
-				case 0 :
+			case 0:
 				switch(opcode2)
 				{
-					case 0 :
-						armcp15->DcacheLock = val;
-						return TRUE;
-					case 1 :
-						armcp15->IcacheLock = val;
-						return TRUE;
-					default :
-						return FALSE;
-			}
-			case 1 :
-			switch(opcode2)
-			{
-				case 0 :
-					armcp15->DTCMRegion = val;
-					MMU.DTCMRegion = val & 0x0FFFFFFC0;
-					/*sprintf(logbuf, "%08X", val);
-					log::ajouter(logbuf);*/
+				case 0:
+					armcp15->DcacheLock = val;
 					return TRUE;
-				case 1 :
+				case 1:
+					armcp15->IcacheLock = val;
+					return TRUE;
+				default:
+					return FALSE;
+				}
+			case 1:
+				switch(opcode2)
+				{
+				case 0:
+					MMU.DTCMRegion = armcp15->DTCMRegion = val & 0x0FFFF000;
+					return TRUE;
+				case 1:
 					armcp15->ITCMRegion = val;
-					/* ITCM base is not writeable! */
+					//ITCM base is not writeable!
 					MMU.ITCMRegion = 0;
 					return TRUE;
-				default :
+				default:
 					return FALSE;
 				}
 			}
 		}
 		return FALSE;
-		default :
-			return FALSE;
+	default:
+		return FALSE;
 	}
 }
 
