@@ -26,7 +26,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
-//#include <pthread.h>
 #include <SDL.h>
 #include "MMU.h"
 
@@ -35,7 +34,15 @@
 #define ADD_KEY(keypad,key) ( (keypad) |= (key) )
 #define RM_KEY(keypad,key) ( (keypad) &= ~(key) )
 #define KEYMASK_(k)	(1 << (k))
-#define JOY_AXIS_(k)    (((k)+1) << 8)
+
+#define JOY_AXIS  0
+#define JOY_HAT  1
+#define JOY_BUTTON  2
+
+#define JOY_HAT_RIGHT 0
+#define JOY_HAT_LEFT 1
+#define JOY_HAT_UP 2
+#define JOY_HAT_DOWN 3
 
 #define NB_KEYS		14
 #define KEY_NONE		0
@@ -73,26 +80,31 @@ struct mouse_status
 };
 
 extern mouse_status mouse;
-
-void set_mouse_coord(signed long x,signed long y);
 #endif // !GTK_UI
 
+struct ctrls_event_config {
+  unsigned short keypad;
+  float nds_screen_size_ratio;
+  int auto_pause;
+  int focused;
+  int sdl_quit;
+  int boost;
+  int fake_mic;
+  void (*resize_cb)(u16 width, u16 height);
+};
+
 void load_default_config(const u16 kbCfg[]);
 BOOL init_joy( void);
 void uninit_joy( void);
-void set_joy_keys(const u16 joyCfg[]);
-void set_kb_keys(const u16 kbCfg[]);
 u16 get_joy_key(int index);
 u16 get_set_joy_key(int index);
-u16 get_joy_axis(int index, int index_opp);
-void get_set_joy_axis(int index, int index_opp);
 void update_keypad(u16 keys);
 u16 get_keypad( void);
 u16 lookup_key (u16 keyval);
 u16 lookup_joy_key (u16 keyval);
 void
-process_ctrls_event( SDL_Event& event, u16 *keypad,
-                      float nds_screen_size_ratio);
+process_ctrls_event( SDL_Event& event,
+                      struct ctrls_event_config *cfg);
 
 void
 process_joystick_events( u16 *keypad);
