@@ -1,37 +1,36 @@
- /* Copyright (C) 2009 DeSmuME team
- *
- * This file is part of DeSmuME
- *
- * DeSmuME is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * DeSmuME is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with DeSmuME; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
+ /* Copyright (C) 2009-2010 DeSmuME team
+
+	This file is free software: you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation, either version 2 of the License, or
+	(at your option) any later version.
+
+	This file is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
+
+	You should have received a copy of the GNU General Public License
+	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+//don't use emufile for files bigger than 2GB! you have been warned! some day this will be fixed.
+
 #ifndef EMUFILE_H
 #define EMUFILE_H
 
 #include <assert.h>
 #include <stdio.h>
 #include <string.h>
-#include "types.h"
 #include <vector>
 #include <algorithm>
 #include <string>
 #include <stdarg.h>
 
-#ifdef _XBOX
-#undef min;
-#undef max;
+#include "types.h"
+
+#ifdef _MSC_VER
+#include <io.h>
 #endif
 
 class EMUFILE {
@@ -43,11 +42,16 @@ public:
 		: failbit(false)
 	{}
 
+
+	//returns a new EMUFILE which is guranteed to be in memory. the EMUFILE you call this on may be deleted. use the returned EMUFILE in its place
+	virtual EMUFILE* memwrap() = 0;
+
 	virtual ~EMUFILE() {}
 	
 	static bool readAllBytes(std::vector<u8>* buf, const std::string& fname);
 
-	bool fail() { return failbit; }
+	bool fail(bool unset=false) { bool ret = failbit; if(unset) unfail(); return ret; }
+	void unfail() { failbit=false; }
 
 	bool eof() { return size()==ftell(); }
 
@@ -78,6 +82,8 @@ public:
 
 	virtual int ftell() = 0;
 	virtual int size() = 0;
+
+	virtual void truncate(s32 length) = 0;
 };
 
 //todo - handle read-only specially?
@@ -94,14 +100,31 @@ protected:
 
 public:
 
-	EMUFILE_MEMORY(std::vector<u8> *underlying) : vec(underlying), ownvec(false), pos(0), len(underlying->size()) { }
-	EMUFILE_MEMORY(u32 preallocate) : vec(new std::vector<u8>()), ownvec(true), pos(0), len(0) { vec->reserve(preallocate); }
+	EMUFILE_MEMORY(std::vector<u8> *underlying) : vec(underlying), ownvec(false), pos(0), len((s32)underlying->size()) { }
+	EMUFILE_MEMORY(u32 preallocate) : vec(new std::vector<u8>()), ownvec(true), pos(0), len(0) { 
+		vec->resize(preallocate);
+		len = preallocate;
+	}
 	EMUFILE_MEMORY() : vec(new std::vector<u8>()), ownvec(true), pos(0), len(0) { vec->reserve(1024); }
+	EMUFILE_MEMORY(void* buf, s32 size) : vec(new std::vector<u8>()), ownvec(true), pos(0), len(size) { 
+		vec->resize(size);
+		if(size != 0)
+			memcpy(&vec[0],buf,size);
+	}
 
 	~EMUFILE_MEMORY() {
 		if(ownvec) delete vec;
 	}
 
+	virtual EMUFILE* memwrap();
+
+	virtual void truncate(s32 length)
+	{
+		vec->resize(length);
+		len = length;
+		if(pos>length) pos=length;
+	}
+
 	u8* buf() { return &(*vec)[0]; }
 
 	std::vector<u8>* get_vec() { return vec; };
@@ -111,22 +134,37 @@ public:
 	virtual int fprintf(const char *format, ...) {
 		va_list argptr;
 		va_start(argptr, format);
-		
+
 		//we dont generate straight into the buffer because it will null terminate (one more byte than we want)
 		int amt = vsnprintf(0,0,format,argptr);
 		char* tempbuf = new char[amt+1];
-		vsprintf(tempbuf,format,argptr);
-		fwrite(tempbuf,amt);
-		delete[] tempbuf;
+
 		va_end(argptr);
+		va_start(argptr, format);
+		vsprintf(tempbuf,format,argptr);
+		
+        fwrite(tempbuf,amt);
+		delete[] tempbuf;
+		
+        va_end(argptr);
 		return amt;
 	};
 
 	virtual int fgetc() {
 		u8 temp;
-		if(_fread(&temp,1) != 1)
-			return EOF;
-		else return temp;
+
+		//need an optimized codepath
+		//if(_fread(&temp,1) != 1)
+		//	return EOF;
+		//else return temp;
+		u32 remain = len-pos;
+		if(remain<1) {
+			failbit = true;
+			return -1;
+		}
+		temp = buf()[pos];
+		pos++;
+		return temp;
 	}
 	virtual int fputc(int c) {
 		u8 temp = (u8)c;
@@ -137,23 +175,15 @@ public:
 		return 0;
 	}
 
-	virtual size_t _fread(const void *ptr, size_t bytes){
-		u32 remain = len-pos;
-		u32 todo = std::min<u32>(remain,(u32)bytes);
-		memcpy((void*)ptr,buf()+pos,todo);
-		pos += todo;
-		if(todo<bytes)
-			failbit = true;
-		return todo;
-	}
+	virtual size_t _fread(const void *ptr, size_t bytes);
 
 	//removing these return values for now so we can find any code that might be using them and make sure
 	//they handle the return values correctly
 
 	virtual void fwrite(const void *ptr, size_t bytes){
-		reserve(pos+bytes);
+		reserve(pos+(s32)bytes);
 		memcpy(buf()+pos,ptr,bytes);
-		pos += bytes;
+		pos += (s32)bytes;
 		len = std::max(pos,len);
 	}
 
@@ -180,21 +210,34 @@ public:
 		return pos;
 	}
 
+	void trim()
+	{
+		vec->resize(len);
+	}
+
 	virtual int size() { return (int)len; }
 };
 
 class EMUFILE_FILE : public EMUFILE { 
 protected:
 	FILE* fp;
+	std::string fname;
+	char mode[16];
 
-public:
-
-	EMUFILE_FILE(const char* fname, const char* mode)
+private:
+	void open(const char* fname, const char* mode)
 	{
 		fp = fopen(fname,mode);
 		if(!fp)
 			failbit = true;
-	};
+		this->fname = fname;
+		strcpy(this->mode,mode);
+	}
+
+public:
+
+	EMUFILE_FILE(const std::string& fname, const char* mode) { open(fname.c_str(),mode); }
+	EMUFILE_FILE(const char* fname, const char* mode) { open(fname,mode); }
 
 	virtual ~EMUFILE_FILE() {
 		if(NULL != fp)
@@ -205,6 +248,12 @@ public:
 		return fp; 
 	}
 
+	virtual EMUFILE* memwrap();
+
+	bool is_open() { return fp != NULL; }
+
+	virtual void truncate(s32 length);
+
 	virtual int fprintf(const char *format, ...) {
 		va_list argptr;
 		va_start(argptr, format);
