@@ -19,6 +19,7 @@
 
 #include "firmware.h"
 #include "NDSSystem.h"
+#include "path.h"
 
 #define DWNUM(i) ((i) >> 2)
 
@@ -565,6 +566,41 @@ bool CFIRMWARE::load()
 		INFO("   * ARM7 unpacked size:         0x%08X (%i) bytes\n", size7, size7);
 	}
 
+	PathInfo path;
+	path.init(CommonSettings.Firmware);
+	path.getpathnoext(path.FIRMWARE, &MMU.fw.userfile[0]);
+	strcat(MMU.fw.userfile, ".dfc");		// DeSmuME Firmware Config
+
+	fp = fopen(MMU.fw.userfile, "rb");
+	if (fp)
+	{
+		char buf[0x300];
+		memset(buf, 0, 0x300);
+		if (fread(buf, 1, 0x100, fp) == 0x100)
+		{
+			printf("- loaded from %s:\n", MMU.fw.userfile);
+			memcpy(&data[0x3FE00], &buf[0], 0x100);
+			memcpy(&data[0x3FF00], &buf[0], 0x100);
+			printf("   * User settings\n");
+			memset(buf, 0, 0x100);
+			if (fread(buf, 1, 0x1D6, fp) == 0x1D6)
+			{
+				memcpy(&data[0x002A], &buf[0], 0x1D6);
+				printf("   * WiFi settings\n");
+
+				memset(buf, 0, 0x1D6);
+				if (fread(buf, 1, 0x300, fp) == 0x300)
+				{
+					memcpy(&data[0x3FA00], &buf[0], 0x300);
+					printf("   * WiFi AP settings\n");
+				}
+			}
+			
+		}
+		fclose(fp);
+	}
+	printf("\n");
+
 	// TODO: add 512Kb support
 	memcpy(MMU.fw.data, data, 256*1024);
 	MMU.fw.fp = NULL;
@@ -869,3 +905,9 @@ void NDS_FillDefaultFirmwareConfigData( struct NDS_fw_config_data *fw_config) {
 	fw_config->touch_cal[1].screen_x = 0xe0 + 1;
 	fw_config->touch_cal[1].screen_y = 0x80 + 1;
 }
+
+void NDS_PatchFirmwareMAC()
+{
+	memcpy((MMU.fw.data + 0x36), FW_Mac, sizeof(FW_Mac));
+	(*(u16*)(MMU.fw.data + 0x2A)) = calc_CRC16(0, (MMU.fw.data + 0x2C), 0x138);
+}
