@@ -1,22 +1,18 @@
-/*	gfx3d.cpp
-	Copyright (C) 2006 yopyop
+/*	Copyright (C) 2006 yopyop
     Copyright (C) 2008-2010 DeSmuME team
 
-    This file is part of DeSmuME
+	This file is free software: you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation, either version 2 of the License, or
+	(at your option) any later version.
 
-    DeSmuME is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+	This file is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
 
-    DeSmuME is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with DeSmuME; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
+	You should have received a copy of the GNU General Public License
+	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 //This file implements the geometry engine hardware component.
@@ -157,13 +153,15 @@ public:
 			//decode a packed command
 decode:
 			//printf("[%05d] gxf: decoding %08X\n",currFrameCounter,val);
-			
+			if (val == 0) return;	// nop
+
 			const u8 commands[] = { val&0xFF, (val>>8)&0xFF, (val>>16)&0xFF, (val>>24)&0xFF };
 			const u8 commandTypes[] = { gfx3d_commandTypes[commands[0]], gfx3d_commandTypes[commands[1]],gfx3d_commandTypes[commands[2]], gfx3d_commandTypes[commands[3]] };
 
 			for(int i=0;i<4;i++) {
 				u8 cmd = commands[i];
 				u8 type = commandTypes[i];
+
 				if(type == INVALID_COMMAND) {
 					commandsPending[i].command = INVALID_COMMAND;
 				} else {
@@ -179,8 +177,12 @@ decode:
 					//these are valid commands with no parameters. they might need special handling
 					//as long as there is a subsequent defined command with parameters, we're safe
 					bool safe = false;
-					for(int j=i+1;j<4;j++) {
-						if(commandTypes[j] != INVALID_COMMAND) {
+					if (i == 3)
+						safe = true;
+					else
+						// start loop from (i+1) always skip last command
+					for(u8 j=i+1;j<4;j++) {
+						if((commandTypes[j] != INVALID_COMMAND)) {
 							safe = true;
 							break;
 						}
@@ -261,7 +263,7 @@ using std::max;
 using std::min;
 
 GFX3D gfx3d;
-Viewer3d_State viewer3d_state;
+Viewer3d_State* viewer3d_state = NULL;
 static GFX3D_Clipper boxtestClipper;
 
 //tables that are provided to anyone
@@ -331,8 +333,8 @@ CACHE_ALIGN MatrixStack	mtxStack[4] = {
 
 int _hack_getMatrixStackLevel(int which) { return mtxStack[which].position; }
 
-static CACHE_ALIGN float		mtxCurrent [4][16];
-static CACHE_ALIGN float		mtxTemporal[16];
+static CACHE_ALIGN s32		mtxCurrent [4][16];
+static CACHE_ALIGN s32		mtxTemporal[16];
 static u32 mode = 0;
 
 // Indexes for matrix loading/multiplication
@@ -343,21 +345,21 @@ static u8 MM4x3ind = 0;
 static u8 MM3x3ind = 0;
 
 // Data for vertex submission
-static CACHE_ALIGN u16		u16coord[4] = {0, 0, 0, 0};
+static CACHE_ALIGN s16		s16coord[4] = {0, 0, 0, 0};
 static char		coordind = 0;
 static u32 vtxFormat = 0;
 static BOOL inBegin = FALSE;
 
 // Data for basic transforms
-static CACHE_ALIGN float	trans[4] = {0.0, 0.0, 0.0, 0.0};
+static CACHE_ALIGN s32	trans[4] = {0, 0, 0, 0};
 static int		transind = 0;
-static CACHE_ALIGN float	scale[4] = {0.0, 0.0, 0.0, 0.0};
+static CACHE_ALIGN s32	scale[4] = {0, 0, 0, 0};
 static int		scaleind = 0;
 static u32 viewport = 0;
 
 //various other registers
-static float _t=0, _s=0;
-static float last_t, last_s;
+static s32 _t=0, _s=0;
+static s32 last_t, last_s;
 static u32 clCmd = 0;
 static u32 clInd = 0;
 
@@ -379,7 +381,7 @@ u32 control = 0;
 
 //light state:
 static u32 lightColor[4] = {0,0,0,0};
-static u32 lightDirection[4] = {0,0,0,0};
+static s32 lightDirection[4] = {0,0,0,0};
 //material state:
 static u16 dsDiffuse, dsAmbient, dsSpecular, dsEmission;
 //used for indexing the shininess table during parameters to shininess command
@@ -394,8 +396,8 @@ static u32 envMode=0;
 static u32 lightMask=0;
 //other things:
 static int texCoordinateTransform = 0;
-static CACHE_ALIGN float cacheLightDirection[4][4];
-static CACHE_ALIGN float cacheHalfVector[4][4];
+static CACHE_ALIGN s32 cacheLightDirection[4][4];
+static CACHE_ALIGN s32 cacheHalfVector[4][4];
 //------------------
 
 #define RENDER_FRONT_SURFACE 0x80
@@ -501,8 +503,9 @@ void gfx3d_init()
 
 void gfx3d_reset()
 {
-	//if this doesn't work on the xbox, we need to find out why.
 	reconstruct(&gfx3d);
+	delete viewer3d_state;
+	viewer3d_state = new Viewer3d_State();
 	
 	gxf_hardware.reset();
 
@@ -521,7 +524,7 @@ void gfx3d_reset()
 	texturePalette = 0;
 	polyAttrPending = 0;
 	mode = 0;
-	u16coord[0] = u16coord[1] = u16coord[2] = u16coord[3] = 0;
+	s16coord[0] = s16coord[1] = s16coord[2] = s16coord[3] = 0;
 	coordind = 0;
 	vtxFormat = 0;
 	memset(trans, 0, sizeof(trans));
@@ -572,8 +575,6 @@ void gfx3d_reset()
 
 	GFX_PIPEclear();
 	GFX_FIFOclear();
-
-	gfx3d.state.invalidateToon = true;
 }
 
 
@@ -581,30 +582,45 @@ void gfx3d_reset()
 //=================================================================================
 //=================================================================================
 
-#define vec3dot(a, b)		(((a[0]) * (b[0])) + ((a[1]) * (b[1])) + ((a[2]) * (b[2])))
+inline float vec3dot(float* a, float* b) {
+	return (((a[0]) * (b[0])) + ((a[1]) * (b[1])) + ((a[2]) * (b[2])));
+}
+
+FORCEINLINE s32 mul_fixed32(s32 a, s32 b)
+{
+	return fx32_shiftdown(fx32_mul(a,b));
+}
+
+FORCEINLINE s32 vec3dot_fixed32(const s32* a, const s32* b) {
+	return fx32_shiftdown(fx32_mul(a[0],b[0]) + fx32_mul(a[1],b[1]) + fx32_mul(a[2],b[2]));
+}
+
+
 #define SUBMITVERTEX(ii, nn) polylist->list[polylist->count].vertIndexes[ii] = tempVertInfo.map[nn];
 //Submit a vertex to the GE
 static void SetVertex()
 {
-	float coord[3] = {
-			float16table[u16coord[0]],
-			float16table[u16coord[1]],
-			float16table[u16coord[2]]
+	s32 coord[3] = {
+		s16coord[0],
+		s16coord[1],
+		s16coord[2]
 	};
 
-	ALIGN(16) float coordTransformed[4] = { coord[0], coord[1], coord[2], 1.f };
+	ALIGN(16) s32 coordTransformed[4] = { coord[0], coord[1], coord[2], (1<<12) };
 
 	if (texCoordinateTransform == 3)
 	{
-		last_s =((coord[0]*mtxCurrent[3][0] +
-					coord[1]*mtxCurrent[3][4] +
-					coord[2]*mtxCurrent[3][8]) + _s * 16.0f) / 16.0f;
-		last_t =((coord[0]*mtxCurrent[3][1] +
-					coord[1]*mtxCurrent[3][5] +
-					coord[2]*mtxCurrent[3][9]) + _t * 16.0f) / 16.0f;
+		//Tested by: Eledees The Adventures of Kai and Zero (E) [title screen and frontend menus]
+		last_s = (s32)(((s64)s16coord[0] * mtxCurrent[3][0] + 
+								(s64)s16coord[1] * mtxCurrent[3][4] + 
+								(s64)s16coord[2] * mtxCurrent[3][8] + 
+								(((s64)(_s))<<24))>>24);
+		last_t = (s32)(((s64)s16coord[0] * mtxCurrent[3][1] + 
+								(s64)s16coord[1] * mtxCurrent[3][5] + 
+								(s64)s16coord[2] * mtxCurrent[3][9] + 
+								(((s64)(_t))<<24))>>24);
 	}
 
-	
 	//refuse to do anything if we have too many verts or polys
 	polygonListCompleted = 0;
 	if(vertlist->count >= VERTLIST_SIZE) 
@@ -618,6 +634,10 @@ static void SetVertex()
 	//when we need to)
 	MatrixMultVec4x4_M2(mtxCurrent[0], coordTransformed);
 
+	//printf("%f %f %f\n",s16coord[0]/4096.0f,s16coord[1]/4096.0f,s16coord[2]/4096.0f);
+	//printf("x %f %f %f %f\n",mtxCurrent[0][0]/4096.0f,mtxCurrent[0][1]/4096.0f,mtxCurrent[0][2]/4096.0f,mtxCurrent[0][3]/4096.0f);
+	//printf(" = %f %f %f %f\n",coordTransformed[0]/4096.0f,coordTransformed[1]/4096.0f,coordTransformed[2]/4096.0f,coordTransformed[3]/4096.0f);
+
 	//TODO - culling should be done here.
 	//TODO - viewport transform?
 
@@ -647,12 +667,12 @@ static void SetVertex()
 	//	//MatrixPrint(mtxCurrent[1]);
 	//}
 
-	vert.texcoord[0] = last_s;
-	vert.texcoord[1] = last_t;
-	vert.coord[0] = coordTransformed[0];
-	vert.coord[1] = coordTransformed[1];
-	vert.coord[2] = coordTransformed[2];
-	vert.coord[3] = coordTransformed[3];
+	vert.texcoord[0] = last_s/16.0f;
+	vert.texcoord[1] = last_t/16.0f;
+	vert.coord[0] = coordTransformed[0]/4096.0f;
+	vert.coord[1] = coordTransformed[1]/4096.0f;
+	vert.coord[2] = coordTransformed[2]/4096.0f;
+	vert.coord[3] = coordTransformed[3]/4096.0f;
 	vert.color[0] = GFX3D_5TO6(colorRGB[0]);
 	vert.color[1] = GFX3D_5TO6(colorRGB[1]);
 	vert.color[2] = GFX3D_5TO6(colorRGB[2]);
@@ -763,25 +783,46 @@ static void gfx3d_glTexImage_cache()
 
 static void gfx3d_glLightDirection_cache(int index)
 {
-	u32 v = lightDirection[index];
+	s32 v = lightDirection[index];
 
-	// Convert format into floating point value
-	cacheLightDirection[index][0] = normalTable[v&1023];
-	cacheLightDirection[index][1] = normalTable[(v>>10)&1023];
-	cacheLightDirection[index][2] = normalTable[(v>>20)&1023];
+	s16 x = ((v<<22)>>22)<<3;
+	s16 y = ((v<<12)>>22)<<3;
+	s16 z = ((v<<2)>>22)<<3;
+
+	cacheLightDirection[index][0] = x;
+	cacheLightDirection[index][1] = y;
+	cacheLightDirection[index][2] = z;
 	cacheLightDirection[index][3] = 0;
 
-	/* Multiply the vector by the directional matrix */
-	MatrixMultVec3x3(mtxCurrent[2], cacheLightDirection[index]);
+	//Multiply the vector by the directional matrix
+	MatrixMultVec3x3_fixed(mtxCurrent[2], cacheLightDirection[index]);
 
-	/* Calculate the half vector */
-	float lineOfSight[4] = {0.0f, 0.0f, -1.0f, 0.0f};
+	//Calculate the half angle vector
+	s32 lineOfSight[4] = {0, 0, (-1)<<12, 0};
 	for(int i = 0; i < 4; i++)
 	{
-		cacheHalfVector[index][i] = ((cacheLightDirection[index][i] + lineOfSight[i]) / 2.0f);
+		cacheHalfVector[index][i] = ((cacheLightDirection[index][i] + lineOfSight[i]));
+	}
+
+	//normalize the half angle vector
+	//can't believe the hardware really does this... but yet it seems...
+	s32 halfLength = ((s32)(sqrt((double)vec3dot_fixed32(cacheHalfVector[index],cacheHalfVector[index]))))<<6;
+
+	if(halfLength!=0)
+	{
+		halfLength = abs(halfLength);
+		halfLength >>= 6;
+		for(int i = 0; i < 4; i++)
+		{
+			s32 temp = cacheHalfVector[index][i];
+			temp <<= 6;
+			temp /= halfLength;
+			cacheHalfVector[index][i] = temp;
+		}
 	}
 }
 
+
 //===============================================================================
 static void gfx3d_glMatrixMode(u32 v)
 {
@@ -818,12 +859,12 @@ static void gfx3d_glPopMatrix(s32 i)
 	//this was necessary to fix sims apartment pets
 	//i = (i<<26)>>26;
 
-	MatrixStackPopMatrix((float*)mtxCurrent[mymode], &mtxStack[mymode], i);
+	MatrixStackPopMatrix(mtxCurrent[mymode], &mtxStack[mymode], i);
 
 	GFX_DELAY(36);
 
 	if (mymode == 2)
-		MatrixStackPopMatrix((float*)mtxCurrent[1], &mtxStack[1], i);
+		MatrixStackPopMatrix(mtxCurrent[1], &mtxStack[1], i);
 
 	MMU_new.gxstat.sb = 1; // set busy
 }
@@ -838,7 +879,7 @@ static void gfx3d_glStoreMatrix(u32 v)
 	if(mymode==0 || mymode==3)
 		v = 0;
 
-	if (v > 31) return;
+	if (v >= 31) return;
 
 	MatrixStackLoadMatrix (&mtxStack[mymode], v, mtxCurrent[mymode]);
 
@@ -858,7 +899,7 @@ static void gfx3d_glRestoreMatrix(u32 v)
 	if(mymode==0 || mymode==3)
 		v = 0;
 
-	if (v > 31) return;
+	if (v >= 31) return;
 
 	MatrixCopy (mtxCurrent[mymode], MatrixStackGetPos(&mtxStack[mymode], v));
 
@@ -885,7 +926,7 @@ static void gfx3d_glLoadIdentity()
 
 static BOOL gfx3d_glLoadMatrix4x4(s32 v)
 {
-	mtxCurrent[mode][ML4x4ind] = (float)((v<<4)>>4);
+	mtxCurrent[mode][ML4x4ind] = v;
 
 	++ML4x4ind;
 	if(ML4x4ind<16) return FALSE;
@@ -893,7 +934,7 @@ static BOOL gfx3d_glLoadMatrix4x4(s32 v)
 
 	GFX_DELAY(19);
 
-	vector_fix2float<4>(mtxCurrent[mode], 4096.f);
+	//vector_fix2float<4>(mtxCurrent[mode], 4096.f);
 
 	if (mode == 2)
 		MatrixCopy (mtxCurrent[1], mtxCurrent[2]);
@@ -904,18 +945,18 @@ static BOOL gfx3d_glLoadMatrix4x4(s32 v)
 
 static BOOL gfx3d_glLoadMatrix4x3(s32 v)
 {
-	mtxCurrent[mode][ML4x3ind] = (float)((v<<4)>>4);
+	mtxCurrent[mode][ML4x3ind] = v;
 
 	ML4x3ind++;
 	if((ML4x3ind & 0x03) == 3) ML4x3ind++;
 	if(ML4x3ind<16) return FALSE;
 	ML4x3ind = 0;
 
-	vector_fix2float<4>(mtxCurrent[mode], 4096.f);
+	//vector_fix2float<4>(mtxCurrent[mode], 4096.f);
 
 	//fill in the unusued matrix values
-	mtxCurrent[mode][3] = mtxCurrent[mode][7] = mtxCurrent[mode][11] = 0.f;
-	mtxCurrent[mode][15] = 1.f;
+	mtxCurrent[mode][3] = mtxCurrent[mode][7] = mtxCurrent[mode][11] = 0;
+	mtxCurrent[mode][15] = (1<<12);
 
 	GFX_DELAY(30);
 
@@ -927,7 +968,7 @@ static BOOL gfx3d_glLoadMatrix4x3(s32 v)
 
 static BOOL gfx3d_glMultMatrix4x4(s32 v)
 {
-	mtxTemporal[MM4x4ind] = (float)((v<<4)>>4);
+	mtxTemporal[MM4x4ind] = v;
 
 	MM4x4ind++;
 	if(MM4x4ind<16) return FALSE;
@@ -935,7 +976,7 @@ static BOOL gfx3d_glMultMatrix4x4(s32 v)
 
 	GFX_DELAY(35);
 
-	vector_fix2float<4>(mtxTemporal, 4096.f);
+	//vector_fix2float<4>(mtxTemporal, 4096.f);
 
 	MatrixMultiply (mtxCurrent[mode], mtxTemporal);
 
@@ -953,7 +994,7 @@ static BOOL gfx3d_glMultMatrix4x4(s32 v)
 
 static BOOL gfx3d_glMultMatrix4x3(s32 v)
 {
-	mtxTemporal[MM4x3ind] = (float)((v<<4)>>4);
+	mtxTemporal[MM4x3ind] = v;
 
 	MM4x3ind++;
 	if((MM4x3ind & 0x03) == 3) MM4x3ind++;
@@ -962,11 +1003,11 @@ static BOOL gfx3d_glMultMatrix4x3(s32 v)
 
 	GFX_DELAY(31);
 
-	vector_fix2float<4>(mtxTemporal, 4096.f);
+	//vector_fix2float<4>(mtxTemporal, 4096.f);
 
 	//fill in the unusued matrix values
-	mtxTemporal[3] = mtxTemporal[7] = mtxTemporal[11] = 0.f;
-	mtxTemporal[15] = 1.f;
+	mtxTemporal[3] = mtxTemporal[7] = mtxTemporal[11] = 0;
+	mtxTemporal[15] = 1<<12;
 
 	MatrixMultiply (mtxCurrent[mode], mtxTemporal);
 
@@ -985,7 +1026,7 @@ static BOOL gfx3d_glMultMatrix4x3(s32 v)
 
 static BOOL gfx3d_glMultMatrix3x3(s32 v)
 {
-	mtxTemporal[MM3x3ind] = (float)((v<<4)>>4);
+	mtxTemporal[MM3x3ind] = v;
 
 
 	MM3x3ind++;
@@ -995,11 +1036,11 @@ static BOOL gfx3d_glMultMatrix3x3(s32 v)
 
 	GFX_DELAY(28);
 
-	vector_fix2float<3>(mtxTemporal, 4096.f);
+	//vector_fix2float<3>(mtxTemporal, 4096.f);
 
 	//fill in the unusued matrix values
 	mtxTemporal[3] = mtxTemporal[7] = mtxTemporal[11] = 0;
-	mtxTemporal[15] = 1;
+	mtxTemporal[15] = 1<<12;
 	mtxTemporal[12] = mtxTemporal[13] = mtxTemporal[14] = 0;
 
 	MatrixMultiply (mtxCurrent[mode], mtxTemporal);
@@ -1020,7 +1061,7 @@ static BOOL gfx3d_glMultMatrix3x3(s32 v)
 
 static BOOL gfx3d_glScale(s32 v)
 {
-	scale[scaleind] = fix2float(v);
+	scale[scaleind] = v;
 
 	++scaleind;
 
@@ -1042,7 +1083,7 @@ static BOOL gfx3d_glScale(s32 v)
 
 static BOOL gfx3d_glTranslate(s32 v)
 {
-	trans[transind] = fix2float(v);
+	trans[transind] = v;
 
 	++transind;
 
@@ -1072,92 +1113,98 @@ static void gfx3d_glColor3b(u32 v)
 	GFX_DELAY(1);
 }
 
-static void gfx3d_glNormal(u32 v)
+static void gfx3d_glNormal(s32 v)
 {
-	int i,c;
-	ALIGN(16) float normal[4] = { normalTable[v&1023],
-						normalTable[(v>>10)&1023],
-						normalTable[(v>>20)&1023],
-						1};
+	s16 nx = ((v<<22)>>22)<<3;
+	s16 ny = ((v<<12)>>22)<<3;
+	s16 nz = ((v<<2)>>22)<<3;
 
 	if (texCoordinateTransform == 2)
 	{
-		last_s =(	(normal[0] *mtxCurrent[3][0] + normal[1] *mtxCurrent[3][4] +
-					 normal[2] *mtxCurrent[3][8]) + (_s*16.0f)) / 16.0f;
-		last_t =(	(normal[0] *mtxCurrent[3][1] + normal[1] *mtxCurrent[3][5] +
-					 normal[2] *mtxCurrent[3][9]) + (_t*16.0f)) / 16.0f;
+		//SM64 highlight rendered star in main menu tests this
+		last_s = (s32)(((s64)nx * mtxCurrent[3][0] + (s64)ny * mtxCurrent[3][4] + (s64)nz * mtxCurrent[3][8] + (_s<<24))>>24);
+		last_t = (s32)(((s64)nx * mtxCurrent[3][1] + (s64)ny * mtxCurrent[3][5] + (s64)nz * mtxCurrent[3][9] + (_t<<24))>>24);
 	}
 
-	//use the current normal transform matrix
-	MatrixMultVec3x3 (mtxCurrent[2], normal);
+	CACHE_ALIGN s32 normal[4] =  { nx,ny,nz,(1<<12) };
+
+	MatrixMultVec3x3_fixed(mtxCurrent[2],normal);
 
 	//apply lighting model
+	u8 diffuse[3] = {
+		(dsDiffuse)&0x1F,
+		(dsDiffuse>>5)&0x1F,
+		(dsDiffuse>>10)&0x1F };
+
+	u8 ambient[3] = {
+		(dsAmbient)&0x1F,
+		(dsAmbient>>5)&0x1F,
+		(dsAmbient>>10)&0x1F };
+
+	u8 emission[3] = {
+		(dsEmission)&0x1F,
+		(dsEmission>>5)&0x1F,
+		(dsEmission>>10)&0x1F };
+
+	u8 specular[3] = {
+		(dsSpecular)&0x1F,
+		(dsSpecular>>5)&0x1F,
+		(dsSpecular>>10)&0x1F };
+
+	int vertexColor[3] = { emission[0], emission[1], emission[2] };
+
+	for(int i=0; i<4; i++)
 	{
-		u8 diffuse[3] = {
-			(dsDiffuse)&0x1F,
-			(dsDiffuse>>5)&0x1F,
-			(dsDiffuse>>10)&0x1F };
+		if(!((lightMask>>i)&1)) continue;
 
-		u8 ambient[3] = {
-			(dsAmbient)&0x1F,
-			(dsAmbient>>5)&0x1F,
-			(dsAmbient>>10)&0x1F };
+		u8 _lightColor[3] = {
+			(lightColor[i])&0x1F,
+			(lightColor[i]>>5)&0x1F,
+			(lightColor[i]>>10)&0x1F };
 
-		u8 emission[3] = {
-			(dsEmission)&0x1F,
-			(dsEmission>>5)&0x1F,
-			(dsEmission>>10)&0x1F };
+		//This formula is the one used by the DS
+		//Reference : http://nocash.emubase.de/gbatek.htm#ds3dpolygonlightparameters
+		s32 fixed_diffuse = std::max(0,-vec3dot_fixed32(cacheLightDirection[i],normal));
+		
+		//todo - this could be cached in this form
+		s32 fixedTempNegativeHalf[] = {-cacheHalfVector[i][0],-cacheHalfVector[i][1],-cacheHalfVector[i][2],-cacheHalfVector[i][3]};
+		s32 dot = vec3dot_fixed32(fixedTempNegativeHalf, normal);
 
-		u8 specular[3] = {
-			(dsSpecular)&0x1F,
-			(dsSpecular>>5)&0x1F,
-			(dsSpecular>>10)&0x1F };
-
-		int vertexColor[3] = { emission[0], emission[1], emission[2] };
-
-		for(i=0; i<4; i++)
+		s32 fixedshininess = 0;
+		if(dot>0) //prevent shininess on opposite side
 		{
-			if(!((lightMask>>i)&1)) continue;
-
-			u8 _lightColor[3] = {
-				(lightColor[i])&0x1F,
-				(lightColor[i]>>5)&0x1F,
-				(lightColor[i]>>10)&0x1F };
-
-			/* This formula is the one used by the DS */
-			/* Reference : http://nocash.emubase.de/gbatek.htm#ds3dpolygonlightparameters */
-
-			float diffuseLevel = std::max(0.0f, -vec3dot(cacheLightDirection[i], normal));
-			float shininessLevel = pow(std::max(0.0f, vec3dot(-cacheHalfVector[i], normal)), 2);
-
-			if(dsSpecular & 0x8000)
-			{
-				int shininessIndex = (int)(shininessLevel * 128);
-				if(shininessIndex >= (int)ARRAY_SIZE(gfx3d.state.shininessTable)) {
-					//we can't print this right now, because when a game triggers this it triggers it _A_LOT_
-					//so wait until we have per-frame diagnostics.
-					//this was tested using Princess Debut (US) after proceeding through the intro and getting the tiara.
-					//After much research, I determined that this was caused by the game feeding in a totally jacked matrix
-					//to mult4x4 from 0x02129B80 (after feeding two other valid matrices)
-					//the game seems to internally index these as: ?, 0x37, 0x2B <-- error
-					//but, man... this is seriously messed up. there must be something going wrong.
-					//maybe it has something to do with what looks like a mirror room effect that is going on during this time?
-					//PROGINFO("ERROR: shininess table out of bounds.\n  maybe an emulator error; maybe a non-unit normal; setting to 0\n");
-					shininessIndex = 0;
-				}
-				shininessLevel = gfx3d.state.shininessTable[shininessIndex];
-			}
-
-			for(c = 0; c < 3; c++)
-			{
-				vertexColor[c] += (int)(((specular[c] * _lightColor[c] * shininessLevel)
-					+ (diffuse[c] * _lightColor[c] * diffuseLevel)
-					+ (ambient[c] * _lightColor[c])) / 31.0f);
-			}
+			//we have cos(a). it seems that we need cos(2a). trig identity is a fast way to get it.
+			//cos^2(a)=(1/2)(1+cos(2a))
+			//2*cos^2(a)-1=cos(2a)
+			fixedshininess = 2*mul_fixed32(dot,dot)-4096;
+			//gbatek is almost right but not quite!
 		}
 
-		for(c=0;c<3;c++)
-			colorRGB[c] = std::min(31,vertexColor[c]);
+		//this seems to need to be saturated, or else the table will overflow.
+		//even without a table, failure to saturate is bad news
+		fixedshininess = std::min(fixedshininess,4095);
+		fixedshininess = std::max(fixedshininess,0);
+		
+		if(dsSpecular & 0x8000)
+		{
+			//shininess is 20.12 fixed point, so >>5 gives us .7 which is 128 entries
+			//the entries are 8bits each so <<4 gives us .12 again, compatible with the lighting formulas below
+			//(according to other normal nds procedures, we might should fill the bottom bits with 1 or 0 according to rules...)
+			fixedshininess = gfx3d.state.shininessTable[fixedshininess>>5]<<4;
+		}
+
+		for(int c = 0; c < 3; c++)
+		{
+			s32 specComp = ((specular[c] * _lightColor[c] * fixedshininess)>>17);  //5 bits for color*color and 12 bits for the shininess
+			s32 diffComp = ((diffuse[c] * _lightColor[c] * fixed_diffuse)>>17); //5bits for the color*color and 12 its for the diffuse
+			s32 ambComp = ((ambient[c] * _lightColor[c])>>5); //5bits for color*color
+			vertexColor[c] += specComp + diffComp + ambComp;
+		}
+	}
+
+	for(int c=0;c<3;c++)
+	{
+		colorRGB[c] = std::min(31,vertexColor[c]);
 	}
 
 	GFX_DELAY(9);
@@ -1167,22 +1214,18 @@ static void gfx3d_glNormal(u32 v)
 	GFX_DELAY_M2((lightMask>>3) & 0x01);
 }
 
-static void gfx3d_glTexCoord(u32 val)
+static void gfx3d_glTexCoord(s32 val)
 {
-	_t = (s16)(val>>16);
-	_s = (s16)(val&0xFFFF);
-
-	_s /= 16.0f;
-	_t /= 16.0f;
+	_s = ((val<<16)>>16);
+	_t = (val>>16);
 
 	if (texCoordinateTransform == 1)
 	{
-		last_s =_s*mtxCurrent[3][0] + _t*mtxCurrent[3][4] +
-				0.0625f*mtxCurrent[3][8] + 0.0625f*mtxCurrent[3][12];
-		last_t =_s*mtxCurrent[3][1] + _t*mtxCurrent[3][5] +
-				0.0625f*mtxCurrent[3][9] + 0.0625f*mtxCurrent[3][13];
+		//dragon quest 4 overworld will test this
+		last_s = (s32)(((s64)(_s<<12) * mtxCurrent[3][0] + (s64)(_t<<12) * mtxCurrent[3][4] + ((s64)mtxCurrent[3][8]<<12) + ((s64)mtxCurrent[3][12]<<12))>>24);
+		last_t = (s32)(((s64)(_s<<12) * mtxCurrent[3][1] + (s64)(_t<<12) * mtxCurrent[3][5] + ((s64)mtxCurrent[3][9]<<12) + ((s64)mtxCurrent[3][13]<<12))>>24);
 	}
-	else
+	else if(texCoordinateTransform == 0)
 	{
 		last_s=_s;
 		last_t=_t;
@@ -1190,21 +1233,18 @@ static void gfx3d_glTexCoord(u32 val)
 	GFX_DELAY(1);
 }
 
-static BOOL gfx3d_glVertex16b(unsigned int v)
+static BOOL gfx3d_glVertex16b(s32 v)
 {
 	if(coordind==0)
 	{
-		//coord[0]		= float16table[v&0xFFFF];
-		//coord[1]		= float16table[v>>16];
-		u16coord[0] = v&0xFFFF;
-		u16coord[1] = (v>>16)&0xFFFF;
+		s16coord[0] = (v<<16)>>16;
+		s16coord[1] = (v>>16)&0xFFFF;
 
 		++coordind;
 		return FALSE;
 	}
 
-	//coord[2]	  = float16table[v&0xFFFF];
-	u16coord[2] = v&0xFFFF;
+	s16coord[2] = (v<<16)>>16;
 
 	coordind = 0;
 	SetVertex ();
@@ -1213,40 +1253,37 @@ static BOOL gfx3d_glVertex16b(unsigned int v)
 	return TRUE;
 }
 
-static void gfx3d_glVertex10b(u32 v)
+static void gfx3d_glVertex10b(s32 v)
 {
-	//coord[0]		= float10Table[v&1023];
-	//coord[1]		= float10Table[(v>>10)&1023];
-	//coord[2]		= float10Table[(v>>20)&1023];
-	u16coord[0] = (v&1023)<<6;
-	u16coord[1] = ((v>>10)&1023)<<6;
-	u16coord[2] = ((v>>20)&1023)<<6;
+	//TODO TODO TODO - contemplate the sign extension - shift in zeroes or ones? zeroes is certainly more normal..
+	s16coord[0] = ((v<<22)>>22)<<6;
+	s16coord[1] = ((v<<12)>>22)<<6;
+	s16coord[2] = ((v<<2)>>22)<<6;
 
 	GFX_DELAY(8);
 	SetVertex ();
 }
 
-static void gfx3d_glVertex3_cord(unsigned int one, unsigned int two, unsigned int v)
+template<int ONE, int TWO>
+static void gfx3d_glVertex3_cord(s32 v)
 {
-	//coord[one]		= float16table[v&0xffff];
-	//coord[two]		= float16table[v>>16];
-	u16coord[one]		= v&0xffff;
-	u16coord[two]		= (v>>16)&0xFFFF;
+	s16coord[ONE]		= (v<<16)>>16;
+	s16coord[TWO]		= (v>>16);
 
 	SetVertex ();
 
 	GFX_DELAY(8);
 }
 
-static void gfx3d_glVertex_rel(u32 v)
+static void gfx3d_glVertex_rel(s32 v)
 {
-	//coord[0]		+= float10RelTable[v&1023];
-	//coord[1]		+= float10RelTable[(v>>10)&1023];
-	//coord[2]		+= float10RelTable[(v>>20)&1023];
+	s16 x = ((v<<22)>>22);
+	s16 y = ((v<<12)>>22);
+	s16 z = ((v<<2)>>22);
 
-	u16coord[0] += (u16)(((s16)((v&1023)<<6))>>6);
-	u16coord[1] += (u16)(((s16)(((v>>10)&1023)<<6))>>6);
-	u16coord[2] += (u16)(((s16)(((v>>20)&1023)<<6))>>6);
+	s16coord[0] += x;
+	s16coord[1] += y;
+	s16coord[2] += z;
 
 
 	SetVertex ();
@@ -1317,7 +1354,7 @@ static void gfx3d_glLightDirection (u32 v)
 {
 	int index = v>>30;
 
-	lightDirection[index] = v;
+	lightDirection[index] = (s32)(v&0x3FFFFFFF);
 	gfx3d_glLightDirection_cache(index);
 	GFX_DELAY(6);
 }
@@ -1331,10 +1368,10 @@ static void gfx3d_glLightColor (u32 v)
 
 static BOOL gfx3d_glShininess (u32 val)
 {
-	gfx3d.state.shininessTable[shininessInd++] = ((val & 0xFF) / 256.0f);
-	gfx3d.state.shininessTable[shininessInd++] = (((val >> 8) & 0xFF) / 256.0f);
-	gfx3d.state.shininessTable[shininessInd++] = (((val >> 16) & 0xFF) / 256.0f);
-	gfx3d.state.shininessTable[shininessInd++] = (((val >> 24) & 0xFF) / 256.0f);
+	gfx3d.state.shininessTable[shininessInd++] = ((val & 0xFF));
+	gfx3d.state.shininessTable[shininessInd++] = (((val >> 8) & 0xFF));
+	gfx3d.state.shininessTable[shininessInd++] = (((val >> 16) & 0xFF));
+	gfx3d.state.shininessTable[shininessInd++] = (((val >> 24) & 0xFF));
 
 	if (shininessInd < 128) return FALSE;
 	shininessInd = 0;
@@ -1371,6 +1408,7 @@ static void gfx3d_glViewPort(u32 v)
 
 static BOOL gfx3d_glBoxTest(u32 v)
 {
+	//printf("boxtest\n");
 	MMU_new.gxstat.tr = 0;		// clear boxtest bit
 	MMU_new.gxstat.tb = 1;		// busy
 
@@ -1467,11 +1505,10 @@ static BOOL gfx3d_glBoxTest(u32 v)
 	for(int i=0;i<8;i++) {
 		//MatrixMultVec4x4_M2(mtxCurrent[0], verts[i].coord);
 
-		//yuck.. cant use the sse2 accelerated ones because vert.coords is not cache aligned or something
-		//i dunno
-		
-		_NOSSE_MatrixMultVec4x4(mtxCurrent[1],verts[i].coord);
-		_NOSSE_MatrixMultVec4x4(mtxCurrent[0],verts[i].coord);
+		CACHE_ALIGN float temp1[16] = {mtxCurrent[1][0]/4096.0f,mtxCurrent[1][1]/4096.0f,mtxCurrent[1][2]/4096.0f,mtxCurrent[1][3]/4096.0f,mtxCurrent[1][4]/4096.0f,mtxCurrent[1][5]/4096.0f,mtxCurrent[1][6]/4096.0f,mtxCurrent[1][7]/4096.0f,mtxCurrent[1][8]/4096.0f,mtxCurrent[1][9]/4096.0f,mtxCurrent[1][10]/4096.0f,mtxCurrent[1][11]/4096.0f,mtxCurrent[1][12]/4096.0f,mtxCurrent[1][13]/4096.0f,mtxCurrent[1][14]/4096.0f,mtxCurrent[1][15]/4096.0f};
+		CACHE_ALIGN float temp0[16] = {mtxCurrent[0][0]/4096.0f,mtxCurrent[0][1]/4096.0f,mtxCurrent[0][2]/4096.0f,mtxCurrent[0][3]/4096.0f,mtxCurrent[0][4]/4096.0f,mtxCurrent[0][5]/4096.0f,mtxCurrent[0][6]/4096.0f,mtxCurrent[0][7]/4096.0f,mtxCurrent[0][8]/4096.0f,mtxCurrent[0][9]/4096.0f,mtxCurrent[0][10]/4096.0f,mtxCurrent[0][11]/4096.0f,mtxCurrent[0][12]/4096.0f,mtxCurrent[0][13]/4096.0f,mtxCurrent[0][14]/4096.0f,mtxCurrent[0][15]/4096.0f};
+		_NOSSE_MatrixMultVec4x4(temp1,verts[i].coord);
+		_NOSSE_MatrixMultVec4x4(temp0,verts[i].coord);
 	}
 
 	//clip each poly
@@ -1505,6 +1542,7 @@ static BOOL gfx3d_glBoxTest(u32 v)
 
 static BOOL gfx3d_glPosTest(u32 v)
 {
+	//printf("postest\n");
 	//this is apparently tested by transformers decepticons and ultimate spiderman
 
 	//printf("POSTEST\n");
@@ -1518,8 +1556,11 @@ static BOOL gfx3d_glPosTest(u32 v)
 	
 	PTcoords[3] = 1.0f;
 
-	MatrixMultVec4x4(mtxCurrent[1], PTcoords);
-	MatrixMultVec4x4(mtxCurrent[0], PTcoords);
+	CACHE_ALIGN float temp1[16] = {mtxCurrent[1][0]/4096.0f,mtxCurrent[1][1]/4096.0f,mtxCurrent[1][2]/4096.0f,mtxCurrent[1][3]/4096.0f,mtxCurrent[1][4]/4096.0f,mtxCurrent[1][5]/4096.0f,mtxCurrent[1][6]/4096.0f,mtxCurrent[1][7]/4096.0f,mtxCurrent[1][8]/4096.0f,mtxCurrent[1][9]/4096.0f,mtxCurrent[1][10]/4096.0f,mtxCurrent[1][11]/4096.0f,mtxCurrent[1][12]/4096.0f,mtxCurrent[1][13]/4096.0f,mtxCurrent[1][14]/4096.0f,mtxCurrent[1][15]/4096.0f};
+	CACHE_ALIGN float temp0[16] = {mtxCurrent[0][0]/4096.0f,mtxCurrent[0][1]/4096.0f,mtxCurrent[0][2]/4096.0f,mtxCurrent[0][3]/4096.0f,mtxCurrent[0][4]/4096.0f,mtxCurrent[0][5]/4096.0f,mtxCurrent[0][6]/4096.0f,mtxCurrent[0][7]/4096.0f,mtxCurrent[0][8]/4096.0f,mtxCurrent[0][9]/4096.0f,mtxCurrent[0][10]/4096.0f,mtxCurrent[0][11]/4096.0f,mtxCurrent[0][12]/4096.0f,mtxCurrent[0][13]/4096.0f,mtxCurrent[0][14]/4096.0f,mtxCurrent[0][15]/4096.0f};
+
+	MatrixMultVec4x4(temp1, PTcoords);
+	MatrixMultVec4x4(temp0, PTcoords);
 
 	MMU_new.gxstat.tb = 0;
 
@@ -1530,6 +1571,7 @@ static BOOL gfx3d_glPosTest(u32 v)
 
 static void gfx3d_glVecTest(u32 v)
 {
+	//printf("vectest\n");
 	GFX_DELAY(5);
 
 	//this is tested by phoenix wright in its evidence inspector modelviewer
@@ -1541,7 +1583,8 @@ static void gfx3d_glVecTest(u32 v)
 						normalTable[(v>>20)&1023],
 						0};
 
-	MatrixMultVec4x4(mtxCurrent[2], normal);
+	CACHE_ALIGN float temp[16] = {mtxCurrent[2][0]/4096.0f,mtxCurrent[2][1]/4096.0f,mtxCurrent[2][2]/4096.0f,mtxCurrent[2][3]/4096.0f,mtxCurrent[2][4]/4096.0f,mtxCurrent[2][5]/4096.0f,mtxCurrent[2][6]/4096.0f,mtxCurrent[2][7]/4096.0f,mtxCurrent[2][8]/4096.0f,mtxCurrent[2][9]/4096.0f,mtxCurrent[2][10]/4096.0f,mtxCurrent[2][11]/4096.0f,mtxCurrent[2][12]/4096.0f,mtxCurrent[2][13]/4096.0f,mtxCurrent[2][14]/4096.0f,mtxCurrent[2][15]/4096.0f};
+	MatrixMultVec4x4(temp, normal);
 
 	s16 x = (s16)(normal[0]*4096);
 	s16 y = (s16)(normal[1]*4096);
@@ -1600,23 +1643,26 @@ int gfx3d_GetNumVertex()
 
 void gfx3d_UpdateToonTable(u8 offset, u16 val)
 {
-	gfx3d.state.rgbToonTable[offset] = RGB15TO32(val, 255);
 	gfx3d.state.invalidateToon = true;
+	gfx3d.state.u16ToonTable[offset] = val;
+	//printf("toon %d set to %04X\n",offset,val);
 }
 
 void gfx3d_UpdateToonTable(u8 offset, u32 val)
 {
 	//C.O.P. sets toon table via this method
-	gfx3d.state.rgbToonTable[offset] = RGB15TO32(val & 0xFFFF, 255);
-	gfx3d.state.rgbToonTable[offset+1] = RGB15TO32(val >> 16, 255);
 	gfx3d.state.invalidateToon = true;
+	gfx3d.state.u16ToonTable[offset] = val & 0xFFFF;
+	gfx3d.state.u16ToonTable[offset+1] = val >> 16;
+	//printf("toon %d set to %04X\n",offset,gfx3d.state.u16ToonTable[offset]);
+	//printf("toon %d set to %04X\n",offset+1,gfx3d.state.u16ToonTable[offset+1]);
 }
 
 s32 gfx3d_GetClipMatrix (unsigned int index)
 {
-	float val = MatrixGetMultipliedIndex (index, mtxCurrent[0], mtxCurrent[1]);
+	s32 val = MatrixGetMultipliedIndex (index, mtxCurrent[0], mtxCurrent[1]);
 
-	val *= (1<<12);
+	//val *= (1<<12);
 
 	return (s32)val;
 }
@@ -1625,7 +1671,8 @@ s32 gfx3d_GetDirectionalMatrix (unsigned int index)
 {
 	int _index = (((index / 3) * 4) + (index % 3));
 
-	return (s32)(mtxCurrent[2][_index]*(1<<12));
+	//return (s32)(mtxCurrent[2][_index]*(1<<12));
+	return mtxCurrent[2][_index];
 }
 
 void gfx3d_glAlphaFunc(u32 v)
@@ -1638,7 +1685,9 @@ unsigned int gfx3d_glGetPosRes(unsigned int index)
 	return (unsigned int)(PTcoords[index] * 4096.0f);
 }
 
-void log3D(u8 cmd, u32 param)
+//#define _3D_LOG_EXEC
+#ifdef _3D_LOG_EXEC
+static void log3D(u8 cmd, u32 param)
 {
 	INFO("3D command 0x%02X: ", cmd);
 	switch (cmd)
@@ -1760,8 +1809,8 @@ void log3D(u8 cmd, u32 param)
 		}
 		printf("\t\t(FIFO size %i)\n", gxFIFO.size);
 }
+#endif
 
-//#define _3D_LOG_EXEC
 static void gfx3d_execute(u8 cmd, u32 param)
 {
 #ifdef _3D_LOG_EXEC
@@ -1826,13 +1875,13 @@ static void gfx3d_execute(u8 cmd, u32 param)
 			gfx3d_glVertex10b(param);
 		break;
 		case 0x25:		// VTX_XY - Set Vertex XY Coordinates (W)
-			gfx3d_glVertex3_cord(0, 1, param);
+			gfx3d_glVertex3_cord<0,1>(param);
 		break;
 		case 0x26:		// VTX_XZ - Set Vertex XZ Coordinates (W)
-			gfx3d_glVertex3_cord(0, 2, param);
+			gfx3d_glVertex3_cord<0,2>(param);
 		break;
 		case 0x27:		// VTX_YZ - Set Vertex YZ Coordinates (W)
-			gfx3d_glVertex3_cord(1, 2, param);
+			gfx3d_glVertex3_cord<1,2>(param);
 		break;
 		case 0x28:		// VTX_DIFF - Set Relative Vertex Coordinates (W)
 			gfx3d_glVertex_rel(param);
@@ -1928,23 +1977,6 @@ void gfx3d_execute3D()
 		} else break;
 	}
 
-
-	//i thought it might be right to move these here, but it didnt help.
-	//maybe its a good idea for later.
-
-	//if(gxFIFO.size <= 127)
-	//{
-	//	if(gxstat.gxfifo_irq == 1) 
-	//		setIF(0, (1<<21)); //the half gxfifo irq
-	//
-	//	//might need to trigger a gxfifo dma
-	//	triggerDma(EDMAMode_GXFifo);
-	//}
-
-	//if(gxFIFO.size == 0) {
-	//	if(gxstat.gxfifo_irq == 2) 
-	//	setIF(0, (1<<21)); //the empty gxfifo irq
-	//}
 }
 
 void gfx3d_glFlush(u32 v)
@@ -2016,6 +2048,9 @@ static void gfx3d_doFlush()
 	gfx3d.state.wbuffer = BIT1(gfx3d.state.activeFlushCommand);
 
 	gfx3d.renderState = gfx3d.state;
+
+	if(!CommonSettings.GFX3D_Texture)
+		gfx3d.renderState.enableTexturing = false;
 	
 	gfx3d.state.activeFlushCommand = gfx3d.state.pendingFlushCommand;
 
@@ -2078,11 +2113,11 @@ static void gfx3d_doFlush()
 
 	if(driver->view3d->IsRunning())
 	{
-		viewer3d_state.frameNumber = currFrameCounter;
-		viewer3d_state.state = gfx3d.state;
-		viewer3d_state.polylist = *gfx3d.polylist;
-		viewer3d_state.vertlist = *gfx3d.vertlist;
-		viewer3d_state.indexlist = gfx3d.indexlist;
+		viewer3d_state->frameNumber = currFrameCounter;
+		viewer3d_state->state = gfx3d.state;
+		viewer3d_state->polylist = *gfx3d.polylist;
+		viewer3d_state->vertlist = *gfx3d.vertlist;
+		viewer3d_state->indexlist = gfx3d.indexlist;
 		driver->view3d->NewFrame();
 	}
 
@@ -2197,13 +2232,19 @@ void gfx3d_Control(u32 v)
 //other misc stuff
 void gfx3d_glGetMatrix(unsigned int m_mode, int index, float* dest)
 {
-	if(index == -1)
-	{
-		MatrixCopy(dest, mtxCurrent[m_mode]);
-		return;
-	}
+	//if(index == -1)
+	//{
+	//	MatrixCopy(dest, mtxCurrent[m_mode]);
+	//	return;
+	//}
 
-	MatrixCopy(dest, MatrixStackGetPos(&mtxStack[m_mode], index));
+	//MatrixCopy(dest, MatrixStackGetPos(&mtxStack[m_mode], index));
+	s32* src;
+	if(index==-1)
+		src = mtxCurrent[m_mode];
+	else src=MatrixStackGetPos(&mtxStack[m_mode],index);
+	for(int i=0;i<16;i++)
+		dest[i] = src[i]/4096.0f;
 }
 
 void gfx3d_glGetLightDirection(unsigned int index, unsigned int* dest)
@@ -2259,7 +2300,7 @@ SFORMAT SF_GFX3D[]={
 	{ "MM4I", 1, 1, &MM4x4ind},
 	{ "MM3I", 1, 1, &MM4x3ind},
 	{ "MMxI", 1, 1, &MM3x3ind},
-	{ "GSCO", 4, 1, u16coord},
+	{ "GSCO", 4, 1, s16coord},
 	{ "GCOI", 1, 1, &coordind},
 	{ "GVFM", 4, 1, &vtxFormat},
 	{ "GTRN", 4, 4, trans},
@@ -2315,8 +2356,8 @@ SFORMAT SF_GFX3D[]={
 	{ "GSCD", 4, 1, &gfx3d.state.clearDepth},
 	{ "GSFC", 4, 4, &gfx3d.state.fogColor},
 	{ "GSFO", 4, 1, &gfx3d.state.fogOffset},
-	{ "GST3", 4, 32, gfx3d.state.rgbToonTable},
-	{ "GSST", 4, 128, &gfx3d.state.shininessTable[0]},
+	{ "GST4", 2, 32, gfx3d.state.u16ToonTable},
+	{ "GSSU", 1, 128, &gfx3d.state.shininessTable[0]},
 	{ "GSSI", 4, 1, &shininessInd},
 	{ "GSAF", 4, 1, &gfx3d.state.activeFlushCommand},
 	{ "GSPF", 4, 1, &gfx3d.state.pendingFlushCommand},
