@@ -73,6 +73,9 @@
 //produce a 32bpp color from a DS RGB16
 #define RGB16TO32(col,alpha) (((alpha)<<24) | ((((col) & 0x7C00)>>7)<<16) | ((((col) & 0x3E0)>>2)<<8) | (((col) & 0x1F)<<3))
 
+//produce a 32bpp color from a ds RGB15, using a table
+#define RGB15TO32_NOALPHA(col) ( color_15bit_to_24bit[col&0x7FFF] )
+
 //produce a 32bpp color from a ds RGB15 plus an 8bit alpha, using a table
 #define RGB15TO32(col,alpha8) ( ((alpha8)<<24) | color_15bit_to_24bit[col&0x7FFF] )
 
@@ -103,6 +106,7 @@ inline u32 RGB15TO6665(u16 col, u8 alpha5)
 
 //produce a 15bpp color from individual 5bit components
 #define R5G5B5TORGB15(r,g,b) ((r)|((g)<<5)|((b)<<10))
+#define RGB15TO32_NOALPHA(col) ( color_15bit_to_24bit[col&0x7FFF] )
 
 //produce a 16bpp color from individual 5bit components
 #define R6G6B6TORGB15(r,g,b) ((r>>1)|((g&0x3E)<<4)|((b&0x3E)<<9))
@@ -317,8 +321,8 @@ struct GFX3D_State
 		for(u32 i=0;i<ARRAY_SIZE(shininessTable);i++)
 			shininessTable[i] = 0;
 
-		for(u32 i=0;i<ARRAY_SIZE(rgbToonTable);i++)
-			rgbToonTable[i] = 0;
+		for(u32 i=0;i<ARRAY_SIZE(u16ToonTable);i++)
+			u16ToonTable[i] = 0;
 	}
 
 	BOOL enableTexturing, enableAlphaTest, enableAlphaBlending, 
@@ -345,8 +349,8 @@ struct GFX3D_State
 	u32 fogShift;
 
 	bool invalidateToon;
-	u32 rgbToonTable[32];
-	float shininessTable[128];
+	u16 u16ToonTable[32];
+	u8 shininessTable[128];
 };
 
 struct Viewer3d_State
@@ -358,7 +362,7 @@ struct Viewer3d_State
 	INDEXLIST indexlist;
 };
 
-extern Viewer3d_State viewer3d_state;
+extern Viewer3d_State* viewer3d_state;
 
 struct GFX3D
 {
