@@ -39,6 +39,29 @@ void _NOSSE_MatrixMultVec4x4 (const float *matrix, float *vecPtr)
 	vecPtr[3] = x * matrix[3] + y * matrix[7] + z * matrix[11] + w * matrix[15];
 }
 
+void MatrixMultVec4x4 (const s32 *matrix, s32 *vecPtr)
+{
+	const s32 x = vecPtr[0];
+	const s32 y = vecPtr[1];
+	const s32 z = vecPtr[2];
+	const s32 w = vecPtr[3];
+
+	vecPtr[0] = fx32_shiftdown(fx32_mul(x,matrix[0]) + fx32_mul(y,matrix[4]) + fx32_mul(z,matrix [8]) + fx32_mul(w,matrix[12]));
+	vecPtr[1] = fx32_shiftdown(fx32_mul(x,matrix[1]) + fx32_mul(y,matrix[5]) + fx32_mul(z,matrix[ 9]) + fx32_mul(w,matrix[13]));
+	vecPtr[2] = fx32_shiftdown(fx32_mul(x,matrix[2]) + fx32_mul(y,matrix[6]) + fx32_mul(z,matrix[10]) + fx32_mul(w,matrix[14]));
+	vecPtr[3] = fx32_shiftdown(fx32_mul(x,matrix[3]) + fx32_mul(y,matrix[7]) + fx32_mul(z,matrix[11]) + fx32_mul(w,matrix[15]));
+}
+
+void MatrixMultVec3x3_fixed(const s32 *matrix, s32 *vecPtr)
+{
+	const s32 x = vecPtr[0];
+	const s32 y = vecPtr[1];
+	const s32 z = vecPtr[2];
+
+	vecPtr[0] = fx32_shiftdown(fx32_mul(x,matrix[0]) + fx32_mul(y,matrix[4]) + fx32_mul(z,matrix[8]));
+	vecPtr[1] = fx32_shiftdown(fx32_mul(x,matrix[1]) + fx32_mul(y,matrix[5]) + fx32_mul(z,matrix[9]));
+	vecPtr[2] = fx32_shiftdown(fx32_mul(x,matrix[2]) + fx32_mul(y,matrix[6]) + fx32_mul(z,matrix[10]));
+}
 
 //-------------------------
 //switched SSE functions: implementations for no SSE
@@ -116,6 +139,12 @@ void MatrixScale (float *matrix, const float *ptr)
 #endif //switched c/asm functions
 //-----------------------------------------
 
+void MatrixInit  (s32 *matrix)
+{
+	memset (matrix, 0, sizeof(float)*16);
+	matrix[0] = matrix[5] = matrix[10] = matrix[15] = 1<<12;
+}
+
 void MatrixInit  (float *matrix)
 {
 	memset (matrix, 0, sizeof(float)*16);
@@ -135,6 +164,13 @@ void MatrixTranspose(float *matrix)
 #undef swap
 }
 
+void	MatrixIdentity			(s32 *matrix)
+{
+	matrix[1] = matrix[2] = matrix[3] = matrix[4] = 0;
+	matrix[6] = matrix[7] = matrix[8] = matrix[9] = 0;
+	matrix[11] = matrix[12] = matrix[13] = matrix[14] = 0;
+	matrix[0] = matrix[5] = matrix[10] = matrix[15] = 1<<12;
+}
 void MatrixIdentity	(float *matrix)
 {
 	matrix[1] = matrix[2] = matrix[3] = matrix[4] = 0.0f;
@@ -151,6 +187,16 @@ float MatrixGetMultipliedIndex (int index, float *matrix, float *rightMatrix)
 			(matrix[iMod+8]*rightMatrix[iDiv+2])+(matrix[iMod+12]*rightMatrix[iDiv+3]);
 }
 
+s32 MatrixGetMultipliedIndex (int index, s32 *matrix, s32 *rightMatrix)
+{
+	int iMod = index%4, iDiv = (index>>2)<<2;
+
+	s64 temp = ((s64)matrix[iMod  ]*rightMatrix[iDiv  ])+((s64)matrix[iMod+ 4]*rightMatrix[iDiv+1])+
+			((s64)matrix[iMod+8]*rightMatrix[iDiv+2])+((s64)matrix[iMod+12]*rightMatrix[iDiv+3]);
+
+	return (s32)(temp>>12);
+}
+
 void MatrixSet (float *matrix, int x, int y, float value)	// TODO
 {
 	matrix [x+(y<<2)] = value;
@@ -177,6 +223,11 @@ void MatrixCopy (float* matrixDST, const float* matrixSRC)
 
 }
 
+void MatrixCopy (s32* matrixDST, const s32* matrixSRC)
+{
+	memcpy(matrixDST,matrixSRC,sizeof(s32)*16);
+}
+
 int MatrixCompare (const float* matrixDST, const float* matrixSRC)
 {
 	return memcmp((void*)matrixDST, matrixSRC, sizeof(float)*16);
@@ -200,7 +251,7 @@ void MatrixStackSetMaxSize (MatrixStack *stack, int size)
 	if (stack->matrix != NULL) {
 		free (stack->matrix);
 	}
-	stack->matrix = (float*) malloc (stack->size*16*sizeof(float));
+	stack->matrix = new s32[stack->size*16*sizeof(s32)];
 
 	for (i = 0; i < stack->size; i++)
 	{
@@ -211,52 +262,53 @@ void MatrixStackSetMaxSize (MatrixStack *stack, int size)
 }
 
 
-MatrixStack::MatrixStack(int size)
+MatrixStack::MatrixStack(int size, int type)
 {
 	MatrixStackSetMaxSize(this,size);
+	this->type = type;
 }
 
-void MatrixStackSetStackPosition (MatrixStack *stack, int pos)
+static void MatrixStackSetStackPosition (MatrixStack *stack, int pos)
 {
-	//printf("SetPosition: %d by %d",stack->position,pos);
 	stack->position += pos;
 
-	//this wraparound behavior fixed sims apartment pets which was constantly going up to 32
-	s32 newpos = stack->position;
-	stack->position &= (stack->size);
-
-	if(newpos != stack->position)
+	if((stack->position < 0) || (stack->position > stack->size))
 		MMU_new.gxstat.se = 1;
-
-	//printf(" to %d (size %d)\n",stack->position,stack->size);
+	stack->position &= stack->size;
 }
 
-void MatrixStackPushMatrix (MatrixStack *stack, const float *ptr)
+void MatrixStackPushMatrix (MatrixStack *stack, const s32 *ptr)
 {
-	MatrixCopy (&stack->matrix[stack->position*16], ptr);
-
+	//printf("Push %i pos %i\n", stack->type, stack->position);
+	if ((stack->type == 0) || (stack->type == 3))
+		MatrixCopy (&stack->matrix[0], ptr);
+	else
+		MatrixCopy (&stack->matrix[stack->position*16], ptr);
 	MatrixStackSetStackPosition (stack, 1);
 }
 
-float * MatrixStackPopMatrix (MatrixStack *stack, int size)
+void MatrixStackPopMatrix (s32 *mtxCurr, MatrixStack *stack, int size)
 {
+	//printf("Pop %i pos %i (change %d)\n", stack->type, stack->position, -size);
 	MatrixStackSetStackPosition(stack, -size);
-
-	return &stack->matrix[stack->position*16];
+	if ((stack->type == 0) || (stack->type == 3))
+		MatrixCopy (mtxCurr, &stack->matrix[0]);
+	else
+		MatrixCopy (mtxCurr, &stack->matrix[stack->position*16]);
 }
 
-float * MatrixStackGetPos (MatrixStack *stack, int pos)
+s32 * MatrixStackGetPos (MatrixStack *stack, int pos)
 {
 	assert(pos<31);
 	return &stack->matrix[pos*16];
 }
 
-float * MatrixStackGet (MatrixStack *stack)
+s32 * MatrixStackGet (MatrixStack *stack)
 {
 	return &stack->matrix[stack->position*16];
 }
 
-void MatrixStackLoadMatrix (MatrixStack *stack, int pos, const float *ptr)
+void MatrixStackLoadMatrix (MatrixStack *stack, int pos, const s32 *ptr)
 {
 	assert(pos<31);
 	MatrixCopy (&stack->matrix[pos*16], ptr);
@@ -354,3 +406,55 @@ void Vector4Copy(float *dst, const float *src)
 }
 
 
+void MatrixMultiply (s32 *matrix, const s32 *rightMatrix)
+{
+	s32 tmpMatrix[16];
+
+	tmpMatrix[0]  = fx32_shiftdown(fx32_mul(matrix[0],rightMatrix[0])+fx32_mul(matrix[4],rightMatrix[1])+fx32_mul(matrix[8],rightMatrix[2])+fx32_mul(matrix[12],rightMatrix[3]));
+	tmpMatrix[1]  = fx32_shiftdown(fx32_mul(matrix[1],rightMatrix[0])+fx32_mul(matrix[5],rightMatrix[1])+fx32_mul(matrix[9],rightMatrix[2])+fx32_mul(matrix[13],rightMatrix[3]));
+	tmpMatrix[2]  = fx32_shiftdown(fx32_mul(matrix[2],rightMatrix[0])+fx32_mul(matrix[6],rightMatrix[1])+fx32_mul(matrix[10],rightMatrix[2])+fx32_mul(matrix[14],rightMatrix[3]));
+	tmpMatrix[3]  = fx32_shiftdown(fx32_mul(matrix[3],rightMatrix[0])+fx32_mul(matrix[7],rightMatrix[1])+fx32_mul(matrix[11],rightMatrix[2])+fx32_mul(matrix[15],rightMatrix[3]));
+
+	tmpMatrix[4]  = fx32_shiftdown(fx32_mul(matrix[0],rightMatrix[4])+fx32_mul(matrix[4],rightMatrix[5])+fx32_mul(matrix[8],rightMatrix[6])+fx32_mul(matrix[12],rightMatrix[7]));
+	tmpMatrix[5]  = fx32_shiftdown(fx32_mul(matrix[1],rightMatrix[4])+fx32_mul(matrix[5],rightMatrix[5])+fx32_mul(matrix[9],rightMatrix[6])+fx32_mul(matrix[13],rightMatrix[7]));
+	tmpMatrix[6]  = fx32_shiftdown(fx32_mul(matrix[2],rightMatrix[4])+fx32_mul(matrix[6],rightMatrix[5])+fx32_mul(matrix[10],rightMatrix[6])+fx32_mul(matrix[14],rightMatrix[7]));
+	tmpMatrix[7]  = fx32_shiftdown(fx32_mul(matrix[3],rightMatrix[4])+fx32_mul(matrix[7],rightMatrix[5])+fx32_mul(matrix[11],rightMatrix[6])+fx32_mul(matrix[15],rightMatrix[7]));
+
+	tmpMatrix[8]  = fx32_shiftdown(fx32_mul(matrix[0],rightMatrix[8])+fx32_mul(matrix[4],rightMatrix[9])+fx32_mul(matrix[8],rightMatrix[10])+fx32_mul(matrix[12],rightMatrix[11]));
+	tmpMatrix[9]  = fx32_shiftdown(fx32_mul(matrix[1],rightMatrix[8])+fx32_mul(matrix[5],rightMatrix[9])+fx32_mul(matrix[9],rightMatrix[10])+fx32_mul(matrix[13],rightMatrix[11]));
+	tmpMatrix[10] = fx32_shiftdown(fx32_mul(matrix[2],rightMatrix[8])+fx32_mul(matrix[6],rightMatrix[9])+fx32_mul(matrix[10],rightMatrix[10])+fx32_mul(matrix[14],rightMatrix[11]));
+	tmpMatrix[11] = fx32_shiftdown(fx32_mul(matrix[3],rightMatrix[8])+fx32_mul(matrix[7],rightMatrix[9])+fx32_mul(matrix[11],rightMatrix[10])+fx32_mul(matrix[15],rightMatrix[11]));
+
+	tmpMatrix[12] = fx32_shiftdown(fx32_mul(matrix[0],rightMatrix[12])+fx32_mul(matrix[4],rightMatrix[13])+fx32_mul(matrix[8],rightMatrix[14])+fx32_mul(matrix[12],rightMatrix[15]));
+	tmpMatrix[13] = fx32_shiftdown(fx32_mul(matrix[1],rightMatrix[12])+fx32_mul(matrix[5],rightMatrix[13])+fx32_mul(matrix[9],rightMatrix[14])+fx32_mul(matrix[13],rightMatrix[15]));
+	tmpMatrix[14] = fx32_shiftdown(fx32_mul(matrix[2],rightMatrix[12])+fx32_mul(matrix[6],rightMatrix[13])+fx32_mul(matrix[10],rightMatrix[14])+fx32_mul(matrix[14],rightMatrix[15]));
+	tmpMatrix[15] = fx32_shiftdown(fx32_mul(matrix[3],rightMatrix[12])+fx32_mul(matrix[7],rightMatrix[13])+fx32_mul(matrix[11],rightMatrix[14])+fx32_mul(matrix[15],rightMatrix[15]));
+
+	memcpy(matrix,tmpMatrix,sizeof(s32)*16);
+}
+
+void MatrixScale(s32 *matrix, const s32 *ptr)
+{
+	//zero 21-sep-2010 - verified unrolling seems faster on my cpu
+	MACRODO_N(12,
+		matrix[X] = fx32_shiftdown(fx32_mul(matrix[X],ptr[X>>2]))
+		);
+}
+
+void MatrixTranslate(s32 *matrix, const s32 *ptr)
+{
+	MACRODO_N(4,
+	{
+		s64 temp = fx32_shiftup(matrix[X+12]);
+		temp += fx32_mul(matrix[X+0],ptr[0]);
+		temp += fx32_mul(matrix[X+4],ptr[1]);
+		temp += fx32_mul(matrix[X+8],ptr[2]);
+		matrix[X+12] = fx32_shiftdown(temp);
+	});
+}
+
+void MatrixMultVec4x4_M2(const s32 *matrix, s32 *vecPtr)
+{
+	MatrixMultVec4x4(matrix+16,vecPtr);
+	MatrixMultVec4x4(matrix,vecPtr);
+}
