@@ -1,21 +1,18 @@
 /*  Copyright (C) 2006-2007 shash
-	Copyright (C) 2009 DeSmuME team
+	Copyright (C) 2009-2010 DeSmuME team
 
-    This file is part of DeSmuME
+	This file is free software: you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation, either version 2 of the License, or
+	(at your option) any later version.
 
-    DeSmuME is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+	This file is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
 
-    DeSmuME is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with DeSmuME; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+	You should have received a copy of the GNU General Public License
+	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #ifndef MATRIX_H
@@ -38,32 +35,36 @@
 struct MatrixStack
 {
 	MatrixStack(int size, int type);
-	float	*matrix;
+	s32		*matrix;
 	s32		position;
 	s32		size;
 	u8		type;
 };
 
 void	MatrixInit				(float *matrix);
+void	MatrixInit				(s32 *matrix);
 
 //In order to conditionally use these asm optimized functions in visual studio
 //without having to make new build types to exclude the assembly files.
 //a bit sloppy, but there aint much to it
 
 float	MatrixGetMultipliedIndex	(int index, float *matrix, float *rightMatrix);
+s32	MatrixGetMultipliedIndex	(int index, s32 *matrix, s32 *rightMatrix);
 void	MatrixSet				(float *matrix, int x, int y, float value);
 void	MatrixCopy				(float * matrixDST, const float * matrixSRC);
+void	MatrixCopy				(s32 * matrixDST, const s32 * matrixSRC);
 int		MatrixCompare				(const float * matrixDST, const float * matrixSRC);
 void	MatrixIdentity			(float *matrix);
+void	MatrixIdentity			(s32 *matrix);
 
 void	MatrixTranspose				(float *matrix);
 void	MatrixStackInit				(MatrixStack *stack);
 void	MatrixStackSetMaxSize		(MatrixStack *stack, int size);
-void	MatrixStackPushMatrix		(MatrixStack *stack, const float *ptr);
-void	MatrixStackPopMatrix		(float *mtxCurr, MatrixStack *stack, int size);
-float*	MatrixStackGetPos			(MatrixStack *stack, int pos);
-float*	MatrixStackGet				(MatrixStack *stack);
-void	MatrixStackLoadMatrix		(MatrixStack *stack, int pos, const float *ptr);
+void	MatrixStackPushMatrix		(MatrixStack *stack, const s32 *ptr);
+void	MatrixStackPopMatrix		(s32 *mtxCurr, MatrixStack *stack, int size);
+s32*	MatrixStackGetPos			(MatrixStack *stack, int pos);
+s32*	MatrixStackGet				(MatrixStack *stack);
+void	MatrixStackLoadMatrix		(MatrixStack *stack, int pos, const s32 *ptr);
 
 void Vector2Copy(float *dst, const float *src);
 void Vector2Add(float *dst, const float *src);
@@ -139,11 +140,7 @@ FORCEINLINE void memset_u16_le(void* dst, u16 val)
 #else //no sse2
 
 template<int NUM>
-#ifdef _XBOX
-FORCEINLINE void memset_u16_le(void* dst, u16 val)
-#else
 static FORCEINLINE void memset_u16_le(void* dst, u16 val)
-#endif
 {
 	for(int i=0;i<NUM;i++)
 		T1WriteWord((u8*)dst,i<<1,val);
@@ -153,6 +150,7 @@ static FORCEINLINE void memset_u16_le(void* dst, u16 val)
 
 // NOSSE version always used in gfx3d.cpp
 void _NOSSE_MatrixMultVec4x4 (const float *matrix, float *vecPtr);
+void MatrixMultVec3x3_fixed(const s32 *matrix, s32 *vecPtr);
 
 //---------------------------
 //switched SSE functions
@@ -219,7 +217,6 @@ FORCEINLINE void MatrixMultVec4x4_M2(const float *matrix, float *vecPtr)
 	MatrixMultVec4x4(matrix,vecPtr);
 }
 
-
 FORCEINLINE void MatrixMultVec3x3(const float * matrix, float * vecPtr)
 {
 	const __m128 vec = _mm_load_ps(vecPtr);
@@ -319,16 +316,19 @@ FORCEINLINE void vector_fix2float(float* matrix, const float divisor)
 }
 
 template<int NUM>
-#ifdef _XBOX
-FORCEINLINE void memset_u8(void* dst, u8 val)
-#else
 static FORCEINLINE void memset_u8(void* dst, u8 val)
-#endif
 {
 	memset(dst,val,NUM);
 }
 
 #endif //switched SSE functions
 
+void MatrixMultVec4x4 (const s32 *matrix, s32 *vecPtr);
 
+void MatrixMultVec4x4_M2(const s32 *matrix, s32 *vecPtr);
+
+void MatrixMultiply(s32* matrix, const s32* rightMatrix);
+void MatrixScale(s32 *matrix, const s32 *ptr);
+void MatrixTranslate(s32 *matrix, const s32 *ptr);
 #endif
+
