@@ -1,6 +1,6 @@
 /*  Copyright (C) 2006 thoduv
     Copyright (C) 2006-2007 Theo Berkau
-	Copyright (C) 2008-2009 DeSmuME team
+	Copyright (C) 2008-2010 DeSmuME team
 
     This file is part of DeSmuME
 
@@ -35,21 +35,22 @@
 #include <xtl.h>	// it`s really need?
 #endif
 
-#define FW_CMD_READ             0x3
-#define FW_CMD_WRITEDISABLE     0x4
-#define FW_CMD_READSTATUS       0x5
-#define FW_CMD_WRITEENABLE      0x6
-#define FW_CMD_PAGEWRITE        0xA
+#define FW_CMD_READ             0x03
+#define FW_CMD_WRITEDISABLE     0x04
+#define FW_CMD_READSTATUS       0x05
+#define FW_CMD_WRITEENABLE      0x06
+#define FW_CMD_PAGEWRITE        0x0A
+#define FW_CMD_READ_ID			0x9F
 
 #define BM_CMD_AUTODETECT       0xFF
-#define BM_CMD_WRITESTATUS      0x1
-#define BM_CMD_WRITELOW         0x2
-#define BM_CMD_READLOW          0x3
-#define BM_CMD_WRITEDISABLE     0x4
-#define BM_CMD_READSTATUS       0x5
-#define BM_CMD_WRITEENABLE      0x6
-#define BM_CMD_WRITEHIGH        0xA
-#define BM_CMD_READHIGH         0xB
+#define BM_CMD_WRITESTATUS      0x01
+#define BM_CMD_WRITELOW         0x02
+#define BM_CMD_READLOW          0x03
+#define BM_CMD_WRITEDISABLE     0x04
+#define BM_CMD_READSTATUS       0x05
+#define BM_CMD_WRITEENABLE      0x06
+#define BM_CMD_WRITEHIGH        0x0A
+#define BM_CMD_READHIGH         0x0B
 
 /* FLASH*/
 #define COMM_PAGE_WRITE		0x0A
@@ -119,6 +120,12 @@ const char *save_names[] = {
 		"FLASH 512Mbit"
 };
 
+//forces the currently selected backup type to be current
+//(can possibly be used to repair poorly chosen save types discovered late in gameplay i.e. pokemon gamers)
+void backup_forceManualBackupType()
+{
+	MMU_new.backupDevice.forceManualBackupType();
+}
 
 void backup_setManualBackupType(int type)
 {
@@ -184,6 +191,27 @@ void fw_reset_com(memory_chip_t *mc)
 			fwrite(mc->data, mc->size, 1, mc->fp);
 		}
 
+		if (mc->isFirmware&&CommonSettings.UseExtFirmware)
+		{
+			// copy User Settings 1 to User Settings 0 area
+			memcpy(&mc->data[0x3FE00], &mc->data[0x3FF00], 0x100);
+			
+			printf("Firmware: save config");
+			FILE *fp = fopen(mc->userfile, "wb");
+			if (fp)
+			{
+				if (fwrite(&mc->data[0x3FF00], 1, 0x100, fp) == 0x100)		// User Settings
+					if (fwrite(&mc->data[0x0002A], 1, 0x1D6, fp) == 0x1D6)  // WiFi Settings
+						if (fwrite(&mc->data[0x3FA00], 1, 0x300, fp) == 0x300)  // WiFi AP Settings
+							printf(" - done\n");
+						else
+							printf(" - failed\n");
+				fclose(fp);
+			}
+			else
+				printf(" - failed\n");
+		}
+
 		mc->write_enable = FALSE;
 	}
 
@@ -222,31 +250,57 @@ u8 fw_transfer(memory_chip_t *mc, u8 data)
 			
 		}
 	}
+	else if(mc->com == FW_CMD_READ_ID)
+	{
+		switch(mc->addr)
+		{
+		//here is an ID string measured from an old ds fat: 62 16 00 (0x62=sanyo)
+		//but we chose to use an ST from martin's ds fat string so programs might have a clue as to the firmware size:
+		//20 40 12
+		case 0: 
+			data = 0x20;
+			mc->addr=1; 
+			break;
+		case 1: 
+			data = 0x40; //according to gbatek this is the device ID for the flash on someone's ds fat
+			mc->addr=2; 
+			break;
+		case 2: 
+			data = 0x12;
+			mc->addr = 0; 
+			break;
+		}
+	}
 	else if(mc->com == FW_CMD_READSTATUS)
 	{
 		return (mc->write_enable ? 0x02 : 0x00);
 	}
-	else	/* finally, check if it's a new command */
+	else	//finally, check if it's a new command
 	{
 		switch(data)
 		{
-			case 0: break;	/* nothing */
+			case 0: break;	//nothing
+
+			case FW_CMD_READ_ID:
+				mc->addr = 0;
+				mc->com = FW_CMD_READ_ID;
+				break;
 			
-			case FW_CMD_READ:    /* read command */
+			case FW_CMD_READ:    //read command
 				mc->addr = 0;
 				mc->addr_shift = 3;
 				mc->com = FW_CMD_READ;
 				break;
 				
-			case FW_CMD_WRITEENABLE:     /* enable writing */
+			case FW_CMD_WRITEENABLE:     //enable writing
 				if(mc->writeable_buffer) { mc->write_enable = TRUE; }
 				break;
 				
-			case FW_CMD_WRITEDISABLE:    /* disable writing */
+			case FW_CMD_WRITEDISABLE:    //disable writing
 				mc->write_enable = FALSE;
 				break;
 				
-			case FW_CMD_PAGEWRITE:       /* write command */
+			case FW_CMD_PAGEWRITE:       //write command
 				if(mc->write_enable)
 				{
 					mc->addr = 0;
@@ -256,7 +310,7 @@ u8 fw_transfer(memory_chip_t *mc, u8 data)
 				else { data = 0; }
 				break;
 			
-			case FW_CMD_READSTATUS:  /* status register command */
+			case FW_CMD_READSTATUS:  //status register command
 				mc->com = FW_CMD_READSTATUS;
 				break;
 				
@@ -271,7 +325,8 @@ u8 fw_transfer(memory_chip_t *mc, u8 data)
 
 bool BackupDevice::save_state(EMUFILE* os)
 {
-	u32 version = 1;
+	u32 version = 2;
+	//v0
 	write32le(version,os);
 	write32le(write_enable,os);
 	write32le(com,os);
@@ -280,7 +335,11 @@ bool BackupDevice::save_state(EMUFILE* os)
 	write32le((u32)state,os);
 	writebuffer(data,os);
 	writebuffer(data_autodetect,os);
+	//v1
 	write32le(addr,os);
+	//v2
+	write8le(motionInitState,os);
+	write8le(motionFlag,os);
 	return true;
 }
 
@@ -288,7 +347,8 @@ bool BackupDevice::load_state(EMUFILE* is)
 {
 	u32 version;
 	if(read32le(&version,is)!=1) return false;
-	if(version==0 || version==1) {
+	if(version>=0)
+	{
 		readbool(&write_enable,is);
 		read32le(&com,is);
 		read32le(&addr_size,is);
@@ -298,9 +358,15 @@ bool BackupDevice::load_state(EMUFILE* is)
 		state = (STATE)temp;
 		readbuffer(data,is);
 		readbuffer(data_autodetect,is);
-		if(version==1)
-			read32le(&addr,is);
 	}
+	if(version>=1)
+		read32le(&addr,is);
+	if(version>=2)
+	{
+		read8le(&motionInitState,is);
+		read8le(&motionFlag,is);
+	}
+
 	return true;
 }
 
@@ -327,17 +393,23 @@ void BackupDevice::movie_mode()
 	reset();
 }
 
-void BackupDevice::reset()
+void BackupDevice::reset_hardware()
 {
+	write_enable = FALSE;
 	com = 0;
 	addr = addr_counter = 0;
+	motionInitState = MOTION_INIT_STATE_IDLE;
+	motionFlag = MOTION_FLAG_NONE;
+	state = DETECTING;
 	flushPending = false;
 	lazyFlushPending = false;
-	data.resize(0);
-	write_enable = FALSE;
-	data_autodetect.resize(0);
+}
 
-	state = DETECTING;
+void BackupDevice::reset()
+{
+	reset_hardware();
+	data.resize(0);
+	data_autodetect.resize(0);
 	addr_size = 0;
 	loadfile();
 
@@ -361,6 +433,7 @@ void BackupDevice::close_rom()
 
 void BackupDevice::reset_command()
 {
+	//printf("MC RESET\n");
 	//for a performance hack, save files are only flushed after each reset command
 	//(hopefully, after each page)
 	if(flushPending)
@@ -424,6 +497,23 @@ void BackupDevice::reset_command()
 }
 u8 BackupDevice::data_command(u8 val, int cpu)
 {
+	//printf("MC CMD: %02X\n",val);
+
+	//motion: some guessing here... hope it doesn't break anything
+	if(com == BM_CMD_READLOW && motionInitState == MOTION_INIT_STATE_RECEIVED_4_B && val == 0)
+	{
+		motionInitState = MOTION_INIT_STATE_IDLE;
+		motionFlag |= MOTION_FLAG_ENABLED;
+		//return 0x04; //return 0x04 to enable motion!!!!!
+		return 0; //but we return 0 to disable it! since we don't emulate it anyway
+	}
+
+	//if the game firmly believes we have motion support, then ignore all motion commands, since theyre not emulated.
+	if(motionFlag & MOTION_FLAG_SENSORMODE)
+	{
+		return 0;
+	}
+
 	if(com == BM_CMD_READLOW || com == BM_CMD_WRITELOW)
 	{
 		//handle data or address
@@ -458,6 +548,7 @@ u8 BackupDevice::data_command(u8 val, int cpu)
 				ensure(addr+1);
 				if(com == BM_CMD_READLOW)
 				{
+					//printf("READ ADR: %08X\n",addr);
 					val = data[addr];
 					//flushPending = true; //is this a good idea? it may slow stuff down, but it is helpful for debugging
 					lazyFlushPending = true; //lets do this instead
@@ -465,9 +556,13 @@ u8 BackupDevice::data_command(u8 val, int cpu)
 				}
 				else 
 				{
-					data[addr] = val;
-					flushPending = true;
-					//printf("writ: %08X\n",addr);
+					if(write_enable)
+					{
+						//printf("WRITE ADR: %08X\n",addr);
+						data[addr] = val;
+						flushPending = true;
+						//printf("writ: %08X\n",addr);
+					}
 				}
 				addr++;
 
@@ -477,8 +572,8 @@ u8 BackupDevice::data_command(u8 val, int cpu)
 	else if(com == BM_CMD_READSTATUS)
 	{
 		//handle request to read status
-		//LOG("Backup Memory Read Status: %02X\n", mc->write_enable << 1);
-		return (write_enable << 1);
+		LOG("Backup Memory Read Status: %02X\n", write_enable << 1);
+		return (write_enable << 1) | (3<<2);
 	}
 	else
 	{
@@ -487,16 +582,51 @@ u8 BackupDevice::data_command(u8 val, int cpu)
 		{
 			case 0: break; //??
 
+			case 0xFE:
+				if(motionInitState == MOTION_INIT_STATE_IDLE) { motionInitState = MOTION_INIT_STATE_FE; return 0; }
+				break;
+			case 0xFD:
+				if(motionInitState == MOTION_INIT_STATE_FE) { motionInitState = MOTION_INIT_STATE_FD; return 0; }
+				break;
+			case 0xFB:
+				if(motionInitState == MOTION_INIT_STATE_FD) { motionInitState = MOTION_INIT_STATE_FB; return 0; }
+				break;
+			case 0xF8:
+				//enable sensor mode
+				if(motionInitState == MOTION_INIT_STATE_FD) 
+				{
+					motionInitState = MOTION_INIT_STATE_IDLE;
+					motionFlag |= MOTION_FLAG_SENSORMODE;
+					return 0;
+				}
+				break;
+			case 0xF9:
+				//disable sensor mode
+				if(motionInitState == MOTION_INIT_STATE_FD) 
+				{
+					motionInitState = MOTION_INIT_STATE_IDLE;
+					motionFlag &= ~MOTION_FLAG_SENSORMODE;
+					return 0;
+				}
+				break;
+
 			case 8:
+				printf("COMMAND%c: Unverified Backup Memory command: %02X FROM %08X\n",(cpu==ARMCPU_ARM9)?'9':'7',val, (cpu==ARMCPU_ARM9)?NDS_ARM9.instruct_adr:NDS_ARM7.instruct_adr);
 				val = 0xAA;
 				break;
-			
+
 			case BM_CMD_WRITEDISABLE:
+				switch(motionInitState)
+				{
+				case MOTION_INIT_STATE_IDLE: motionInitState = MOTION_INIT_STATE_RECEIVED_4; break;
+				case MOTION_INIT_STATE_RECEIVED_4: motionInitState = MOTION_INIT_STATE_RECEIVED_4_B; break;
+				}
 				write_enable = FALSE;
 				break;
 							
 			case BM_CMD_READSTATUS:
 				com = BM_CMD_READSTATUS;
+				val = (write_enable << 1) | (3<<2);
 				break;
 
 			case BM_CMD_WRITEENABLE:
@@ -531,7 +661,10 @@ u8 BackupDevice::data_command(u8 val, int cpu)
 			default:
 				printf("COMMAND%c: Unhandled Backup Memory command: %02X FROM %08X\n",(cpu==ARMCPU_ARM9)?'9':'7',val, (cpu==ARMCPU_ARM9)?NDS_ARM9.instruct_adr:NDS_ARM7.instruct_adr);
 				break;
-		}
+		} //switch(val)
+
+		//motion control state machine broke, return to ground
+		motionInitState = MOTION_INIT_STATE_IDLE;
 	}
 	return val;
 }
@@ -1069,3 +1202,9 @@ bool BackupDevice::load_movie(EMUFILE* is) {
 
 	return true;
 }
+
+void BackupDevice::forceManualBackupType()
+{
+	addr_size = addr_size_for_old_save_size(save_types[CommonSettings.manualBackupType][1]);
+	state = RUNNING;
+}
\ No newline at end of file
