@@ -317,6 +317,8 @@ struct PolyAttr
 		if(mode==3 && polyid !=0) return !backfacing;
 		//another reasonable possibility is that we should be forcing back faces to draw (mariokart doesnt use them)
 		//and then only using a single bit buffer (but a cursory test of this doesnt actually work)
+		//
+		//this code needs to be here for shadows in wizard of oz to work.
 
 		switch((val>>6)&3) {
 			case 0: return false;
@@ -365,6 +367,7 @@ public:
 	{
 		Sampler() {}
 
+		bool enabled;
 		int width, height;
 		int wmask, hmask;
 		int wrap;
@@ -379,6 +382,7 @@ public:
 			wmask = width-1;
 			hmask = height-1;
 			wrap = (texParam>>16)&0xF;
+			enabled = gfx3d.renderState.enableTexturing && (texFormat!=0);
 		}
 
 		FORCEINLINE void clamp(int &val, const int size, const int sizemask){
@@ -430,6 +434,9 @@ public:
 
 	FORCEINLINE FragmentColor sample(float u, float v)
 	{
+		static FragmentColor white = MakeFragmentColor(63,63,63,31);
+		if(!sampler.enabled) return white;
+
 		//finally, we can use floor here. but, it is slower than we want.
 		//the best solution is probably to wait until the pipeline is full of fixed point
 		s32 iu = s32floor(u);
@@ -477,13 +484,16 @@ public:
 			//#endif
 			break;
 		case 1: //decal
-			u = shader.invu*shader.w;
-			v = shader.invv*shader.w;
-			texColor = sample(u,v);
-			dst.r = decal_table[texColor.a][texColor.r][shader.materialColor.r];
-			dst.g = decal_table[texColor.a][texColor.g][shader.materialColor.g];
-			dst.b = decal_table[texColor.a][texColor.b][shader.materialColor.b];
-			dst.a = shader.materialColor.a;
+			if(sampler.enabled)
+			{
+				u = shader.invu*shader.w;
+				v = shader.invv*shader.w;
+				texColor = sample(u,v);
+				dst.r = decal_table[texColor.a][texColor.r][shader.materialColor.r];
+				dst.g = decal_table[texColor.a][texColor.g][shader.materialColor.g];
+				dst.b = decal_table[texColor.a][texColor.b][shader.materialColor.b];
+				dst.a = shader.materialColor.a;
+			} else dst = shader.materialColor;
 			break;
 		case 2: //toon/highlight shading
 			{
@@ -491,33 +501,24 @@ public:
 				v = shader.invv*shader.w;
 				texColor = sample(u,v);
 				FragmentColor toonColor = engine->toonTable[shader.materialColor.r>>1];
-				if(sampler.texFormat == 0)
+			
+				if(gfx3d.renderState.shading == GFX3D_State::HIGHLIGHT)
 				{
-					//if no texture is set then we dont need to modulate texture with toon 
-					//but rather just use toon directly
-					dst = toonColor;
-					dst.a = shader.materialColor.a;
+					dst.r = modulate_table[texColor.r][shader.materialColor.r];
+					dst.g = modulate_table[texColor.g][shader.materialColor.r];
+					dst.b = modulate_table[texColor.b][shader.materialColor.r];
+					dst.a = modulate_table[GFX3D_5TO6(texColor.a)][GFX3D_5TO6(shader.materialColor.a)]>>1;
+
+					dst.r = min<u8>(63, (dst.r + toonColor.r));
+					dst.g = min<u8>(63, (dst.g + toonColor.g));
+					dst.b = min<u8>(63, (dst.b + toonColor.b));
 				}
 				else
 				{
-					if(gfx3d.renderState.shading == GFX3D_State::HIGHLIGHT)
-					{
-						dst.r = modulate_table[texColor.r][shader.materialColor.r];
-						dst.g = modulate_table[texColor.g][shader.materialColor.r];
-						dst.b = modulate_table[texColor.b][shader.materialColor.r];
-						dst.a = modulate_table[GFX3D_5TO6(texColor.a)][GFX3D_5TO6(shader.materialColor.a)]>>1;
-
-						dst.r = min<u8>(63, (dst.r + toonColor.r));
-						dst.g = min<u8>(63, (dst.g + toonColor.g));
-						dst.b = min<u8>(63, (dst.b + toonColor.b));
-					}
-					else
-					{
-						dst.r = modulate_table[texColor.r][toonColor.r];
-						dst.g = modulate_table[texColor.g][toonColor.g];
-						dst.b = modulate_table[texColor.b][toonColor.b];
-						dst.a = modulate_table[GFX3D_5TO6(texColor.a)][GFX3D_5TO6(shader.materialColor.a)]>>1;
-					}
+					dst.r = modulate_table[texColor.r][toonColor.r];
+					dst.g = modulate_table[texColor.g][toonColor.g];
+					dst.b = modulate_table[texColor.b][toonColor.b];
+					dst.a = modulate_table[GFX3D_5TO6(texColor.a)][GFX3D_5TO6(shader.materialColor.a)]>>1;
 				}
 
 			}
@@ -526,20 +527,12 @@ public:
 			//is this right? only with the material color?
 			dst = shader.materialColor;
 			break;
-		case 4: //our own special mode which only uses the material color (for when texturing is disabled)
-			dst = shader.materialColor;
-			break;
-
 		}
 	}
 
 	void setupShader(u32 polyattr)
 	{
 		shader.mode = (polyattr>>4)&0x3;
-		//if there is no texture set, then set to the mode which doesnt even use a texture
-		//(no texture makes sense for toon/highlight mode)
-		if(sampler.texFormat == 0 && (shader.mode == 0 || shader.mode == 1))
-			shader.mode = 4;
 	}
 
 	FORCEINLINE void pixel(int adr,float r, float g, float b, float invu, float invv, float w, float z)
@@ -562,10 +555,23 @@ public:
 
 		if(polyAttr.decalMode)
 		{
-			if(depth != destFragment.depth)
+			if ( CommonSettings.GFX3D_Zelda_Shadow_Depth_Hack > 0)
 			{
-				goto depth_fail;
+				if(depth<destFragment.depth - CommonSettings.GFX3D_Zelda_Shadow_Depth_Hack
+					|| depth>destFragment.depth + CommonSettings.GFX3D_Zelda_Shadow_Depth_Hack) 
+				{
+					goto depth_fail;
+				}
+
 			}
+			else
+			{
+				if(depth != destFragment.depth)
+				{
+					goto depth_fail;
+				}
+			}
+
 		}
 		else
 		{
@@ -580,20 +586,12 @@ public:
 		{
 			if(polyAttr.polyid == 0)
 			{
-				//that's right! stencil buffers, despite reports to the contrary, must be more than 1 bit
-				//this is necessary to make these cases work all at once.
-				//1. sm64 (standing near signs and blocks)
-				//2. mariokart (no glitches in shadow shape in kart selector)
-				//3. mariokart (no junk beneath platform in kart selector / no shadow beneath grate floor in bowser stage)
-				//(specifically, the shadows in mario kart are complicated concave shapes)
-				destFragment.stencil++;
 				goto rejected_fragment;
 			}
 			else
 			{
-				if(destFragment.stencil)
+				if(destFragment.stencil==0)
 				{
-					destFragment.stencil--;
 					goto rejected_fragment;
 				}	
 
@@ -670,9 +668,22 @@ public:
 
 		}
 
+		//shadow cases: (need multi-bit stencil buffer to cope with all of these, especially the mariokart compelx shadows)
+		//1. sm64 (standing near signs and blocks)
+		//2. mariokart (no glitches in shadow shape in kart selector)
+		//3. mariokart (no junk beneath platform in kart selector / no shadow beneath grate floor in bowser stage)
+		//(specifically, the shadows in mario kart are complicated concave shapes)
+
+		goto done;
 		depth_fail:
+		if(shader.mode == 3 && polyAttr.polyid == 0)
+			destFragment.stencil++;
 		rejected_fragment:
+		done:
 		;
+
+		if(shader.mode == 3 && polyAttr.polyid != 0 && destFragment.stencil)
+			destFragment.stencil--;
 	}
 
 	//draws a single scanline
@@ -1154,14 +1165,18 @@ void SoftRasterizerEngine::initFramebuffer(const int width, const int height, co
 
 void SoftRasterizerEngine::updateToonTable()
 {
-	if (!gfx3d.renderState.invalidateToon) return;
 	//convert the toon colors
 	for(int i=0;i<32;i++) {
-		toonTable[i].r = (gfx3d.renderState.rgbToonTable[i] >> 2) & 0x3F;
-		toonTable[i].g = (gfx3d.renderState.rgbToonTable[i] >> 10) & 0x3F;
-		toonTable[i].b = (gfx3d.renderState.rgbToonTable[i] >> 18) & 0x3F;
+		#ifdef WORDS_BIGENDIAN
+			u32 u32temp = RGB15TO32_NOALPHA(gfx3d.renderState.u16ToonTable[i]);
+			toonTable[i].r = (u32temp >> 2) & 0x3F;
+			toonTable[i].g = (u32temp >> 10) & 0x3F;
+			toonTable[i].b = (u32temp >> 18) & 0x3F;
+		#else
+			toonTable[i].color = (RGB15TO32_NOALPHA(gfx3d.renderState.u16ToonTable[i])>>2)&0x3F3F3F3F;
+		#endif
+		//printf("%d %d %d %d\n",toonTable[i].r,toonTable[i].g,toonTable[i].b,toonTable[i].a);
 	}
-	gfx3d.renderState.invalidateToon = false;
 }
 
 void SoftRasterizerEngine::updateFogTable()
@@ -1246,7 +1261,10 @@ void SoftRasterizerEngine::framebufferProcess()
 		for(int i=0;i<8;i++)
 		{
 			u16 col = T1ReadWord(MMU.MMU_MEM[ARMCPU_ARM9][0x40], 0x330+i*2);
-			edgeMarkColors[i].color = RGB15TO5555(col,0x0F);
+			edgeMarkColors[i].color = RGB15TO5555(col,gfx3d.state.enableAntialiasing ? 0x0F : 0x1F);
+			edgeMarkColors[i].r = GFX3D_5TO6(edgeMarkColors[i].r);
+			edgeMarkColors[i].g = GFX3D_5TO6(edgeMarkColors[i].g);
+			edgeMarkColors[i].b = GFX3D_5TO6(edgeMarkColors[i].b);
 
 			// this seems to be the only thing that selectively disables edge marking
 			edgeMarkDisabled[i] = (col == 0x7FFF);
