@@ -1,22 +1,18 @@
 /*  Copyright (C) 2006 Theo Berkau
+	Copyright (C) 2006-2010 DeSmuME team
 
-    Ideas borrowed from Stephane Dallongeville's SCSP core
+	This file is free software: you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation, either version 2 of the License, or
+	(at your option) any later version.
 
-    This file is part of DeSmuME
+	This file is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
 
-    DeSmuME is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    DeSmuME is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with DeSmuME; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+	You should have received a copy of the GNU General Public License
+	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #ifndef REGISTERS_H
@@ -287,6 +283,10 @@
 #define REG_DISPA_DISPCAPCNT                    0x04000064
 #define REG_DISPA_DISPMMEMFIFO                  0x04000068
 
+#define REG_DISPA_DISP3DCNT_BIT_RDLINES_UNDERFLOW 0x1000
+#define REG_DISPA_DISP3DCNT_BIT_RAM_OVERFLOW 0x2000
+#define REG_DISPA_DISP3DCNT_BITS_ACK (REG_DISPA_DISP3DCNT_BIT_RDLINES_UNDERFLOW|REG_DISPA_DISP3DCNT_BIT_RAM_OVERFLOW)
+
 
 #define eng_3D_RDLINES_COUNT   0x04000320
 #define eng_3D_EDGE_COLOR      0x04000330
@@ -346,6 +346,65 @@
 #define eng_3D_VEC_RESULT      0x04000630
 #define eng_3D_CLIPMTX_RESULT  0x04000640
 #define eng_3D_VECMTX_RESULT   0x04000680
-  
+
+#define IPCFIFOCNT_SENDEMPTY 0x0001
+#define IPCFIFOCNT_SENDFULL 0x0002
+#define IPCFIFOCNT_SENDIRQEN 0x0004
+#define IPCFIFOCNT_SENDCLEAR 0x0008
+#define IPCFIFOCNT_RECVEMPTY 0x0100
+#define IPCFIFOCNT_RECVFULL 0x0200
+#define IPCFIFOCNT_RECVIRQEN 0x0400
+#define IPCFIFOCNT_FIFOERROR 0x4000
+#define IPCFIFOCNT_FIFOENABLE 0x8000
+#define IPCFIFOCNT_WRITEABLE (IPCFIFOCNT_SENDIRQEN | IPCFIFOCNT_RECVIRQEN | IPCFIFOCNT_FIFOENABLE)
+
+#define IRQ_BIT_LCD_VBLANK 0
+#define IRQ_BIT_LCD_HBLANK 1
+#define IRQ_BIT_LCD_VMATCH 2
+#define IRQ_BIT_TIMER_0 3
+#define IRQ_BIT_TIMER_1 4
+#define IRQ_BIT_TIMER_2 5
+#define IRQ_BIT_TIMER_3 6
+#define IRQ_BIT_ARM7_SIO 7
+#define IRQ_BIT_DMA_0 8
+#define IRQ_BIT_DMA_2 9
+#define IRQ_BIT_DMA_3 10
+#define IRQ_BIT_DMA_4 11
+#define IRQ_BIT_KEYPAD 12
+#define IRQ_BIT_GAMEPAK 13
+#define IRQ_BIT_IPCSYNC 16
+#define IRQ_BIT_IPCFIFO_SENDEMPTY 17
+#define IRQ_BIT_IPCFIFO_RECVNONEMPTY 18
+#define IRQ_BIT_GC_TRANSFER_COMPLETE 19
+#define IRQ_BIT_GC_IREQ_MC 20
+#define IRQ_BIT_ARM9_GXFIFO 21
+#define IRQ_BIT_ARM7_FOLD 22
+#define IRQ_BIT_ARM7_SPI 23
+#define IRQ_BIT_ARM7_WIFI 24
+
+#define IRQ_MASK_LCD_VBLANK (1<<0)
+#define IRQ_MASK_LCD_HBLANK (1<<1)
+#define IRQ_MASK_LCD_VMATCH (1<<2)
+#define IRQ_MASK_TIMER_0 (1<<3)
+#define IRQ_MASK_TIMER_1 (1<<4)
+#define IRQ_MASK_TIMER_2 (1<<5)
+#define IRQ_MASK_TIMER_3 (1<<6)
+#define IRQ_MASK_ARM7_SIO (1<<7)
+#define IRQ_MASK_DMA_0 (1<<8)
+#define IRQ_MASK_DMA_2 (1<<9)
+#define IRQ_MASK_DMA_3 (1<<10)
+#define IRQ_MASK_DMA_4 (1<<11)
+#define IRQ_MASK_KEYPAD (1<<12)
+#define IRQ_MASK_GAMEPAK (1<<13)
+#define IRQ_MASK_IPCSYNC (1<<16)
+#define IRQ_MASK_IPCFIFO_SENDEMPTY (1<<17)
+#define IRQ_MASK_IPCFIFO_RECVNONEMPTY (1<<18)
+#define IRQ_MASK_GC_TRANSFER_COMPLETE (1<<19)
+#define IRQ_MASK_GC_IREQ_MC (1<<20)
+#define IRQ_MASK_ARM9_GXFIFO (1<<21)
+#define IRQ_MASK_ARM7_FOLD (1<<22)
+#define IRQ_MASK_ARM7_SPI (1<<23)
+#define IRQ_MASK_ARM7_WIFI (1<<24)
+
 
 #endif
