@@ -3,21 +3,18 @@
     Copyright (C) 2007 Pascal Giard
 	Copyright (C) 2008-2010 DeSmuME team
 
-    This file is part of DeSmuME
+	This file is free software: you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation, either version 2 of the License, or
+	(at your option) any later version.
 
-    DeSmuME is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+	This file is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
 
-    DeSmuME is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with DeSmuME; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+	You should have received a copy of the GNU General Public License
+	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #ifdef HAVE_LIBZ
@@ -96,8 +93,8 @@ SFORMAT SF_ARM7[]={
 	{ "7int", 4, 1, &NDS_ARM7.intVector },
 	{ "7LDT", 1, 1, &NDS_ARM7.LDTBit },
 	{ "7Wai", 4, 1, &NDS_ARM7.waitIRQ },
-	//{ "7wIR", 4, 1, &NDS_ARM7.wIRQ, },
-	{ "7wir", 4, 1, &NDS_ARM7.wirq, },
+	{ "7hef", 4, 1, &NDS_ARM7.halt_IE_and_IF },
+	{ "7iws", 1, 1, &NDS_ARM7.intrWaitARM_state },
 	{ 0 }
 };
 
@@ -133,8 +130,8 @@ SFORMAT SF_ARM9[]={
 	{ "9int", 4, 1, &NDS_ARM9.intVector},
 	{ "9LDT", 1, 1, &NDS_ARM9.LDTBit},
 	{ "9Wai", 4, 1, &NDS_ARM9.waitIRQ},
-	//{ "9wIR", 4, 1, &NDS_ARM9.wIRQ},
-	{ "9wir", 4, 1, &NDS_ARM9.wirq},
+	{ "9hef", 4, 1, &NDS_ARM9.halt_IE_and_IF },
+	{ "9iws", 1, 1, &NDS_ARM7.intrWaitARM_state },
 	{ 0 }
 };
 
@@ -172,6 +169,7 @@ SFORMAT SF_NDS[]={
 	{ "_ENH", 4, 1, &nds.ensataHandshake},
 	{ "_ENI", 4, 1, &nds.ensataIpcSyncCounter},
 	{ "_SLP", 4, 1, &nds.sleeping},
+	{ "_FBS", 4, 1, &nds.freezeBus},
 	{ "_CEJ", 4, 1, &nds.cardEjected},
 	{ "_P00", 1, 1, &nds.power1.lcd},
 	{ "_P01", 1, 1, &nds.power1.gpuMain},
@@ -190,7 +188,6 @@ SFORMAT SF_MMU[]={
 	{ "M7RG", 1, sizeof(MMU.ARM7_REG), MMU.ARM7_REG},
 	{ "M7WI", 1, sizeof(MMU.ARM7_WIRAM), MMU.ARM7_WIRAM},
 	{ "MSWI", 1, sizeof(MMU.SWIRAM), MMU.SWIRAM},
-	{ "MCRA", 1, sizeof(MMU.CART_RAM), MMU.CART_RAM},
 	{ "M9RW", 1, 1,       &MMU.ARM9_RW_MODE},
 	{ "MDTC", 4, 1,       &MMU.DTCMRegion},
 	{ "MITC", 4, 1,       &MMU.ITCMRegion},
@@ -201,7 +198,7 @@ SFORMAT SF_MMU[]={
 	{ "MTRL", 2, 8,       MMU.timerReload},
 	{ "MIME", 4, 2,       MMU.reg_IME},
 	{ "MIE_", 4, 2,       MMU.reg_IE},
-	{ "MIF_", 4, 2,       MMU.reg_IF},
+	{ "MIF_", 4, 2,       MMU.reg_IF_bits},
 
 	{ "MGXC", 8, 1,       &MMU.gfx3dCycles},
 	
@@ -252,6 +249,8 @@ SFORMAT SF_MMU[]={
 	{ "PMCN", 1, 1,			&MMU.powerMan_CntReg},
 	{ "PMCW", 4, 1,			&MMU.powerMan_CntRegWritten},
 	{ "PMCR", 1, 5,			&MMU.powerMan_Reg},
+
+	{ "MR3D", 4, 1,		&MMU.reg_DISP3DCNT_bits},
 	
 	{ 0 }
 };
@@ -264,7 +263,7 @@ SFORMAT SF_MOVIE[]={
 
 static void mmu_savestate(EMUFILE* os)
 {
-	u32 version = 4;
+	u32 version = 5;
 	write32le(version,os);
 	
 	//version 2:
@@ -448,6 +447,16 @@ static bool mmu_loadstate(EMUFILE* is, int size)
 	ok &= MMU_new.sqrt.loadstate(is,version);
 	ok &= MMU_new.div.loadstate(is,version);
 
+	//to prevent old savestates from confusing IF bits, mask out ones which had been stored but should have been generated
+	MMU.reg_IF_bits[0] &= ~0x00260000;
+	MMU.reg_IF_bits[1] &= ~0x00060000;
+
+	MMU_new.gxstat.fifo_low = gxFIFO.size <= 127;
+	MMU_new.gxstat.fifo_empty = gxFIFO.size == 0;
+
+	if(version < 5)
+		MMU.reg_DISP3DCNT_bits = T1ReadWord(MMU.ARM9_REG,0x60);
+
 	return ok;
 }
 
@@ -497,10 +506,11 @@ static void cp15_saveone(armcp15_t *cp15, EMUFILE* os)
 static void cp15_savestate(EMUFILE* os)
 {
 	//version
-	write32le(0,os);
+	write32le(1,os);
 
 	cp15_saveone((armcp15_t *)NDS_ARM9.coproc[15],os);
-	cp15_saveone((armcp15_t *)NDS_ARM7.coproc[15],os);
+	//ARM7 not have coprocessor
+	//cp15_saveone((armcp15_t *)NDS_ARM7.coproc[15],os);
 }
 
 static bool cp15_loadone(armcp15_t *cp15, EMUFILE* is)
@@ -553,10 +563,19 @@ static bool cp15_loadstate(EMUFILE* is, int size)
 	//read version
 	u32 version;
 	if(read32le(&version,is) != 1) return false;
-	if(version != 0) return false;
+	if(version > 1) return false;
 
 	if(!cp15_loadone((armcp15_t *)NDS_ARM9.coproc[15],is)) return false;
-	if(!cp15_loadone((armcp15_t *)NDS_ARM7.coproc[15],is)) return false;
+	
+	if(version == 0)
+	{
+		//ARM7 not have coprocessor
+		u8 *tmp_buf = new u8 [sizeof(armcp15_t)];
+		if (!tmp_buf) return false;
+		if(!cp15_loadone((armcp15_t *)tmp_buf,is)) return false;
+		delete [] tmp_buf;
+		tmp_buf = NULL;
+	}
 
 	return true;
 }
@@ -662,64 +681,6 @@ void loadstate_slot(int num)
    }
 }
 
-u8 sram_read (u32 address) {
-	address = address - SRAM_ADDRESS;
-
-	if ( address > SRAM_SIZE )
-		return 0;
-
-	return MMU.CART_RAM[address];
-
-}
-
-void sram_write (u32 address, u8 value) {
-
-	address = address - SRAM_ADDRESS;
-
-	if ( address < SRAM_SIZE )
-		MMU.CART_RAM[address] = value;
-
-}
-
-int sram_load (const char *file_name) {
-
-	FILE *file;
-	size_t elems_read;
-
-	file = fopen ( file_name, "rb" );
-	if( file == NULL )
-		return 0;
-
-	elems_read = fread ( MMU.CART_RAM, SRAM_SIZE, 1, file );
-
-	fclose ( file );
-
-	osd->setLineColor(255, 255, 255);
-	osd->addLine("Loaded SRAM");
-
-	return 1;
-
-}
-
-int sram_save (const char *file_name) {
-
-	FILE *file;
-	size_t elems_written;
-
-	file = fopen ( file_name, "wb" );
-	if( file == NULL )
-		return 0;
-
-	elems_written = fwrite ( MMU.CART_RAM, SRAM_SIZE, 1, file );
-
-	fclose ( file );
-
-	osd->setLineColor(255, 255, 255);
-	osd->addLine("Saved SRAM");
-
-	return 1;
-
-}
 
 // note: guessSF is so we don't have to do a linear search through the SFORMAT array every time
 // in the (most common) case that we already know where the next entry is.
