@@ -23,7 +23,7 @@ using std::max;
 class MemSpan
 {
 public:
-	static const int MAXSIZE = 8;
+	static const int MAXSIZE = 17; //max size for textures: 1024*1024*2 bytes / 128*1024 banks + 1 for wraparound
 
 	MemSpan() 
 		: numItems(0), size(0)
@@ -127,6 +127,7 @@ static MemSpan MemSpan_TexMem(u32 ofs, u32 len)
 		currofs += curr.len;
 		u8* ptr = MMU.texInfo.textureSlotAddr[slot];
 		
+		//TODO - dont alert if the masterbrightnesses are max or min
 		if(ptr == MMU.blank_memory) {
 			PROGINFO("Tried to reference unmapped texture memory: slot %d\n",slot);
 		}
@@ -158,6 +159,7 @@ static MemSpan MemSpan_TexPalette(u32 ofs, u32 len)
 		currofs += curr.len;
 		u8* ptr = MMU.texInfo.texPalSlot[slot];
 		
+		//TODO - dont alert if the masterbrightnesses are max or min
 		if(ptr == MMU.blank_memory) {
 			PROGINFO("Tried to reference unmapped texture palette memory: 16k slot #%d\n",slot);
 		}
