@@ -1,10 +1,3 @@
-#ifdef WIN32
-#pragma comment(lib,"wxmsw28_core.lib")
-#pragma comment(lib,"wxbase28.lib")
-#else
-#define lstrlen(a) strlen((a))
-#endif
-
 #include "NDSSystem.h"
 #include "GPU_osd.h"
 #include <wx/wxprec.h>
@@ -19,8 +12,8 @@
 #include "OGLRender.h"
 #include "firmware.h"
 
-#ifdef WIN32
-#include "snddx.h"
+#ifndef WIN32
+#define lstrlen(a) strlen((a))
 #endif
 
 #include <wx/stdpaths.h>
@@ -28,6 +21,7 @@
 #include "LuaWindow.h"
 #include "PadSimple/GUI/ConfigDlg.h"
 #include "PadSimple/pluginspecs_pad.h"
+#include "wxdlg/wxcontrolsconfigdialog.h"
 
 #ifndef WX_PRECOMP
 #include <wx/wx.h>
@@ -44,14 +38,11 @@
 #define GAP_MAX	90
 static int nds_gap_size;
 static int nds_screen_rotation_angle;
+static bool Touch = false;
 
 SoundInterface_struct *SNDCoreList[] = {
         &SNDDummy,
-#ifdef WIN32
-//        &SNDDIRECTX,
-#else
         &SNDSDL,
-#endif
         NULL
 };
 
@@ -170,6 +161,68 @@ public:
 			NDS_LoadROM(dialog.GetPath().mb_str(), dialog.GetPath().mb_str());
 		}
 	}
+
+	//----------------------------------------------------------------------------
+	//   Touchscreen
+	//----------------------------------------------------------------------------
+
+	// De-transform coordinates.
+	// Returns true if the coordinates are within the touchscreen.
+	bool DetransformTouchCoords(int& X, int& Y)
+	{
+		int dtX, dtY;
+		
+		// TODO: descaling (when scaling is supported)
+
+		// De-rotate coordinates
+		switch (nds_screen_rotation_angle)
+		{
+		case 0: dtX = X; dtY = Y - 191 - nds_gap_size; break;
+		case 90: dtX = Y; dtY = 191 - X; break;
+		case 180: dtX = 255 - X; dtY = 191 - Y; break;
+		case 270: dtX = 255 - Y; dtY = X - 191 - nds_gap_size; break;
+		}
+
+		// Atleast one of the coordinates is out of range
+		if ((dtX < 0) || (dtX > 255) || (dtY < 0) || (dtY > 191))
+		{
+			X = wxClip(dtX, 0, 255);
+			Y = wxClip(dtY, 0, 191);
+			return false;
+		}
+		else
+		{
+			X = dtX;
+			Y = dtY;
+			return true;
+		}
+	}
+
+	void OnTouchEvent(wxMouseEvent& evt)
+	{
+		wxPoint pt = evt.GetPosition();
+		bool inside = DetransformTouchCoords(pt.x, pt.y);
+
+		if (evt.LeftDown() && inside)
+		{
+			Touch = true;
+			NDS_setTouchPos((u16)pt.x, (u16)pt.y);
+		}
+		else if(evt.LeftUp() && Touch)
+		{
+			Touch = false;
+			NDS_releaseTouch();
+		}
+		else if (Touch)
+		{
+			NDS_setTouchPos((u16)pt.x, (u16)pt.y);
+		}
+	}
+
+	//----------------------------------------------------------------------------
+	//   Video
+	//----------------------------------------------------------------------------
+
 	void gpu_screen_to_rgb(u8 *rgb1, u8 *rgb2)
 	{
 		u16 gpu_pixel;
@@ -253,14 +306,17 @@ loop:
 	void onIdle(wxIdleEvent &event){
 		Refresh(false);
 		event.RequestMore();
-		applyInput();
-		if(execute) { 
+		if (execute)
+		{
+			applyInput();
 			NDS_exec<false>();
 			SPU_Emulate_user();
-		};
-		osd->update();
-		DrawHUD();
-		osd->clear();
+			osd->update();
+			DrawHUD();
+			osd->clear();
+		}
+		else
+			wxMilliSleep(250);
 		// wxMicroSleep(16.7*1000);
 	}
 
@@ -391,9 +447,10 @@ loop:
 
 	void OnOpenControllerConfiguration(wxCommandEvent& WXUNUSED (event))
 	{
-#ifndef _MSC_VER
-		new PADConfigDialogSimple(this);
-#endif
+//#ifndef _MSC_VER
+	//	new PADConfigDialogSimple(this);
+		(new wxControlsConfigDialog(this))->ShowModal();
+//#endif
 	}
 
 	wxMenu* MakeStatesSubMenu( int baseid ) const
@@ -521,10 +578,16 @@ void DesmumeFrame::onResize(wxSizeEvent &event) {
 }
 
 BEGIN_EVENT_TABLE(DesmumeFrame, wxFrame)
+
 EVT_PAINT(DesmumeFrame::onPaint)
 EVT_IDLE(DesmumeFrame::onIdle)
 EVT_SIZE(DesmumeFrame::onResize)
+EVT_LEFT_DOWN(DesmumeFrame::OnTouchEvent)
+EVT_LEFT_UP(DesmumeFrame::OnTouchEvent)
+EVT_LEFT_DCLICK(DesmumeFrame::OnTouchEvent)
+EVT_MOTION(DesmumeFrame::OnTouchEvent)
 EVT_CLOSE(DesmumeFrame::OnClose)
+
 EVT_MENU(wxID_EXIT, DesmumeFrame::OnQuit)
 EVT_MENU(wxID_OPEN, DesmumeFrame::LoadRom)
 EVT_MENU(wxID_ABOUT,DesmumeFrame::OnAbout)
@@ -605,9 +668,7 @@ void DesmumeFrame::NDSInitialize() {
 #else
 	NDS_Init();
 #endif
-#ifndef WIN32
 	SPU_ChangeSoundCore(SNDCORE_SDL, 735 * 4);
-#endif
 	NDS_3D_ChangeCore(0);
 	NDS_CreateDummyFirmware( &fw_config);
 }
@@ -625,7 +686,6 @@ bool Desmume::OnInit()
 #endif
 
 	SetAppName(_T("desmume"));
-	//comment for devs: or you may use wxConfig instead of wxFileConfig, so it will be wxRegConfig on MSW and wxFileConfig on other platforms
 	wxConfigBase *pConfig = new wxFileConfig();
 	wxConfigBase::Set(pConfig);
 	wxString emu_version(EMU_DESMUME_NAME_AND_VERSION(), wxConvUTF8);
@@ -648,7 +708,6 @@ bool Desmume::OnInit()
 DesmumeFrame::DesmumeFrame(const wxString& title)
 : wxFrame(NULL, wxID_ANY, title)
 {
-
 	history = new wxFileHistory;
 	wxMenu *fileMenu = new wxMenu;
 	wxMenu *emulationMenu = new wxMenu;
@@ -696,27 +755,37 @@ DesmumeFrame::DesmumeFrame(const wxString& title)
 	}
 	viewMenu->AppendSubMenu(rotateMenu, _T("Rotate"));
 	viewMenu->AppendSeparator();
-	viewMenu->Append(wFrameCounter, _T("&Display Frame Counter"));
-	viewMenu->Append(wFPS, _T("&Display FPS"));
-	viewMenu->Append(wDisplayInput, _T("&Display Input"));
-	viewMenu->Append(wDisplayGraphicalInput, _T("&Display Graphical Input"));
-	viewMenu->Append(wDisplayLagCounter, _T("&Display Lag Counter"));
-	viewMenu->Append(wDisplayMicrophone, _T("&Display Microphone"));
+	viewMenu->AppendCheckItem(wFrameCounter, _T("&Display Frame Counter"));
+	viewMenu->AppendCheckItem(wFPS, _T("&Display FPS"));
+	viewMenu->AppendCheckItem(wDisplayInput, _T("&Display Input"));
+	viewMenu->AppendCheckItem(wDisplayGraphicalInput, _T("&Display Graphical Input"));
+	viewMenu->AppendCheckItem(wDisplayLagCounter, _T("&Display Lag Counter"));
+	viewMenu->AppendCheckItem(wDisplayMicrophone, _T("&Display Microphone"));
 
 	toolsMenu->Append(w3dView, _T("&3d Viewer"));
 	wxMenu *layersMenu = new wxMenu;
 	{
 		layersMenu->AppendCheckItem(wMainGPU, _T("Main GPU"));
-		layersMenu->Append(wMainBG0, _T("Main BG 0"));
-		layersMenu->Append(wMainBG1, _T("Main BG 1"));
-		layersMenu->Append(wMainBG2, _T("Main BG 2"));
-		layersMenu->Append(wMainBG3, _T("Main BG 3"));
+		layersMenu->Check(wMainGPU, true);
+		layersMenu->AppendCheckItem(wMainBG0, _T("Main BG 0"));
+		layersMenu->Check(wMainBG0, true);
+		layersMenu->AppendCheckItem(wMainBG1, _T("Main BG 1"));
+		layersMenu->Check(wMainBG1, true);
+		layersMenu->AppendCheckItem(wMainBG2, _T("Main BG 2"));
+		layersMenu->Check(wMainBG2, true);
+		layersMenu->AppendCheckItem(wMainBG3, _T("Main BG 3"));
+		layersMenu->Check(wMainBG3, true);
 		layersMenu->AppendSeparator();
-		layersMenu->Append(wSubGPU, _T("Sub GPU"));
-		layersMenu->Append(wSubBG0, _T("Sub BG 0"));
-		layersMenu->Append(wSubBG1, _T("Sub BG 1"));
-		layersMenu->Append(wSubBG2, _T("Sub BG 2"));
-		layersMenu->Append(wSubBG3, _T("Sub BG 3"));
+		layersMenu->AppendCheckItem(wSubGPU, _T("Sub GPU"));
+		layersMenu->Check(wSubGPU, true);
+		layersMenu->AppendCheckItem(wSubBG0, _T("Sub BG 0"));
+		layersMenu->Check(wSubBG0, true);
+		layersMenu->AppendCheckItem(wSubBG1, _T("Sub BG 1"));
+		layersMenu->Check(wSubBG1, true);
+		layersMenu->AppendCheckItem(wSubBG2, _T("Sub BG 2"));
+		layersMenu->Check(wSubBG2, true);
+		layersMenu->AppendCheckItem(wSubBG3, _T("Sub BG 3"));
+		layersMenu->Check(wSubBG3, true);
 	}
 
 	configMenu->Append(wConfigureControls, _T("Controls"));
@@ -785,6 +854,12 @@ void DesmumeFrame::OnClose(wxCloseEvent &event) {
 }
 
 void DesmumeFrame::OnOpenRecent(wxCommandEvent &event) {
-	execute = true;
-	NDS_LoadROM(history->GetHistoryFile(event.GetId()-wxID_FILE1).mb_str(),NULL);
+	int ret;
+	size_t id = event.GetId()-wxID_FILE1;
+
+	ret = NDS_LoadROM(history->GetHistoryFile(id).mb_str(), history->GetHistoryFile(id).mb_str());
+	if (ret > 0)
+		execute = true;
+	else
+		history->RemoveFileFromHistory(id);
 }
